/* By the power of binutils and the one true editor, here is a compacted
   form of the ll_init_COUNTRY settings from ltmodem.sys.

   Copyright (C) 1999 Jamie Lokier.

   These parameters are extracted from ltmodem.sys, by examining the
   disassembly of each ll_init_<country> function.  The script
   xcountry.pl was used, so the country_init_<country> arrays are reliable.

   country_init_base was extracted from x_init_international.  There was
   no ll_init_twn function: I created it to match the effect in
   x_init_international for Taiwan.

   The `countries' array was generated with lots of keyboard macros after
   reconstructing the code in ltmodem.sys's x_init_international.  I'm
   fairly sure it is correct.  There are some strange rules in that code.
   My version of x_init_international is quite different, but should
   have the same effect. */

#ifdef __STDC__
#define H(h,n) & H_##h - homol, n
#else
#define H(h,n) & H_/**/h - homol, n
#endif
#define X(c) c, sizeof (c)

typedef const u8_t Country_Init;

Country_Init country_init_base [] = {
  H( pulse_dial_make_time			,   36	),
  H( pulse_dial_break_time			,   64	),
  H( pulse_dial_digit_pattern			,    1	),
  H( DTMF_high_tone_level_ndB			,    6	),
  H( min_DTMF_dial_speed			,   50	),
  H( max_DTMF_dial_speed			,  150	),
  H( call_progress_detection_threshold_ndB	,   43	),
  H( dial_tone_detection_delay			,    0	),
  H( dial_tone_hole_time			,   50	),
  H( ring_count_reset_delay			,   60	),
  H( min_auto_answer_ring_count			,    1	),
  H( max_auto_answer_ring_count			,   15	),
  H( min_ring_signal_period			,   18	),
  H( max_ring_signal_period			,   80	),
  H( min_ring_cadence_on_time			,   14	),
  H( max_no_carrier_disconnect_time		,  255	),
  H( max_failed_attempts_anywhere		,  255	),
  H( min_ring_cadence_off_time			,   30	),
  H( min_busy_cadence_on_time			,    0	),
  H( max_busy_cadence_on_time			,    0	),
  H( busy_cycles_threshold			,    4	),
  H( min_busy_cadence_off_time			,    0	),
  H( max_busy_cadence_off_time			,    0	),
  H( detect_number_unavailable			,    0	),
  H( answer_tone_validation_time		,   10	),
  H( blacklist_or_delay_flag			,    0	),
  H( max_failure_count_before_blacklist		,    5	),
  H( troubled_call_delay			,    1	),
  H( ineffective_call_delay			,    1	),
  H( erroneous_call_delay			,    1	),
  H( blacklist_duration				,   12	),
  H( overcurrent_sense_validation_time		,    0	),
  H( calling_tone_flag				,    1	),
  H( blind_dial_flag				,    1	),
  H( busy_detect_disable			,    1	),
  H( guard_tone_type				,    0	),
  H( bell_modes_permitted			,    1	),
  H( off_hook_restrictions			,    0	),
  H( shunt_relay_on_time			,    5	),
  H( hook_flash_time				,   50	),
  H( min_blind_dial_pause			,    2	),
  H( max_blind_dial_pause			,   65	),
  H( min_no_answer_timeout			,   35	),
  H( max_no_answer_timeout			,   65	),
  H( min_dial_pause_time			,    2	),
  H( max_dial_pause_time			,   65	),
  H( min_no_carrier_disconnect_time		,    1	),
  H( transmit_level_ndB				,   10	),
  H( TAD_rx_gain				,    3	),
  H( full_wave_ring_detect			,    0	),
  H( max_failed_attempts_stage_1		,    1	),
  H( erronous_call_increment_count		,    1	),
  H( dial_tone_validation_time			,   25	),
  H( detect_busy_tone_during_dial_tone		,    0	),
  H( DTMF_high_low_tone_level_difference_dB	,    2	),
  H( not_local_phone_detection_in_speakerphone	,    0	),
  H( not_pulse_dial_flag			,    0	),
  H( dial_tone_or_call_progress_filter		,    1	),
  H( dial_tone_detection_threshold_dB		,   43	),
  H( disallow_ABCD_dialing			,    0	),
  H( comma_pause_duration_limit			,   30	),
  H( TAD_tx_level_dB				,   10	),
  H( data_or_fax_carrier_rx_threshold_dB	,   37	),
  H( data_or_fax_answer_detection_threshold_dB	,   43	),
  H( dialtone_level				,    0	),
  H( fax_rx_threshold_dB			,   43	),
  H( disallow_mixed_tone_and_pulse_dial		,    0	),
  H( ringer_impedance_relay_flag		,    0	),
  H( DC_loop_VI_characteristics_relay_flag	,    0	),
  H( DC_loop_limiting_relay_flag		,    0	),
  H( real_complex_impedance_setting		,    0	),
  H( mercury_cioca_MSB_rx_gain			,  136	),
  H( mercury_cioca_LSB_hybrid			,  128	),
  H( mercury_ciocb_MSB_tx_gain			,  107	),
  H( mercury_ciocb_LSB_BOM_control		,   30	),
  H( mercury_ciocb_MSB_tx_gain_in_TAD_mode	,   19	),
  H( mercury_ciocb_MSB_tx_gain_in_FDSP_mode	,  205	),
  H( dial_modifier_validation			,    0	),
  H( 0x7f					,    0	),
  H( 0xd6					,    0	)
};

Country_Init country_init_tbr21 [] = {
  H( max_failed_attempts_anywhere		,  255	),
  H( dial_tone_validation_time			,    5	),
  H( dial_tone_detection_threshold_dB		,   40	),
  H( dial_tone_hole_time			,   10	),
  H( dial_tone_or_call_progress_filter		,    5	),
  H( transmit_level_ndB				,   10	),
  H( min_DTMF_dial_speed			,   90	),
  H( max_DTMF_dial_speed			,   90	),
  H( min_blind_dial_pause			,    3	),
  H( max_blind_dial_pause			,    7	),
  H( min_dial_pause_time			,    2	),
  H( max_dial_pause_time			,   20	),
  H( busy_cycles_threshold			,    2	),
  H( call_progress_detection_threshold_ndB	,   38	),
  H( ring_count_reset_delay			,   80	),
  H( min_auto_answer_ring_count			,    2	),
  H( max_auto_answer_ring_count			,    6	),
  H( hook_flash_time				,   10	),
  H( answer_tone_validation_time		,   10	),
  H( blacklist_or_delay_flag			,    1	),
  H( max_failure_count_before_blacklist		,   15	),
  H( troubled_call_delay			,    0	),
  H( ineffective_call_delay			,    0	),
  H( erroneous_call_delay			,    0	),
  H( dial_modifier_validation			,    1	),
  H( real_complex_impedance_setting		,    2	),
  H( DC_loop_limiting_relay_flag		,    1	),
  H( DTMF_high_tone_level_ndB			,    2	),
  H( DTMF_high_low_tone_level_difference_dB	,    4	),
  H( shunt_relay_on_time			,  130	)
};

Country_Init country_init_aust [] = {
  H( pulse_dial_make_time			,   33	),
  H( pulse_dial_break_time			,   64	),
  H( DTMF_high_tone_level_ndB			,    8	),
  H( min_DTMF_dial_speed			,   80	),
  H( max_DTMF_dial_speed			,  150	),
  H( dial_tone_hole_time			,  100	),
  H( ring_count_reset_delay			,   80	),
  H( min_auto_answer_ring_count			,    2	),
  H( max_auto_answer_ring_count			,    6	),
  H( min_ring_signal_period			,   21	),
  H( max_ring_signal_period			,   92	),
  H( detect_number_unavailable			,    1	),
  H( blacklist_or_delay_flag			,    1	),
  H( max_failure_count_before_blacklist		,   10	),
  H( erroneous_call_delay			,  120	),
  H( shunt_relay_on_time			,   30	),
  H( hook_flash_time				,    9	),
  H( min_blind_dial_pause			,    2	),
  H( max_blind_dial_pause			,    5	),
  H( min_no_answer_timeout			,   30	),
  H( max_no_answer_timeout			,   60	),
  H( max_dial_pause_time			,    5	),
  H( transmit_level_ndB				,    9	),
  H( DC_loop_VI_characteristics_relay_flag	,    1	),
  H( real_complex_impedance_setting		,    2	)
};

Country_Init country_init_bel [] = {
  H( disallow_ABCD_dialing			,    0	),
  H( pulse_dial_make_time			,   33	),
  H( pulse_dial_break_time			,   67	),
  H( DTMF_high_tone_level_ndB			,    6	),
  H( min_DTMF_dial_speed			,   90	),
  H( max_DTMF_dial_speed			,   90	),
  H( call_progress_detection_threshold_ndB	,   37	),
  H( dial_tone_hole_time			,   30	),
  H( ring_count_reset_delay			,   80	),
  H( min_auto_answer_ring_count			,    2	),
  H( max_auto_answer_ring_count			,    5	),
  H( min_busy_cadence_on_time			,   40	),
  H( max_busy_cadence_on_time			,   60	),
  H( min_busy_cadence_off_time			,   40	),
  H( max_busy_cadence_off_time			,   60	),
  H( answer_tone_validation_time		,   15	),
  H( blacklist_or_delay_flag			,    1	),
  H( max_failure_count_before_blacklist		,    4	),
  H( blacklist_duration				,    6	),
  H( blind_dial_flag				,    1	),
  H( min_blind_dial_pause			,    4	),
  H( max_blind_dial_pause			,    4	),
  H( hook_flash_time				,   12	),
  H( min_dial_pause_time			,    4	),
  H( max_dial_pause_time			,    8	),
  H( transmit_level_ndB				,    9	),
  H( dial_tone_validation_time			,    8	),
  H( dial_tone_or_call_progress_filter		,    4	),
  H( dial_tone_detection_threshold_dB		,   24	),
  H( TAD_tx_level_dB				,    9	),
  H( dial_modifier_validation			,    1	),
  H( min_no_answer_timeout			,   30	),
  H( max_no_answer_timeout			,   60	)
};

Country_Init country_init_den [] = {
  H( blacklist_duration				,  255	),
  H( max_no_carrier_disconnect_time		,  255	),
  H( dial_tone_or_call_progress_filter		,    0	),
  H( ringer_impedance_relay_flag		,    0	),
  H( DTMF_high_tone_level_ndB			,    8	),
  H( min_DTMF_dial_speed			,   90	),
  H( max_DTMF_dial_speed			,   90	),
  H( call_progress_detection_threshold_ndB	,   47	),
  H( ring_count_reset_delay			,  100	),
  H( min_auto_answer_ring_count			,    1	),
  H( max_auto_answer_ring_count			,    4	),
  H( min_ring_cadence_on_time			,   15	),
  H( detect_number_unavailable			,    1	),
  H( answer_tone_validation_time		,   32	),
  H( blacklist_or_delay_flag			,    1	),
  H( max_failure_count_before_blacklist		,   10	),
  H( calling_tone_flag				,    2	),
  H( blind_dial_flag				,    1	),
  H( min_blind_dial_pause			,    4	),
  H( max_blind_dial_pause			,    4	),
  H( hook_flash_time				,    9	),
  H( min_no_answer_timeout			,   30	),
  H( max_no_answer_timeout			,   60	),
  H( min_dial_pause_time			,    2	),
  H( max_dial_pause_time			,   20	),
  H( transmit_level_ndB				,    9	),
  H( not_pulse_dial_flag			,    1	),
  H( dial_tone_detection_threshold_dB		,   38	),
  H( dial_modifier_validation			,    1	)
};

Country_Init country_init_fin [] = {
  H( ringer_impedance_relay_flag		,    0	),
  H( pulse_dial_make_time			,   40	),
  H( pulse_dial_break_time			,   60	),
  H( DTMF_high_tone_level_ndB			,    8	),
  H( min_DTMF_dial_speed			,   90	),
  H( max_DTMF_dial_speed			,   90	),
  H( dial_tone_hole_time			,   28	),
  H( ring_count_reset_delay			,  100	),
  H( min_auto_answer_ring_count			,    1	),
  H( max_auto_answer_ring_count			,    4	),
  H( min_busy_cadence_on_time			,   10	),
  H( max_busy_cadence_on_time			,   66	),
  H( min_busy_cadence_off_time			,   10	),
  H( max_busy_cadence_off_time			,   65	),
  H( answer_tone_validation_time		,   20	),
  H( blacklist_or_delay_flag			,    1	),
  H( max_failure_count_before_blacklist		,    4	),
  H( erroneous_call_delay			,   60	),
  H( blacklist_duration				,    6	),
  H( calling_tone_flag				,    2	),
  H( hook_flash_time				,    9	),
  H( min_blind_dial_pause			,    4	),
  H( max_blind_dial_pause			,    4	),
  H( min_dial_pause_time			,    4	),
  H( max_dial_pause_time			,    8	),
  H( transmit_level_ndB				,    9	),
  H( dial_modifier_validation			,    1	),
  H( min_no_answer_timeout			,   30	),
  H( max_no_answer_timeout			,   60	)
};

Country_Init country_init_fra [] = {
  H( blacklist_duration				,  255	),
  H( pulse_dial_make_time			,   34	),
  H( pulse_dial_break_time			,   67	),
  H( min_DTMF_dial_speed			,   90	),
  H( max_DTMF_dial_speed			,   90	),
  H( call_progress_detection_threshold_ndB	,   38	),
  H( dial_tone_detection_delay			,    5	),
  H( dial_tone_hole_time			,   40	),
  H( ring_count_reset_delay			,   70	),
  H( min_auto_answer_ring_count			,    2	),
  H( max_auto_answer_ring_count			,    3	),
  H( min_ring_signal_period			,   15	),
  H( max_ring_signal_period			,   66	),
  H( min_ring_cadence_on_time			,   24	),
  H( min_busy_cadence_on_time			,   40	),
  H( max_busy_cadence_on_time			,   63	),
  H( min_busy_cadence_off_time			,   37	),
  H( max_busy_cadence_off_time			,   63	),
  H( busy_cycles_threshold			,    2	),
  H( answer_tone_validation_time		,   45	),
  H( blacklist_or_delay_flag			,    1	),
  H( max_failure_count_before_blacklist		,    6	),
  H( blind_dial_flag				,    1	),
  H( guard_tone_type				,    2	),
  H( bell_modes_permitted			,    0	),
  H( off_hook_restrictions			,    0	),
  H( hook_flash_time				,   27	),
  H( min_blind_dial_pause			,    4	),
  H( max_blind_dial_pause			,    4	),
  H( min_no_answer_timeout			,   30	),
  H( max_no_answer_timeout			,   60	),
  H( min_dial_pause_time			,    2	),
  H( max_dial_pause_time			,    2	),
  H( transmit_level_ndB				,    9	),
  H( max_failed_attempts_stage_1		,    0	),
  H( dial_tone_validation_time			,   16	),
  H( dial_tone_detection_threshold_dB		,   36	),
  H( data_or_fax_carrier_rx_threshold_dB	,   32	),
  H( data_or_fax_answer_detection_threshold_dB	,   36	),
  H( dial_modifier_validation			,    1	),
  H( DC_loop_limiting_relay_flag		,    1	)
};

Country_Init country_init_ger [] = {
  H( off_hook_restrictions			,    0	),
  H( pulse_dial_make_time			,   41	),
  H( pulse_dial_break_time			,   60	),
  H( DTMF_high_tone_level_ndB			,   13	),
  H( min_DTMF_dial_speed			,   90	),
  H( max_DTMF_dial_speed			,   90	),
  H( min_auto_answer_ring_count			,    1	),
  H( max_auto_answer_ring_count			,    6	),
  H( min_ring_signal_period			,   20	),
  H( max_ring_signal_period			,   63	),
  H( min_busy_cadence_on_time			,    9	),
  H( max_busy_cadence_on_time			,   55	),
  H( min_busy_cadence_off_time			,   20	),
  H( max_busy_cadence_off_time			,   60	),
  H( answer_tone_validation_time		,   20	),
  H( blacklist_or_delay_flag			,    1	),
  H( max_failure_count_before_blacklist		,   12	),
  H( blacklist_duration				,    6	),
  H( calling_tone_flag				,    1	),
  H( hook_flash_time				,   18	),
  H( min_blind_dial_pause			,    4	),
  H( max_blind_dial_pause			,    4	),
  H( min_no_answer_timeout			,   30	),
  H( max_no_answer_timeout			,   60	),
  H( min_dial_pause_time			,    4	),
  H( max_dial_pause_time			,    6	),
  H( transmit_level_ndB				,   14	),
  H( max_failed_attempts_anywhere		,   12	),
  H( max_failed_attempts_stage_1		,    2	),
  H( dial_tone_validation_time			,   19	),
  H( dial_tone_or_call_progress_filter		,    5	),
  H( DTMF_high_low_tone_level_difference_dB	,    3	),
  H( TAD_tx_level_dB				,   12	),
  H( disallow_mixed_tone_and_pulse_dial		,    1	),
  H( dial_modifier_validation			,    1	),
  H( DC_loop_VI_characteristics_relay_flag	,    1	),
  H( real_complex_impedance_setting		,    2	)
};

Country_Init country_init_neth [] = {
  H( blacklist_or_delay_flag			,    0	),
  H( not_pulse_dial_flag			,    0	),
  H( pulse_dial_make_time			,   40	),
  H( pulse_dial_break_time			,   60	),
  H( DTMF_high_tone_level_ndB			,    8	),
  H( min_DTMF_dial_speed			,   90	),
  H( max_DTMF_dial_speed			,   90	),
  H( min_auto_answer_ring_count			,    2	),
  H( max_auto_answer_ring_count			,    4	),
  H( min_ring_signal_period			,   44	),
  H( min_ring_cadence_on_time			,   15	),
  H( min_busy_cadence_on_time			,   15	),
  H( max_busy_cadence_on_time			,   65	),
  H( min_busy_cadence_off_time			,   11	),
  H( max_busy_cadence_off_time			,   65	),
  H( call_progress_detection_threshold_ndB	,   34	),
  H( dial_tone_hole_time			,   30	),
  H( ring_count_reset_delay			,   80	),
  H( answer_tone_validation_time		,   20	),
  H( blacklist_duration				,    6	),
  H( calling_tone_flag				,    2	),
  H( blind_dial_flag				,    1	),
  H( min_blind_dial_pause			,    5	),
  H( max_blind_dial_pause			,    5	),
  H( guard_tone_type				,    2	),
  H( shunt_relay_on_time			,   27	),
  H( hook_flash_time				,   11	),
  H( min_no_answer_timeout			,   30	),
  H( max_no_answer_timeout			,   60	),
  H( max_dial_pause_time			,    2	),
  H( max_failed_attempts_anywhere		,   15	),
  H( dial_tone_validation_time			,   15	),
  H( dial_tone_or_call_progress_filter		,    5	),
  H( dial_tone_detection_threshold_dB		,   27	),
  H( dialtone_level				,    9	),
  H( disallow_mixed_tone_and_pulse_dial		,    1	),
  H( dial_modifier_validation			,    1	),
  H( transmit_level_ndB				,    9	)
};

Country_Init country_init_ita [] = {
  H( busy_detect_disable			,    0	),
  H( pulse_dial_make_time			,   41	),
  H( pulse_dial_break_time			,   65	),
  H( min_DTMF_dial_speed			,   90	),
  H( max_DTMF_dial_speed			,   90	),
  H( call_progress_detection_threshold_ndB	,   50	),
  H( dial_tone_hole_time			,   10	),
  H( ring_count_reset_delay			,   80	),
  H( min_auto_answer_ring_count			,    1	),
  H( max_auto_answer_ring_count			,    4	),
  H( busy_cycles_threshold			,    2	),
  H( answer_tone_validation_time		,   20	),
  H( blacklist_or_delay_flag			,    1	),
  H( max_failure_count_before_blacklist		,    4	),
  H( blacklist_duration				,    6	),
  H( blind_dial_flag				,    1	),
  H( min_blind_dial_pause			,    4	),
  H( guard_tone_type				,    2	),
  H( off_hook_restrictions			,    2	),
  H( hook_flash_time				,    9	),
  H( max_blind_dial_pause			,   10	),
  H( min_no_answer_timeout			,   30	),
  H( max_no_answer_timeout			,   60	),
  H( transmit_level_ndB				,    9	),
  H( dial_tone_validation_time			,   22	),
  H( dial_tone_or_call_progress_filter		,    4	),
  H( dial_tone_detection_threshold_dB		,   40	),
  H( dial_modifier_validation			,    1	),
  H( data_or_fax_carrier_rx_threshold_dB	,   36	)
};

Country_Init country_init_nz [] = {
  H( blacklist_or_delay_flag			,    0	),
  H( off_hook_restrictions			,    0	),
  H( pulse_dial_make_time			,   33	),
  H( pulse_dial_break_time			,   67	),
  H( pulse_dial_digit_pattern			,    3	),
  H( DTMF_high_tone_level_ndB			,    8	),
  H( min_DTMF_dial_speed			,   90	),
  H( max_DTMF_dial_speed			,   90	),
  H( dial_tone_hole_time			,   30	),
  H( ring_count_reset_delay			,   80	),
  H( min_auto_answer_ring_count			,    3	),
  H( max_auto_answer_ring_count			,    4	),
  H( detect_number_unavailable			,    1	),
  H( transmit_level_ndB				,    9	),
  H( hook_flash_time				,   71	),
  H( min_blind_dial_pause			,    4	),
  H( max_blind_dial_pause			,    4	),
  H( min_no_answer_timeout			,   30	),
  H( max_no_answer_timeout			,   60	),
  H( min_dial_pause_time			,    4	),
  H( max_dial_pause_time			,    8	),
  H( DC_loop_VI_characteristics_relay_flag	,    1	)
};

Country_Init country_init_nor [] = {
  H( DTMF_high_tone_level_ndB			,    8	),
  H( min_DTMF_dial_speed			,   90	),
  H( max_DTMF_dial_speed			,   90	),
  H( call_progress_detection_threshold_ndB	,   40	),
  H( dial_tone_hole_time			,   50	),
  H( ring_count_reset_delay			,  100	),
  H( min_auto_answer_ring_count			,    1	),
  H( max_auto_answer_ring_count			,    4	),
  H( min_busy_cadence_on_time			,   10	),
  H( max_busy_cadence_on_time			,   66	),
  H( busy_cycles_threshold			,    2	),
  H( min_busy_cadence_off_time			,   10	),
  H( max_busy_cadence_off_time			,   65	),
  H( detect_number_unavailable			,    1	),
  H( answer_tone_validation_time		,   32	),
  H( blacklist_or_delay_flag			,    1	),
  H( max_failure_count_before_blacklist		,   10	),
  H( blacklist_duration				,    3	),
  H( calling_tone_flag				,    2	),
  H( guard_tone_type				,    2	),
  H( off_hook_restrictions			,    0	),
  H( hook_flash_time				,   10	),
  H( min_blind_dial_pause			,    4	),
  H( max_blind_dial_pause			,    4	),
  H( min_no_answer_timeout			,   30	),
  H( max_no_answer_timeout			,   60	),
  H( min_dial_pause_time			,    4	),
  H( max_dial_pause_time			,    8	),
  H( transmit_level_ndB				,    9	),
  H( not_pulse_dial_flag			,    1	),
  H( dial_tone_validation_time			,   15	),
  H( dial_tone_or_call_progress_filter		,    0	),
  H( dial_tone_detection_threshold_dB		,   39	),
  H( dial_modifier_validation			,    1	),
  H( ringer_impedance_relay_flag		,    0	)
};

Country_Init country_init_spa [] = {
  H( blacklist_or_delay_flag			,    0	),
  H( off_hook_restrictions			,    0	),
  H( pulse_dial_make_time			,   33	),
  H( pulse_dial_break_time			,   67	),
  H( min_DTMF_dial_speed			,   90	),
  H( max_DTMF_dial_speed			,   90	),
  H( dial_tone_hole_time			,  120	),
  H( ring_count_reset_delay			,   80	),
  H( min_auto_answer_ring_count			,    1	),
  H( max_auto_answer_ring_count			,    4	),
  H( min_ring_cadence_on_time			,   75	),
  H( answer_tone_validation_time		,   20	),
  H( max_failure_count_before_blacklist		,    4	),
  H( blacklist_duration				,    6	),
  H( calling_tone_flag				,    2	),
  H( guard_tone_type				,    2	),
  H( hook_flash_time				,   10	),
  H( min_blind_dial_pause			,    4	),
  H( max_blind_dial_pause			,    4	),
  H( min_no_answer_timeout			,   30	),
  H( max_no_answer_timeout			,   60	),
  H( min_dial_pause_time			,    5	),
  H( max_dial_pause_time			,    5	),
  H( transmit_level_ndB				,    9	),
  H( dial_tone_validation_time			,   12	),
  H( dial_tone_or_call_progress_filter		,    5	),
  H( dial_tone_detection_threshold_dB		,   41	),
  H( dial_modifier_validation			,    1	)
};

Country_Init country_init_swe [] = {
  H( detect_number_unavailable			,    0	),
  H( max_failed_attempts_stage_1		,    0	),
  H( pulse_dial_make_time			,   40	),
  H( pulse_dial_break_time			,   60	),
  H( pulse_dial_digit_pattern			,    2	),
  H( DTMF_high_tone_level_ndB			,    8	),
  H( min_DTMF_dial_speed			,   90	),
  H( max_DTMF_dial_speed			,   90	),
  H( min_auto_answer_ring_count			,    1	),
  H( max_auto_answer_ring_count			,    2	),
  H( call_progress_detection_threshold_ndB	,   50	),
  H( dial_tone_hole_time			,   50	),
  H( ring_count_reset_delay			,  100	),
  H( min_ring_signal_period			,   23	),
  H( max_ring_signal_period			,   60	),
  H( min_busy_cadence_on_time			,   20	),
  H( max_busy_cadence_on_time			,   30	),
  H( min_busy_cadence_off_time			,   20	),
  H( max_busy_cadence_off_time			,   85	),
  H( answer_tone_validation_time		,   12	),
  H( blacklist_or_delay_flag			,    1	),
  H( max_failure_count_before_blacklist		,   10	),
  H( blacklist_duration				,    6	),
  H( blind_dial_flag				,    1	),
  H( calling_tone_flag				,    2	),
  H( guard_tone_type				,    2	),
  H( hook_flash_time				,    9	),
  H( min_blind_dial_pause			,    4	),
  H( max_blind_dial_pause			,    4	),
  H( min_no_answer_timeout			,   30	),
  H( max_no_answer_timeout			,   60	),
  H( min_dial_pause_time			,    4	),
  H( max_dial_pause_time			,    8	),
  H( transmit_level_ndB				,    9	),
  H( max_failed_attempts_anywhere		,   10	),
  H( dial_tone_validation_time			,   15	),
  H( DTMF_high_low_tone_level_difference_dB	,    3	),
  H( not_local_phone_detection_in_speakerphone	,    1	),
  H( not_pulse_dial_flag			,    1	),
  H( dial_tone_or_call_progress_filter		,    3	),
  H( dial_tone_detection_threshold_dB		,   36	),
  H( TAD_tx_level_dB				,   14	),
  H( dial_modifier_validation			,    1	),
  H( DC_loop_VI_characteristics_relay_flag	,    1	)
};

Country_Init country_init_swi [] = {
  H( pulse_dial_make_time			,   40	),
  H( pulse_dial_break_time			,   60	),
  H( min_DTMF_dial_speed			,   90	),
  H( max_DTMF_dial_speed			,   90	),
  H( call_progress_detection_threshold_ndB	,   37	),
  H( dial_tone_hole_time			,   30	),
  H( ring_count_reset_delay			,   80	),
  H( min_auto_answer_ring_count			,    2	),
  H( max_auto_answer_ring_count			,    4	),
  H( max_ring_signal_period			,   60	),
  H( min_busy_cadence_on_time			,    8	),
  H( max_busy_cadence_on_time			,   80	),
  H( busy_cycles_threshold			,    2	),
  H( min_busy_cadence_off_time			,    8	),
  H( max_busy_cadence_off_time			,   80	),
  H( answer_tone_validation_time		,   12	),
  H( blacklist_or_delay_flag			,    1	),
  H( max_failure_count_before_blacklist		,    5	),
  H( blacklist_duration				,   36	),
  H( guard_tone_type				,    2	),
  H( off_hook_restrictions			,    0	),
  H( hook_flash_time				,   12	),
  H( min_blind_dial_pause			,    4	),
  H( max_blind_dial_pause			,    4	),
  H( min_no_answer_timeout			,   30	),
  H( max_no_answer_timeout			,   60	),
  H( min_dial_pause_time			,    4	),
  H( max_dial_pause_time			,    8	),
  H( max_failed_attempts_anywhere		,   24	),
  H( erronous_call_increment_count		,    6	),
  H( dial_tone_validation_time			,   10	),
  H( detect_busy_tone_during_dial_tone		,    1	),
  H( dial_tone_or_call_progress_filter		,    0	),
  H( dial_tone_detection_threshold_dB		,   32	),
  H( TAD_tx_level_dB				,    7	),
  H( dial_modifier_validation			,    1	),
  H( transmit_level_ndB				,    9	),
  H( not_pulse_dial_flag			,    0	),
  H( DC_loop_VI_characteristics_relay_flag	,    1	),
  H( real_complex_impedance_setting		,    2	)
};

Country_Init country_init_uk [] = {
  H( blacklist_or_delay_flag			,    0	),
  H( off_hook_restrictions			,    0	),
  H( pulse_dial_make_time			,   34	),
  H( pulse_dial_break_time			,   68	),
  H( DTMF_high_tone_level_ndB			,    6	),
  H( min_DTMF_dial_speed			,   90	),
  H( max_DTMF_dial_speed			,   90	),
  H( min_auto_answer_ring_count			,    1	),
  H( max_auto_answer_ring_count			,    4	),
  H( detect_number_unavailable			,    1	),
  H( guard_tone_type				,    2	),
  H( hook_flash_time				,    7	),
  H( min_blind_dial_pause			,    4	),
  H( max_blind_dial_pause			,    4	),
  H( min_no_answer_timeout			,   30	),
  H( max_no_answer_timeout			,   60	),
  H( min_dial_pause_time			,    4	),
  H( max_dial_pause_time			,    8	),
  H( transmit_level_ndB				,    9	),
  H( DTMF_high_low_tone_level_difference_dB	,    3	),
  H( dial_tone_or_call_progress_filter		,    3	),
  H( DC_loop_VI_characteristics_relay_flag	,    1	)
};

Country_Init country_init_aus [] = {
  H( blacklist_or_delay_flag			,    0	),
  H( off_hook_restrictions			,    0	),
  H( pulse_dial_make_time			,   40	),
  H( pulse_dial_break_time			,   60	),
  H( min_DTMF_dial_speed			,   90	),
  H( max_DTMF_dial_speed			,   90	),
  H( call_progress_detection_threshold_ndB	,   45	),
  H( dial_tone_hole_time			,  100	),
  H( ring_count_reset_delay			,   80	),
  H( min_auto_answer_ring_count			,    1	),
  H( max_auto_answer_ring_count			,    4	),
  H( busy_cycles_threshold			,    2	),
  H( answer_tone_validation_time		,   20	),
  H( max_failure_count_before_blacklist		,    6	),
  H( calling_tone_flag				,    2	),
  H( hook_flash_time				,    9	),
  H( min_blind_dial_pause			,    4	),
  H( max_blind_dial_pause			,    4	),
  H( min_no_answer_timeout			,   30	),
  H( max_no_answer_timeout			,   60	),
  H( min_dial_pause_time			,    4	),
  H( max_dial_pause_time			,    6	),
  H( transmit_level_ndB				,    9	),
  H( max_failed_attempts_anywhere		,    6	),
  H( dial_tone_validation_time			,    8	),
  H( dial_tone_or_call_progress_filter		,    6	),
  H( dial_tone_detection_threshold_dB		,   45	),
  H( dial_modifier_validation			,    1	)
};

Country_Init country_init_jpn [] = {
  H( max_blind_dial_pause			,  255	),
  H( pulse_dial_make_time			,   34	),
  H( pulse_dial_break_time			,   66	),
  H( DTMF_high_tone_level_ndB			,    7	),
  H( min_DTMF_dial_speed			,   70	),
  H( dial_tone_hole_time			,  250	),
  H( ring_count_reset_delay			,   80	),
  H( min_ring_cadence_off_time			,   40	),
  H( answer_tone_validation_time		,   50	),
  H( blacklist_or_delay_flag			,    1	),
  H( blacklist_duration				,    3	),
  H( max_failure_count_before_blacklist		,    3	),
  H( troubled_call_delay			,    0	),
  H( erroneous_call_delay			,    0	),
  H( hook_flash_time				,   50	),
  H( min_blind_dial_pause			,    4	),
  H( min_no_answer_timeout			,   35	),
  H( max_no_answer_timeout			,   59	),
  H( transmit_level_ndB				,   14	),
  H( max_failed_attempts_stage_1		,    0	),
  H( TAD_tx_level_dB				,   13	),
  H( 0x7f					,    1	)
};

Country_Init country_init_kor [] = {
  H( pulse_dial_make_time			,   32	),
  H( pulse_dial_break_time			,   64	),
  H( DTMF_high_tone_level_ndB			,    6	),
  H( min_blind_dial_pause			,    4	),
  H( transmit_level_ndB				,   11	)
};

Country_Init country_init_mal [] = {
  H( DTMF_high_tone_level_ndB			,    8	)
};

Country_Init country_init_sng [] = {
  H( pulse_dial_make_time			,   32	),
  H( pulse_dial_break_time			,   64	),
  H( DTMF_high_tone_level_ndB			,    7	),
  H( min_DTMF_dial_speed			,   70	),
  H( dial_tone_hole_time			,   30	),
  H( ring_count_reset_delay			,   80	),
  H( detect_number_unavailable			,    1	),
  H( blacklist_or_delay_flag			,    1	),
  H( max_failure_count_before_blacklist		,   10	),
  H( blacklist_duration				,    0	),
  H( calling_tone_flag				,    0	),
  H( hook_flash_time				,    9	),
  H( min_blind_dial_pause			,    3	),
  H( max_blind_dial_pause			,   20	),
  H( min_no_answer_timeout			,    5	),
  H( max_no_answer_timeout			,   35	),
  H( transmit_level_ndB				,   11	),
  H( max_failed_attempts_stage_1		,    0	)
};

Country_Init country_init_twn [] = {
  H( min_blind_dial_pause			,    3  )
};

Country_Init country_init_tha [] = {
  H( blacklist_or_delay_flag			,    0	),
  H( calling_tone_flag				,    0	),
  H( pulse_dial_make_time			,   40	),
  H( pulse_dial_break_time			,   60	),
  H( call_progress_detection_threshold_ndB	,   35	),
  H( dial_tone_hole_time			,   30	),
  H( ring_count_reset_delay			,   80	),
  H( answer_tone_validation_time		,    5	),
  H( hook_flash_time				,    9	),
  H( max_blind_dial_pause			,   10	),
  H( max_dial_pause_time			,   10	),
  H( dial_tone_detection_threshold_dB		,   35	)
};

Country_Init country_init_por [] = {
  H( blacklist_or_delay_flag			,    0	),
  H( max_failed_attempts_stage_1		,    0	),
  H( pulse_dial_make_time			,   33	),
  H( pulse_dial_break_time			,   67	),
  H( min_DTMF_dial_speed			,   90	),
  H( max_DTMF_dial_speed			,   90	),
  H( dial_tone_hole_time			,  250	),
  H( min_auto_answer_ring_count			,    1	),
  H( max_auto_answer_ring_count			,    4	),
  H( max_ring_signal_period			,   86	),
  H( min_ring_cadence_on_time			,   24	),
  H( answer_tone_validation_time		,   30	),
  H( blacklist_duration				,    6	),
  H( hook_flash_time				,   20	),
  H( min_blind_dial_pause			,    4	),
  H( max_blind_dial_pause			,    4	),
  H( max_dial_pause_time			,    5	),
  H( transmit_level_ndB				,    9	),
  H( dial_tone_or_call_progress_filter		,    3	),
  H( dial_tone_detection_threshold_dB		,   35	),
  H( disallow_ABCD_dialing			,    1	),
  H( dial_modifier_validation			,    1	),
  H( min_no_answer_timeout			,   30	),
  H( max_no_answer_timeout			,   60	)
};

Country_Init country_init_na [] = {
  H( dial_tone_hole_time			,  255	),
  H( max_auto_answer_ring_count			,  255	),
  H( max_no_answer_timeout			,  255	),
  H( comma_pause_duration_limit			,  255	),
  H( ringer_impedance_relay_flag		,    0	),
  H( pulse_dial_make_time			,   39	),
  H( pulse_dial_break_time			,   61	),
  H( min_busy_cadence_on_time			,   18	),
  H( max_busy_cadence_on_time			,   60	),
  H( min_busy_cadence_off_time			,   18	),
  H( max_busy_cadence_off_time			,   60	),
  H( answer_tone_validation_time		,   10	),
  H( min_no_answer_timeout			,    1	),
  H( detect_busy_tone_during_dial_tone		,    1	),
  H( dial_tone_or_call_progress_filter		,    5	),
  H( TAD_tx_level_dB				,   10	)
};

Country_Init country_init_ire [] = {
  H( not_pulse_dial_flag			,    0	),
  H( min_ring_cadence_on_time			,   15	),
  H( pulse_dial_break_time			,   67	),
  H( max_failure_count_before_blacklist		,    4	),
  H( blacklist_duration				,    6	),
  H( min_no_answer_timeout			,   10	),
  H( max_no_answer_timeout			,   39	),
  H( answer_tone_validation_time		,    9	),
  H( dial_tone_or_call_progress_filter		,    2	),
  H( data_or_fax_carrier_rx_threshold_dB	,   33	)
};

Country_Init country_init_hun [] = {
  H( real_complex_impedance_setting		,    0	),
  H( pulse_dial_make_time			,   33	),
  H( pulse_dial_break_time			,   66	),
  H( dial_tone_validation_time			,    8	),
  H( dial_tone_detection_threshold_dB		,   45	),
  H( dial_tone_hole_time			,   30	),
  H( dial_tone_or_call_progress_filter		,    6	),
  H( DTMF_high_tone_level_ndB			,    7	),
  H( DTMF_high_low_tone_level_difference_dB	,    3	),
  H( transmit_level_ndB				,   10	),
  H( min_DTMF_dial_speed			,   60	),
  H( max_DTMF_dial_speed			,   95	),
  H( min_blind_dial_pause			,    2	),
  H( max_blind_dial_pause			,    8	),
  H( min_dial_pause_time			,    1	),
  H( max_dial_pause_time			,   20	),
  H( busy_cycles_threshold			,    2	),
  H( call_progress_detection_threshold_ndB	,   45	),
  H( ring_count_reset_delay			,   80	),
  H( min_auto_answer_ring_count			,    2	),
  H( max_auto_answer_ring_count			,    6	),
  H( hook_flash_time				,   10	),
  H( blacklist_or_delay_flag			,    1	),
  H( max_failed_attempts_anywhere		,    8	),
  H( dial_modifier_validation			,    1	),
  H( max_failure_count_before_blacklist		,    8	),
  H( troubled_call_delay			,    1	),
  H( ineffective_call_delay			,    1	),
  H( erroneous_call_delay			,    1	),
  H( blacklist_duration				,  240	)
};

Country_Init country_init_sa [] = {
  H( pulse_dial_make_time			,   33	),
  H( pulse_dial_break_time			,   67	),
  H( DTMF_high_tone_level_ndB			,    5	),
  H( transmit_level_ndB				,   10	),
  H( DC_loop_VI_characteristics_relay_flag	,    1	),
  H( real_complex_impedance_setting		,    2	)
};

Country_Init country_init_pol [] = {
  H( blacklist_or_delay_flag			,    0	),
  H( real_complex_impedance_setting		,    0	),
  H( pulse_dial_make_time			,   33	),
  H( pulse_dial_break_time			,   67	),
  H( dial_tone_validation_time			,    8	),
  H( dial_tone_detection_threshold_dB		,   45	),
  H( dial_tone_hole_time			,   30	),
  H( dial_tone_or_call_progress_filter		,    6	),
  H( DTMF_high_tone_level_ndB			,    6	),
  H( DTMF_high_low_tone_level_difference_dB	,    2	),
  H( transmit_level_ndB				,   10	),
  H( min_DTMF_dial_speed			,   95	),
  H( max_DTMF_dial_speed			,   95	),
  H( min_blind_dial_pause			,    6	),
  H( max_blind_dial_pause			,    6	),
  H( min_dial_pause_time			,    6	),
  H( max_dial_pause_time			,   20	),
  H( busy_cycles_threshold			,    2	),
  H( call_progress_detection_threshold_ndB	,   45	),
  H( ring_count_reset_delay			,   80	),
  H( min_auto_answer_ring_count			,    2	),
  H( max_auto_answer_ring_count			,    6	),
  H( hook_flash_time				,   10	)
};

Country_Init country_init_czech [] = {
  H( pulse_dial_make_time			,   40	),
  H( pulse_dial_break_time			,   60	),
  H( dial_tone_validation_time			,    8	),
  H( dial_tone_detection_threshold_dB		,   35	),
  H( dial_tone_hole_time			,   30	),
  H( dial_tone_or_call_progress_filter		,    6	),
  H( DTMF_high_tone_level_ndB			,    6	),
  H( DTMF_high_low_tone_level_difference_dB	,    2	),
  H( transmit_level_ndB				,   10	),
  H( min_DTMF_dial_speed			,   95	),
  H( max_DTMF_dial_speed			,   95	),
  H( min_blind_dial_pause			,    2	),
  H( max_blind_dial_pause			,   20	),
  H( min_dial_pause_time			,    2	),
  H( max_dial_pause_time			,   20	),
  H( busy_cycles_threshold			,    2	),
  H( call_progress_detection_threshold_ndB	,   35	),
  H( ring_count_reset_delay			,   80	),
  H( min_auto_answer_ring_count			,    2	),
  H( max_auto_answer_ring_count			,    6	),
  H( hook_flash_time				,   10	),
  H( blacklist_or_delay_flag			,    1	),
  H( max_failed_attempts_anywhere		,   12	),
  H( dial_modifier_validation			,    1	),
  H( max_failure_count_before_blacklist		,   12	),
  H( troubled_call_delay			,    1	),
  H( ineffective_call_delay			,    1	),
  H( erroneous_call_delay			,    1	),
  H( blacklist_duration				,  240	),
  H( real_complex_impedance_setting		,    2	)
};

typedef const struct
{
  char special_flag;		/* See x_init_international */
  const char * name;
  Country_Init * sequence;
  int sequence_length;
  Country_Init * inherited;
  int inherited_length;
} Country;

Country countries [] =
{
#define COUNTRY_AUSTRALIA	1
  /* 01 */ { 0, "Australia",	  X(country_init_aust)			   },
#define COUNTRY_BELGIUM		2
  /* 02 */ { 0, "Belgium",	  X(country_init_bel)			   },
#define COUNTRY_DENMARK		3
  /* 03 */ { 1, "Denmark",	  X(country_init_den)			   },
#define COUNTRY_FINLAND		4
  /* 04 */ { 0, "Finland",	  X(country_init_fin)			   },
#define COUNTRY_FRANCE		5
  /* 05 */ { 0, "France",	  X(country_init_fra)			   },
#define COUNTRY_GERMANY		6
  /* 06 */ { 0, "Germany",	  X(country_init_ger)			   },
#define COUNTRY_NETHERLANDS	7
  /* 07 */ { 1, "Netherlands",	  X(country_init_neth)			   },
#define COUNTRY_ITALY		8
  /* 08 */ { 0, "Italy",	  X(country_init_ita)			   },
#define COUNTRY_NEW_ZEALAND	9
  /* 09 */ { 0, "New Zealand",	  X(country_init_nz)			   },
#define COUNTRY_NORWAY		10
  /* 10 */ { 0, "Norway",	  X(country_init_nor)			   },
#define COUNTRY_SPAIN		11
  /* 11 */ { 0, "Spain",	  X(country_init_spa)			   },
#define COUNTRY_SWEDEN		12
  /* 12 */ { 1, "Sweden",	  X(country_init_swe)			   },
#define COUNTRY_SWITZERLAND	13
  /* 13 */ { 2, "Switzerland",	  X(country_init_swi)			   },
#define COUNTRY_UNITED_KINGDOM	14
  /* 14 */ { 0, "United Kingdom", X(country_init_uk)			   },
#define COUNTRY_AUSTRIA		15
  /* 15 */ { 0, "Austria",	  X(country_init_aus)			   },
#define COUNTRY_JAPAN		16
  /* 16 */ { 0, "Japan",	  X(country_init_jpn)			   },
#define COUNTRY_PRC		17
  /* 17 */ { 0, "PRC"							   },
#define COUNTRY_KOREA		18
  /* 18 */ { 0, "Korea",	  X(country_init_kor)			   },
#define COUNTRY_MALAYSIA	19
  /* 19 */ { 0, "Malaysia",	  X(country_init_mal), X(country_init_sng) },
#define COUNTRY_SINGAPORE	20
  /* 20 */ { 0, "Singapore",	  X(country_init_sng)			   },
#define COUNTRY_TAIWAN		21
  /* 21 */ { 0, "Taiwan",	  X(country_init_twn)			   },
#define COUNTRY_THAILAND	22
  /* 22 */ { 0, "Thailand",	  X(country_init_tha)			   },
#define COUNTRY_INDONESIA	23
  /* 23 */ { 0, "Indonesia"						   },
#define COUNTRY_PORTUGAL	24
  /* 24 */ { 0, "Portugal",	  X(country_init_por)			   },
#define COUNTRY_NORTH_AMERICA	25
  /* 25 */ { 0, "North America",  X(country_init_na)			   },
#define COUNTRY_IRELAND		26
  /* 26 */ { 0, "Ireland",	  X(country_init_ire), X(country_init_uk)  },
#define COUNTRY_HONG KONG	27
  /* 27 */ { 0, "Hong Kong",	  			  		   },
#define COUNTRY_CANADA		28
  /* 28 */ { 0, "Canada",	  		       X(country_init_na)  },
#define COUNTRY_MEXICO		29
  /* 29 */ { 0, "Mexico",	  			  		   },
#define COUNTRY_INDIA		30
  /* 30 */ { 0, "India",	  			  		   },
#define COUNTRY_VIETNAM		31
  /* 31 */ { 0, "Vietnam",	  			  		   },
#define COUNTRY_PHILIPPINES	32
  /* 32 */ { 0, "Philippines",	  			  		   },
#define COUNTRY_GREECE		33
  /* 33 */ { 0, "Greece",	  			  		   },
#define COUNTRY_HUNGARY		34
  /* 34 */ { 0, "Hungary",	  X(country_init_hun)			   },
#define COUNTRY_TURKEY		35
  /* 35 */ { 0, "Turkey",	  			  		   },
#define COUNTRY_SOUTH AFRICA	36
  /* 36 */ { 0, "South Africa",	  X(country_init_sa)			   },
#define COUNTRY_POLAND		37
  /* 37 */ { 0, "Poland",	  X(country_init_pol)			   },
  /* FIXME: ltmodem.sys contains the string "Slovania".  Is it a typo? */
#define COUNTRY_SLOVENIA	38
  /* 38 */ { 0, "Slovenia",	  		  			   },
#define COUNTRY_SLOVAKIA	39
  /* 39 */ { 0, "Slovakia",	  		  			   },
#define COUNTRY_CZECH REPUBLIC	40
  /* 40 */ { 0, "Czech Republic", X(country_init_czech)			   },
#define COUNTRY_LUXEMBOURG	41
  /* 41 */ { 0, "Luxembourg"						   },
#define COUNTRY_EUROPE		42
  /* 42 */ { 0, "Europe"						   },
#define COUNTRY_BRAZIL		43
  /* 43 */ { 0, "Brazil",	  			  		   },
#define COUNTRY_BULGARIA	44
  /* 44 */ { 0, "Bulgaria",	  			  		   },
#define COUNTRY_CYPRUS		45
  /* 45 */ { 0, "Cyprus",	  			  		   },
#define COUNTRY_ICELAND		46
  /* 46 */ { 0, "Iceland",	  			  		   },
#define COUNTRY_RUSSIA		47
  /* 47 */ { 0, "Russia",	  			  		   },
};

#define nr_countries (sizeof (countries) / sizeof (countries [0]))

#undef H
#undef X
