/* Define this to select your country */
#ifndef COUNTRY
#define COUNTRY "Czech Republic"
#endif
#define SLOW_TIMER 0

#include <asm/io.h>
#include <asm/system.h>
#include <sys/time.h>
#include <stdio.h>
#include <signal.h>
#include "dsp.c"
#include "ltmodem.h"
#include <unistd.h>
#include <fcntl.h>

#ifndef wmb
#define wmb mb
#endif
#ifndef set_wmb
#define set_wmb(var, value) do { (var) = (value); wmb (); } while (0)
#endif

/* Used by IBM Blacktip */
#ifndef CMOS_READ
#define CMOS_READ(addr) ({ outb_p((addr), 0x70); inb_p(0x71); })
#endif

typedef unsigned char  u8_t;
typedef unsigned short u16_t;
typedef unsigned int   u32_t;

u8_t _S [0xd7];
#define homol (_S + 0x86)

/* Offsets 0x86 to 0xd6 are actually 0-80 in the homol array.
   Lower offsets are in the S array.
   homol [0] is the current country code.
   homol [80] seems to be set for European countries (look for tbr21). */
#define S(x) (*((x) < 0x86 ? _S + (x) : homol + (x) - 0x86))

#include "homol.h"
#include "countries.h"

/* {{{ Variables */

int x_modem_state;
bool x_modem_originate;
u8_t dp_bamil_rd7, dp_bamil_rbc;
u8_t dp_ring_int, dp_ring_int_count;
volatile u8_t dp_byte_c, dp_byte_d, dp_byte_e, dp_byte_f;
u8_t ring_int, ring_int_count;
u16_t S_0x77;
u16_t S_0x81;
u8_t  S_0xd6;
u8_t  S_0x75;
u8_t  S_0x7e;

char dp_dsp_data_in_progress;
bool x_dsp_mars;
u8_t dp_readram_failcnt;
bool dp_sleep;

bool ToshibaFlag;
bool IBMBlacktip1Flag;
bool CpqFlag;
bool dp_tad_downloaded;
bool dp_fax_downloaded;
bool dp_first_call;
bool D3Flag;
bool V34Mode;
u16_t dp_version;
u8_t x_line_rate, x_line_rate_tx;

unsigned short BaseAddress;	/* Base I/O port address for DSP. */
unsigned short ComAddress;

u8_t x_chip_version;
u8_t Irq, V16550_IRQ_Number;

bool cell_active;
bool x_status;
int dp_dialing_status = 0;
u32_t x_modem_mode;

const char * country_name;

int dp_local_phone_state, dp_local_phone_status;
unsigned dp_local_phone_timer;
u16_t dp_threshold;

/* Over current protection */
unsigned dp_ocp_timer;

/* Dial tone detection */
int dp_tone_state;
int dp_dialtone_cycles;
unsigned dp_dial_tone_timer, dp_tone_timer, dp_tone_on_time, dp_tone_off_time;
u16_t peak_energy;

/* Answer tone detection */
int dp_answer_state;
unsigned dp_answer_on_timer, dp_answer_off_timer;

/* Busy tone detection */
int dp_busy_state;
unsigned dp_busy_tone_timer, dp_busy_on_cadence, dp_busy_off_cadence;
unsigned dp_continuous_busy_timer;
u16_t peak_busy_energy;

/* Quiet answer detection */
unsigned dp_quiet_answer_timer;

/* Bong tone detection */
unsigned dp_bong_tone1_on_time;

/* {{{ Dialing variables */

enum {
  DIAL_INIT,
  DIAL_1,
  DIAL_WAITING_FOR_DIALTONE,
  DIAL_WAITING_FOR_W_DIALTONE,
  DIAL_HOOK_FLASH,
  DIAL_HOOK_FLASH_RECOVERY,
  DIAL_CHECKING_QUIET_ANSWER,
  DIAL_WAITING_FOR_BONG,
  DIAL_DIGIT,
  DIAL_PAUSE,
  DIAL_DIGIT_RECOVERY
} dp_dial_state;

int dp_dial_status;
u8_t dp_busy_cycles;
u16_t dp_max_pause;

unsigned dp_timer1;
unsigned dp_dial_timer;
unsigned dp_long_timer;
unsigned dp_dial_pause;
bool dp_redial;
char * dp_dial_ptr;
bool dp_calling_tone_disable;

/* }}} */

int dp_state, x_dial_residual, x_modem_state, x_modem_status;
u16_t dp_wDspRetrainState;
u8_t DpIdleFillChar;
bool dp_fdsp_record;

/* }}} */

/* DUMMY FUNCTIONS */
void at_init_diagnostics (void) {}
void cell_init (void) {}
void cell_nec_rx (void) {}
void cell_onhook (void) {}
void cell_isr (void) {}
void cell_send () {}
bool cell_originate () { return false; }
bool cell_state_online () { return false; }
void cell_dial (int digit) {}

void dp_update_rate (void) {}
void ll_save_diagnostics (void) {}
void v8bis_app_reset (void) {}
void dp_init_blacklist (void) {}
void v8bis_power_up_init (void) {}
void x_wakeup (void) {}
void io_sync_power_up_init (void) {}
void x_send_mail (int a, int b, int c, int d) {}
void io_dual_port_rx () {}
void dual_port_rx () {}
void dual_port_tx () {}
void load_number (int digit, char * digits) {}
void dp_update_blacklist (char * digits, bool whether) {}
bool dp_check_list (char * digits) { return false; }
bool dp_stored_number (char * digits) { return true; }
bool dp_test_dialstring (char * digits) { return true; }

unsigned dp_cmd_timer, dp_timer;
#define START_TIMER(timer)   ((timer) = x_current_time ())
#define ELAPSED_TIME(timer) (x_elapsed_time (timer))

u16_t S_0x4f, S_0x4b, S_0x5d, S_0x61, S_0x64;
u8_t ETC_mode, x_test_mode_on;
char dp_dial_string [1000];

/* {{{ read_eeprom */

int
read_eeprom (int addr)
{
  /* Fake. */
  int i;
  if (addr == 0)
    for (i = 0; i < nr_countries; i++)
      if (!strcmp (COUNTRY, countries [i].name))
	return i + 1;
  return 0;
}

/* }}} */
/* {{{ ll_init_vendor */

/* All that effort to initialise from tables, then this insists on
   changing it again!  No I haven't a clue why.  Europe in general shows
   strongly here... */

void
ll_init_vendor (void)
{
  int i;

  H_shunt_relay_on_time = 255;
  H_transmit_level_ndB = 10;
  H_overcurrent_sense_validation_time = 2;
  H_busy_cycles_threshold = 4;

  switch (country_code)
    {
    case COUNTRY_BELGIUM:
    case COUNTRY_DENMARK:
    case COUNTRY_FINLAND:
    case COUNTRY_FRANCE:
    case COUNTRY_GERMANY:
    case COUNTRY_ITALY:
    case COUNTRY_NORWAY:
    case COUNTRY_SPAIN:
    case COUNTRY_SWITZERLAND:
    case COUNTRY_UNITED_KINGDOM:
    case COUNTRY_AUSTRIA:
    case COUNTRY_PORTUGAL:
    case COUNTRY_IRELAND:
    case COUNTRY_GREECE:
      S_0xd6 = 1;
      break;

    case COUNTRY_NETHERLANDS:
      H_shunt_relay_on_time = 27;
      S_0xd6 = 1;
      break;

    case COUNTRY_SWEDEN:
      H_transmit_level_ndB = 13;
      S_0xd6 = 1;
      break;

    case COUNTRY_EUROPE:
      for (i = 0; i < sizeof (country_init_uk); i += 2)
	homol [country_init_uk [i]] = country_init_uk [i+1];
      S_0xd6 = 1;
      country_name = "Ctr21";
      break;

    case COUNTRY_AUSTRALIA:
      H_shunt_relay_on_time = 30;
      break;

    case COUNTRY_JAPAN:
      H_DTMF_high_tone_level_ndB = 6;
      H_transmit_level_ndB = 15;
      break;

    case COUNTRY_NORTH_AMERICA:
    case COUNTRY_CANADA:
      H_pulse_dial_make_time = 0x27;
      H_pulse_dial_break_time = 0x3d;
      H_DTMF_high_tone_level_ndB = 2;
      break;
    }
}

/* }}} */
/* {{{ ll_update_parameters */

void
ll_update_parameters (void)
{
#if 0
  u8_t value;
  int i;

  if (!RegCountryKeyExist (country_code))
    return;
  for (i = 80; i > 0; i++)
    if (ReadParmFromRegistry (country_code, i, &value))
      homol [i] = value;
#endif
}

/* }}} */
/* {{{ x_init_international */

void
x_init_international (void)
{
  int i;
  for (i = 0; i < sizeof (country_init_base); i += 2)
    homol [country_init_base [i]] = country_init_base [i+1];

  if ((S_0x75 & 0xf0) == 0x50)
    H_min_ring_cadence_on_time = 0xf;

  if (x_dsp_mars)
    H_full_wave_ring_detect = 1;

  country_code = read_eeprom (0);

  if (country_code < 1 || country_code > nr_countries)
    country_code = COUNTRY_NORTH_AMERICA;

  {
    Country * country = countries + country_code - 1;

    country_name = country->name;

    if ((country->special_flag == 1 && (S_0x75 & 0xf0) == 0x50)
	|| (country->special_flag == 2 && S_0x81 == 0x13e0))
      country = countries + COUNTRY_UNITED_KINGDOM - 1;
    
    if (country->inherited)
      for (i = 0; i < country->inherited_length; i += 2)
	homol [country->inherited [i]] = country->inherited [i+1];

    if (country->sequence)
      for (i = 0; i < country->sequence_length; i += 2)
	homol [country->sequence [i]] = country->sequence [i+1];
  }

  ll_init_vendor ();
  ll_update_parameters ();

  if (S_0xd6 == 1)
    {
      for (i = 0; i < sizeof (country_init_tbr21); i += 2)
	homol [country_init_tbr21 [i]] = country_init_tbr21 [i+1];
    }
  ll_update_parameters ();

  S_line_transmit_level == H_transmit_level_ndB;
  if (H_DTMF_high_tone_level_ndB < 2)
    H_DTMF_high_tone_level_ndB = 2;

  PRINTF ("Country is %s\n", country_name);
}

/* }}} */
/* {{{ x_init_international_default */

/* FIXME.  This needs to be rechecked.  I was very tired when translating. */

void
x_init_international_default (void)
{
  S (0x35) = H_guard_tone_type;

  if (H_calling_tone_flag == 0)
    S_data_calling_tone = 0;

  if (!H_bell_modes_permitted
      || (country_code != COUNTRY_NORTH_AMERICA
	  && country_code != COUNTRY_CANADA))
    S (0x3a) = 0;

  if (S_connect_completion_timeout < H_min_no_answer_timeout)
    S_connect_completion_timeout = H_min_no_answer_timeout;
  if (S_connect_completion_timeout > H_max_no_answer_timeout)
    S_connect_completion_timeout = H_max_no_answer_timeout;

  H_no_dialtone_timeout_with_W_modifier = S_connect_completion_timeout;

  switch (country_code)
    {
    case COUNTRY_AUSTRALIA:  // 27300
      S_wait_before_dialing = 4;
      S_DTMF_dial_speed = 95;
      break;
    case COUNTRY_DENMARK:  // 2733d
      S_DTMF_dial_speed = 90;
      break;
    case COUNTRY_FINLAND:  // 27313
      S_auto_disconnect_delay = 2;
      break;
    case COUNTRY_FRANCE:  // 2731f
      S_connect_completion_timeout = 140;
      S_auto_disconnect_delay = 2;
      S_DTMF_dial_speed = 80;
      H_no_dialtone_timeout_with_W_modifier = 10;
      break;
    case COUNTRY_GERMANY:  // 2733d
    case COUNTRY_NETHERLANDS:  // 2733d
      S_DTMF_dial_speed = 90;
      break;
    case COUNTRY_NEW_ZEALAND:  // 2734f
      S_auto_disconnect_delay = 15;
      break;
    case COUNTRY_NORWAY: // 27346
      S_auto_disconnect_delay = 9;
      break;
    case COUNTRY_SPAIN: // 27358
      H_no_dialtone_timeout_with_W_modifier = 11;
      break;
    case COUNTRY_SWEDEN: // 27398
      S_DTMF_dial_speed = 70;
      break;
    case COUNTRY_UNITED_KINGDOM: // 27443
      S_wait_before_dialing = 4;
      S_DTMF_dial_speed = 85;
      break;
    case COUNTRY_JAPAN: // 27438
      S_data_calling_tone = 0;
      break;
    case COUNTRY_KOREA: // 27425
      S_wait_before_dialing = 4;
      S_connect_completion_timeout = 45;
    case COUNTRY_PORTUGAL: // 27412
      S_auto_disconnect_delay = 2;
      S_inactivity_timer = 4;
      break;
    case COUNTRY_NORTH_AMERICA: // 27438
      S_data_calling_tone = 0;
      break;
    case COUNTRY_IRELAND: // 27391
      S_wait_before_dialing = 4;
      S_DTMF_dial_speed = 0x46;
      break;
    }

  cell_nec_rx ();

  if (S_DTMF_dial_speed < H_min_DTMF_dial_speed)
    S_DTMF_dial_speed = H_min_DTMF_dial_speed;
  if (S_DTMF_dial_speed > H_max_DTMF_dial_speed)
    S_DTMF_dial_speed = H_max_DTMF_dial_speed;

  if (S_wait_before_dialing < H_min_blind_dial_pause)
    S_wait_before_dialing = H_min_blind_dial_pause;
  if (S_wait_before_dialing > H_max_blind_dial_pause)
    S_wait_before_dialing = H_max_blind_dial_pause;

  if (S_comma_dial_time < H_min_dial_pause_time)
    S_comma_dial_time = H_min_dial_pause_time;
  if (S_comma_dial_time > H_max_dial_pause_time)
    S_comma_dial_time = H_max_dial_pause_time;
}

/* }}} */
/* {{{ x_init_modem_options */

void
x_init_modem_options (void)
{
  S (0)                        = 0;
  S_ring_count                 = 0;
  S_AT_escape_char             = '+';
  S_command_termination_char   = 13;
  S_response_format_char       = 10;
  S_command_backspace_char     = 8;
  S_wait_before_dialing        = 2;
  S_connect_completion_timeout = 50;
  S_comma_dial_time            = 2;
  S_auto_disconnect_delay      = 20;
  S_DTMF_dial_speed            = 95;
  S_escape_guard_time          = 50;
  S_inactivity_timer           = 0;
  S_data_calling_tone          = 1;
  S_dial_line_rate             = 0;
  S_auto_rate                  = 1;
  S_auto_mode                  = 1;
  S_0x4f                       = 0xffff;
  S (0x40)                     = read_eeprom (1);
  S_V34_enable                 = 1;
  S_enable_K56flex_settings    = 1;
  S_network_codec_type         = 1;
  S (0x6b)                     = 0;
  S (0x6d)                     = 0;
  S (0x83)                     = 1;
  S (0x2c)                     = 1;
  S (0x70)                     = 0;
  S (0x6f)                     = 0;
  S (0x2d)                     = 1;
  S (0x2e)                     = 1;
  S (0x2f)                     = 1;
  S (0x57)                     = 1;
  S (0x31)                     = 0;
  S (0x32)                     = 1;
  S (0x33)                     = 1;
  S (0x34)                     = 2;
  S (0x51)                     = 0;
  S (0x35)                     = 0;
  S_speaker_volume             = 2;
  S_speaker_enable             = 1;
  S_negotiation_fallback       = 7;
  S_LAPM_negotiation_flags     = 7;
  S (0x38)                     = 6;
  S (0x39)                     = 3;
  S (0x3a)                     = 1;
  S (0x3b)                     = 0;
  S (0x58)                     = 2;
  S (0xe6)                     = 0x23;
  S_sleep_mode_timer_2         = 10;
  S (0x41)                     = 0;
  S (0x42)                     = 0x80;
  S (0x43)                     = 0x80;
  S (0x44)                     = 0;
  S (0x45)                     = 50;
  S (0x46)                     = 10;
  S (0x47)                     = 0x80;
  S (0x48)                     = 50;
  S (0x49)                     = 0x81;
  S (0x4a)                     = 1;
  S_0x4b                       = 100;
  S (0x4d)                     = 0;
  S (0x4e)                     = 0;
  S (0x1d)                     = 0;
  S (0x1f)                     = 0;
  S_synthetic_ring_volume      = 10;
  S_synthetic_ring_frequency   = 0;
  S (0x71)                     = 0;
  S (0x72)                     = 0;
  S (0x52)                     = 0;
  S (0x74)                     = 0;
  S (0x53)                     = 1;
  S (0x54)                     = 0xc1;
  S (0x55)                     = 1;
  S (0x6a)                     = 1;
  S (0x56)                     = 0;
  S_0x5d                       = 0x9650;
  S_0x61                       = 0x3800;
  S_0x64                       = 0x03ff;
  ETC_mode                     = 0;
  S (0x28)                     = 0;
  S (0x27)                     = 0;
  S_general_options            = 8;
  S_results_options            = 0x70;
  S (0x76)                     = 0;
  S (0x79)                     = 0xfa;
  S (0x7a)                     = 0;
  S (0x7b)                     = 5;
  S (0x7c)                     = 3;
  S (0x7d)                     = 0;
  S (0x80)                     = 0;

  if (!x_test_mode_on)
    x_init_international ();
  io_sync_power_up_init ();
  v8bis_power_up_init ();
  x_init_international_default ();

  dp_dial_string [0] = 0;
  if ((S_0x75 & 0xf0) == 0x30 && country_code == COUNTRY_JAPAN)
    dp_init_blacklist ();
}

/* }}} */

/* {{{ x_current_time */

unsigned
x_current_time (void)
{
  struct timeval tv;
  gettimeofday (&tv, 0);
#if SLOW_TIMER
  return tv.tv_sec * 50 + tv.tv_usec / 20000;
#else
  return tv.tv_sec * 1000 + tv.tv_usec / 1000;
#endif
}

/* }}} */
/* {{{ x_elapsed_time */

unsigned
x_elapsed_time (unsigned from)
{
  dp_dsp_isr();	/* Hack: so we don't need interrupts to work */
  return x_current_time () - from;
}

/* }}} */
/* {{{ x_sleep */

void
x_sleep (unsigned duration)
{
  struct timeval tv;
  tv.tv_sec = duration / 1000;
  tv.tv_usec = (duration % 1000) * 1000;
  select (0, 0, 0, 0, &tv);
}

/* }}} */

/* This is a faster way to do what ltmodem.sys does w.r.t. interrupts
   and the address I/O port.  Note that this is NOT safe on SMP (neither
   is the ltmodem.sys method).  Spinlocks will have to be added for SMP. */

int dp_current_reg;	/* Bit 8 says whether current op is a read or write */

/* {{{ dp_int_regread */

u8_t
__dp_int_regread (u16_t index_port, u8_t reg)
{
  outb_p (reg, index_port);
  return inb_p (BaseAddress + 1);
}

inline u8_t
dp_int_regread (u8_t reg)
{
  u16_t index_port =
    BaseAddress + ((reg <= 0x7f || (reg >= 0xa0 && reg <= 0xcf)) ? 0 : 2);
  return __dp_int_regread (index_port, reg);
}

/* }}} */
/* {{{ dp_int_regwrite */

void
dp_int_regwrite (u8_t reg, u8_t byte)
{
  outb_p (reg, BaseAddress);
  outb_p (byte, BaseAddress + 1);
}

/* }}} */
/* {{{ dp_restore_regs_after_interrupt */

/* My invention. */
inline void
dp_restore_regs_after_interrupt (void)
{
  int reg = dp_current_reg;
  u16_t index_port = BaseAddress;
  if (!(reg <= 0x17f || (reg >= 0x1a0 && reg <= 0x1cf)))
    index_port += 2;
  outb_p ((u8_t) reg, index_port);
}

/* }}} */
/* {{{ dp_regread */

inline u8_t
dp_regread (u8_t reg)
{
  set_wmb (dp_current_reg, reg + 0x100);
  return dp_int_regread (reg);
}

/* }}} */
/* {{{ dp_regwrite */

inline void
dp_regwrite (u8_t reg, u8_t byte)
{
  set_wmb (dp_current_reg, reg);
  dp_int_regwrite (reg, byte);
}

/* }}} */
/* {{{ dp_regandor */

inline void
dp_regandor (u16_t addr, u8_t and, u8_t or)
{
  dp_regwrite (addr, (dp_regread (addr) & and) | or);
}

/* }}} */

/* {{{ Write_mdm_byte */

void
Write_mdm_byte (u16_t offset, u8_t byte)
{
  outb_p (byte, BaseAddress + offset);
}

/* }}} */
/* {{{ Write_mdm_word */

void
Write_mdm_word (u16_t offset, u16_t word)
{
  outw_p (word, BaseAddress + offset);
}

/* }}} */
/* {{{ Init_Mars */

void
Init_Mars (void)
{
  Write_mdm_byte (0x37, 0x22);
  Write_mdm_word (0x30, 0x0001);
  Write_mdm_word (0x3c, 0x00ff);
  Write_mdm_word (0x3e, 0x0001);
}

/* }}} */

/* {{{ dp_dsp_regread_poll */

u8_t
dp_dsp_regread_poll (u8_t a, u8_t b, u8_t match)
{
  dp_byte_f = 0;
  START_TIMER (dp_cmd_timer);
  dp_regwrite (0xd8, 0x18);
  dp_regwrite (0x36, b);
  dp_regwrite (0x37, a);
  if (match != dp_byte_f)
    while (ELAPSED_TIME (dp_cmd_timer) < 200
	   && (match != (dp_byte_f = dp_regread (0x35))))
      /*empty*/;
  return dp_byte_f;
}

/* }}} */
/* {{{ dp_dsp_regread */

u8_t
dp_dsp_regread (u8_t dspreg)
{
  dp_byte_f = 0;
  dp_regwrite (0xd8, 0x18);
  dp_regwrite (0x36, dspreg);
  dp_regwrite (0x37, 0x06);
  START_TIMER (dp_cmd_timer);
  while (dp_byte_f == 0 && ELAPSED_TIME (dp_cmd_timer) < 200)
    /*empty*/;
  if (dp_byte_f != 2)
    {
      dp_regwrite (0xd8, 0xff);
      dp_byte_e = 0;
    }
  return dp_byte_e;
}

/* }}} */
/* {{{ wait_for_core_read */

void
wait_for_core_read (void)
{
  START_TIMER (dp_cmd_timer);
  while (!(dp_regread (0xd8) & 0x10) && ELAPSED_TIME (dp_cmd_timer) < 100)
    /*empty*/;
}

/* }}} */
/* {{{ dp_modem_command */

void
dp_modem_command (u8_t command, u8_t arg1, u8_t arg2)
{
  if (command == 0x0e || command == 0x0c)
    {
      if ((dp_dsp_regread (0xb5) & 0x01) == 0)
	dp_regread (0xb0);

      if ((dp_dsp_regread (0xb5) & 0x10) == 0)
	{
	  dp_regwrite (0xb6, 0xef);
	  dp_dsp_regread (0xb0);
	  dp_regwrite (0xb0, 0x00);
	}

      dp_regwrite (0xb7, 0xff);
      dp_regwrite (0xb6, 0xee);
      dp_bamil_rd7 = 0xf3;
      dp_regwrite (0xd7, 0xf3);
      dp_dsp_data_in_progress = 0;
    }

  dp_regwrite (0xd8, 0x18);
  dp_regwrite (0x35, arg2);
  dp_regwrite (0x36, arg1);
  dp_regwrite (0x37, command);
  wait_for_core_read ();
}

/* }}} */
/* {{{ dp_modem_command_long */

void
dp_modem_command_long (u8_t command, u8_t arg1, u8_t arg2,
		       u8_t arg3, u8_t arg4)
{
  dp_regwrite (0xd8, 0x18);
  dp_regwrite (0x33, arg4);
  dp_regwrite (0x34, arg3);
  dp_regwrite (0x35, arg2);
  dp_regwrite (0x36, arg1);
  dp_regwrite (0x37, command);
  wait_for_core_read ();
}

/* }}} */

/* {{{ dp_write_dsp_ram */

void
dp_write_dsp_ram (u16_t addr, u16_t data)
{
  dp_regwrite (0xd8, 0x18);
  dp_regwrite (0x32, data);
  dp_regwrite (0x33, data >> 8);
  dp_regwrite (0x34, addr);
  dp_regwrite (0x35, addr >> 8);
  dp_regwrite (0x37, 0x01);
  wait_for_core_read ();
}

/* }}} */
/* {{{ dp_read_dsp_ram */

u16_t
dp_read_dsp_ram (u16_t addr)
{
  dp_byte_f = 0;
  dp_regwrite (0xd8, 0x18);
  dp_regwrite (0x34, addr);
  dp_regwrite (0x35, addr >> 8);
  dp_regwrite (0x37, 0x04);
  START_TIMER (dp_cmd_timer);
  while (dp_byte_f == 0 && ELAPSED_TIME (dp_cmd_timer) < 200)
    /*empty*/;
  if (dp_byte_f != 1)
    {
      dp_regwrite (0xd8, 0xff);
      dp_byte_e = 0;
      dp_byte_d = 0;
      if (x_dsp_mars == 1)
	Write_mdm_word (0x3c, 0x1ff);
      dp_readram_failcnt++;
    }
  else
    dp_readram_failcnt = 0;
  return ((dp_byte_e << 8) + dp_byte_d);
}

/* }}} */
/* {{{ dp_write_dsp_array */

void
dp_write_dsp_array (u16_t * from, u16_t addr, u8_t count)
{
  do
    dp_write_dsp_ram (addr++, *from++);
  while (--count);
}

/* }}} */
/* {{{ dp_download_dsp */

void
dp_download_dsp (u16_t * data, unsigned count)
{
  u8_t * ptr = (u8_t *) data;
  u8_t save_rd7 = dp_bamil_rd7;
  if (x_dsp_mars == 1)
    {
      Write_mdm_word (0x3c, 0x1ff);
      Write_mdm_word (0x3e, 0);
    }
  dp_bamil_rd7 = 0xff;
  dp_regwrite (0xd8, 0xff);
  dp_regwrite (0xd7, 0xff);
  dp_modem_command_long (0x29, 0, 0, 0, 0);
  dp_write_dsp_ram (0x39, 0);

  while (count != 0)
    {
      unsigned block_size = *(u16_t *) (ptr + 2);
      unsigned counter;

      count -= (3 + block_size);
      counter = 0;
      dp_regwrite (0x30, 1);
      do
	dp_regwrite (0x31 + counter, *ptr++);
      while (++counter < 6);
      dp_regwrite (0xd8, 0x18);
      dp_regwrite (0x37, 0x1a);
      wait_for_core_read ();

      while (block_size != 0)
	{
	  unsigned tmp = block_size >= 3 ? 3 : block_size;
	  block_size -= tmp;
	  tmp += tmp;
	  counter = 0;
	  do
	    dp_regwrite (0x30 + counter, *ptr++);
	  while (++counter < tmp);
	  dp_regwrite (0xd8, 0x18);
	  dp_regwrite (0x37, tmp | (block_size ? 0xc0 : 0xc8));
	  wait_for_core_read ();
	}
    }

  dp_regwrite (0xb7, 0xff);
  dp_regwrite (0xd8, 0xff);
  dp_bamil_rd7 = save_rd7;
  dp_regwrite (0xd7, dp_bamil_rd7);
  if (x_dsp_mars == 1)
    {
      Write_mdm_word (0x3c, 0x1ff);
      Write_mdm_word (0x3e, 1);
    }
}

/* }}} */

/* {{{ dp_dsp_version */

u16_t
dp_dsp_version (void)
{
  if (S (0x6b) == 1 && !V34Mode)
    return dp_read_dsp_ram (0xef);

  dp_byte_f = 0;
  dp_modem_command (0x10, 0, 0);
  while (dp_byte_f != 5 && ELAPSED_TIME (dp_cmd_timer) < 100)
    /*empty*/;
  if (dp_byte_f == 5)
    return ((dp_byte_e << 8) + dp_byte_d);
  else
    return 0;
}

/* }}} */
/* {{{ dp_dsp_rom_checksum */

u16_t
dp_dsp_rom_checksum (void)
{
  dp_byte_f = 0;
  dp_modem_command (0x10, 0, 0);
  while (dp_byte_f != 5 && ELAPSED_TIME (dp_cmd_timer) < 100)
    /*empty*/;
  if (dp_byte_f == 5)
    dp_version = (dp_byte_e << 8) | dp_byte_d;
  else
    dp_version = 0;

  dp_byte_f = 0;
  dp_modem_command (0x14, 0, 0);
  while (dp_byte_f != 3 && ELAPSED_TIME (dp_cmd_timer) < 200)
    /*empty*/;
  if (dp_byte_f == 3)
    return dp_read_dsp_ram (0x49);
  else
    return 0;
}

/* }}} */
/* {{{ dp_dsp_checksum */

u16_t
dp_dsp_checksum (void)
{
  if (x_modem_state == 6)
    return dp_read_dsp_ram (0x49);
  else
    return dp_dsp_rom_checksum ();
}

/* }}} */

/* {{{ dp_ring_detected */

bool
dp_ring_detected (void)
{
  return (dp_read_dsp_ram (0x3a) & 0x20) ? true : false;
}

/* }}} */
/* {{{ dp_change_mercury_gain */

void
dp_change_mercury_gain (u8_t d0, u8_t d1, u8_t d2, u8_t d3)
{
  dp_regwrite (0x32, d3);
  dp_modem_command_long (0x2f, 1, d0, d1, d2);
  dp_read_dsp_ram (0x3a);
}

/* }}} */
/* {{{ dp_set_mercury_gain */

void
dp_set_mercury_gain (void)
{
  dp_change_mercury_gain (H_mercury_cioca_MSB_rx_gain,
			  H_mercury_cioca_LSB_hybrid,
			  H_mercury_ciocb_MSB_tx_gain,
			  H_mercury_ciocb_LSB_BOM_control);
}

/* }}} */

/* {{{ writecioc */

void
writecioc (u8_t reg, u8_t value)
{
  unsigned timer;
  START_TIMER (timer);
  while (!(dp_regread (0xd5) & 2) && ELAPSED_TIME (timer) < 500)
    /*empty*/;
  dp_regwrite (0x2b, reg);
  dp_regwrite (0x2a, value);
}

/* }}} */
/* {{{ init_1027 */

void
init_1027 (bool flag)
{
  unsigned timer;
  dp_regandor (0xcf, 0xff, 0x20);
  writecioc (0x1f, 0xc0);
  START_TIMER (timer);
  while (ELAPSED_TIME (timer) < 5)
    /*empty*/;
  dp_regandor (0xda, 0xfe, 0x00);
  writecioc (flag ? 0x1f : 0x0f, 0xc0);
  writecioc (0x40, 0x00);
  writecioc (0x85, 0x42);
  writecioc (0xc8, 0x26);
  START_TIMER (timer);
  while (!(dp_regread (0xd5) & 2) && ELAPSED_TIME (timer) < 500)
    /*empty*/;
  dp_regandor (0xda, 0xfe, 0x01);
  dp_regandor (0xcf, 0xdf, 0x00);
}

/* }}} */
/* {{{ set_boardid */

void
set_boardid (void)
{
  S_0x75 = (dp_regread (0xb9) >> 1) & 3;
}

/* }}} */
/* {{{ readbio */

inline u8_t
readbio (u16_t mask)
{
  return ((mask & 0x0f00) == 0x0f00 ? 0
	  : (mask & dp_regread ((mask & 0x0f00 == 0x0100) ? 0xbb
				: (mask & 0x0f00 == 0x0200) ? 0xdc
				: 0xb9)));
}

/* }}} */
/* {{{ writebio */

/* What a long winded way to say "set or clear bit X in reg Y..." */

void
writebio (u16_t mask, bool turn_on)
{
  if ((mask & 0xff) > 0x80)
    return;

  if ((mask & 0xf00) != 0x300)
    {
      u8_t dpreg = ((mask & 0xf00) == 0x100 ? 0xbb
		    : (mask & 0xf00) == 0x200 ? 0xdc
		    : 0xb9);
      u8_t value;

      if ((turn_on && !(mask & 0x1000))
	  || !turn_on && (mask & 0x1000))
	value = dp_regread (dpreg) | (mask & 0xff);
      else
	value = dp_regread (dpreg) & ~(mask & 0xff);
      dp_regwrite (dpreg, value);
      return;
    }

  /* writebiom */
  if ((turn_on & !(mask & 0x1000))
      || (!turn_on && (mask & 0x1000)))
    H_mercury_ciocb_LSB_BOM_control |= (mask & 0x0f);
  else
    H_mercury_ciocb_LSB_BOM_control &= ~(mask & 0x0f);
  dp_set_mercury_gain ();
}

/* }}} */
/* {{{ x_input */

inline u8_t
x_input (int arg)
{
  switch (arg)
    {
    case 56:
      if (readbio (x_dsp_mars ? 0xffff : 0x1120))
	return 1;
      readbio (x_dsp_mars ? 0xffff : 0x1120);
      break;

    case 57:
      if (!readbio (x_dsp_mars ? 0xffff : 0x1120))
	return 1;
      readbio (x_dsp_mars ? 0xffff : 0x1120);
      break;

    case 62:
      return readbio (0x1002);

    case 64:
      return (S_0x75 & 0x0f);

    case 65:
      if (cell_active)
	break;
      if (readbio (x_dsp_mars ? 0x1110 : 0x1140))
	return 1;
      readbio (x_dsp_mars ? 0x1110 : 0x1140);
      break;
    }

  return 0;
}

/* }}} */
/* {{{ x_output_init */

void
x_output_init (void)
{
  if (x_dsp_mars)
    {
      dp_regwrite (0xb8, 0x01);
      dp_regwrite (0xb9, 0x01);
      dp_regwrite (0xba, 0x40);
      dp_regwrite (0xbb, 0x40);
      dp_regwrite (0xdb, 0x41);
      dp_regwrite (0xdc, 0x00);
    }
  else
    {
      dp_regwrite (0xb8, 0xe0);
      dp_regwrite (0xb9, 0xc0);
      dp_regwrite (0xba, 0x80);
      dp_regwrite (0xbb, 0x80);
      dp_regwrite (0xdb, 0x00);
      dp_regwrite (0xdc, 0x00);
    }
}

/* }}} */
/* {{{ x_output_deinit */

void
x_output_deinit (void)
{
  /* I can't explain the point of this.
     Am I misreading the code?  I don't think so. */
  int AX = x_dsp_mars ? 0xf00 : 0x300;

  if (AX == 0x300)
    {
      dp_regwrite (0xb8, 0x00);
      dp_regwrite (0xba, 0x00);
      dp_regwrite (0xdb, 0x00);
      dp_change_mercury_gain (0, 0, 0, H_mercury_ciocb_LSB_BOM_control);
    }
  else
    {
      if (AX == 0)
	{
	  dp_regwrite (0xb8, x_dsp_mars ? 0xff : 0x02);
	  dp_regwrite (0xba, 0x00);
	  dp_regwrite (0xdb, 0x00);
	}
      else if (AX == 0x100)
	{
	  dp_regwrite (0xb8, 0x00);
	  dp_regwrite (0xba, x_dsp_mars ? 0xff : 0x02);
	  dp_regwrite (0xdb, 0x00);
	}
      else if (AX == 0x200)
	{
	  dp_regwrite (0xb8, 0x00);
	  dp_regwrite (0xba, 0x00);
	  dp_regwrite (0xdb, x_dsp_mars ? 0xff : 0x02);
	}
      dp_change_mercury_gain (0, 0, 0, 14);
    }
}

/* }}} */
/* {{{ x_output */

void x_set_hardware_options (void);

void
x_output (int arg)
{
 repeat:
  switch (arg)
    {
    case 0: // 0002e71c
      x_output_init ();
      set_boardid ();
      x_set_hardware_options ();
      break;
    case 1: // 0002e730
      x_output_deinit ();
      break;
    case 2: // 0002e73a
      dp_modem_command (0x11, 0, 0);
      break;
    case 3: // 0002e740
      dp_modem_command (0x12, 0, 0);
      break;
    case 7: // 0002e706
      dp_modem_command (0x27, 1, 0);
      arg = 24;
      goto repeat;
    case 8: // 0002e6f8
      dp_modem_command (0x27, 2, 0);
      arg = 23;
      goto repeat;
    case 9: // 0002e7dc
      dp_write_dsp_ram (0x0d, ((x_dsp_mars
				&& ((S_0x7e & 7) == 1 || (S_0x7e & 7) == 2))
			       ? 0x60 : 0x40));
      break;
    case 10: // 0002e7b8
      dp_write_dsp_ram (0x0d, ((x_dsp_mars
				&& ((S_0x7e & 7) == 1 || (S_0x7e & 7) == 2))
			       ? 0x100 : 0x70));
      break;
    case 11: // 0002e7b1
      dp_write_dsp_ram (0x0d, 0x130);
      break;
    case 12: // 0002e7aa
      dp_write_dsp_ram (0x0d, 0x400);
      break;
    case 13: // 0002e74e
      if (H_0x7f & 0x80)
	dp_change_mercury_gain (0, 2,
				H_mercury_ciocb_MSB_tx_gain,
				1 | H_mercury_ciocb_LSB_BOM_control);
      else
	writebio (x_dsp_mars ? 0x01 : 0x40, 0);
      break;
    case 14: // 0002e77e
      if (H_0x7f & 0x80)
	dp_set_mercury_gain ();
      else
	writebio (x_dsp_mars ? 0x01 : 0x40, 1);
      break;
    case 15: // 0002e9e4
      if (H_shunt_relay_on_time != 0 && !CpqFlag)
	writebio (x_dsp_mars ? 0x304 : 0x80, 0);
      break;
    case 16: // 0002ea07
      if (H_shunt_relay_on_time != 0 && !CpqFlag)
	writebio (x_dsp_mars ? 0x304 : 0x80, 1);
      break;
    case 17: // 0002ea34
      writebio (0xffff, 0);
      if (CpqFlag)
	writebio (0x302, 0);
      break;
    case 18: // 0002ea55
      writebio (0xffff, 1);
      if (CpqFlag)
	writebio (0x302, 1);
      break;
    case 23: // 0002e807
      if (x_dsp_mars && ((S_0x7e & 7) == 1 || ((S_0x7e) & 7) == 2))
	{
	  if (S_0x81 == 0x1179)
	    writebio (0x1240, 0);
	}
      else
	{
	  dp_regwrite (0xda, x_dsp_mars ? 5 : 7);
	  dp_regandor (0xcb, 0xf3, 0x00);
	  if (x_dsp_mars)
	    init_1027 (1);
	  writebio (x_dsp_mars ? 0x1240 : 0xffff, 0);
	  if (CpqFlag)
	    writebio (0x180, 0);
	}
      break;
    case 24: // 0002e89b
      if (x_dsp_mars && ((S_0x7e & 7) == 1 || ((S_0x7e) & 7) == 2))
	{
	  if (S_0x81 == 0x1179)
	    writebio (0x1240, 1);
	  if (CpqFlag)
	    writebio (0x180, 1);
	}
      else
	{
	  writebio (x_dsp_mars ? 0x1240 : 0xffff, 1);
	  if (CpqFlag)
	    writebio (0x180, 1);
	  if (x_dsp_mars)
	    {
	      dp_regandor (0xcb, 0xf3, 0x00);
	      init_1027 (0);
	    }
	  dp_regandor (0xcf, 0x3f, 0x00);
	  dp_regwrite (0xda, 1);
	  dp_regandor (0xcb, 0xff, 0x08);
	  dp_modem_command_long (0x03, 0xd1, 0, 0xea, 0);
	}
      break;
    case 26: // 0002eac3
      writebio (x_dsp_mars ? 0xffff : 0x180, 0);
      break;
    case 25: // 0002eacf
      writebio (x_dsp_mars ? 0xffff : 0x180, 1);
      break;
    case 27: // 0002ea72
      writebio (x_dsp_mars ? 0x308 : 0x304, 0);
      break;
    case 28: // 0002ea7c
      writebio (x_dsp_mars ? 0x308 : 0x304, 1);
      break;
    case 29: // 0002ea8d
      writebio (x_dsp_mars ? 0xffff : 0x308, 0);
      break;
    case 30: // 0002ea97
      writebio (x_dsp_mars ? 0xffff : 0x308, 1);
      break;
    case 31: // 0002eaa8
      writebio (x_dsp_mars ? 0xffff : 0x302, 0);
      break;
    case 32: // 0002eab2
      writebio (x_dsp_mars ? 0xffff : 0x302, 1);
      break;
    case 35: // 0002e9ba
      writebio (0x1001, 0);
      break;
    case 36: // 0002e9bd
      writebio (0x1001, 1);
      break;
    case 39: // 0002e9c8
      writebio (0x1004, 0);
      break;
    case 40: // 0002e9cb
      writebio (0x1004, 1);
      break;
    case 41: // 0002e9d9
      writebio (0x1008, 0);
      break;
    case 42: // 0002e9d6
      writebio (0x1008, 1);
      break;
    case 43: // 0002e964
      if (x_dsp_mars)
	{
	  dp_regwrite (0xb9, dp_regread (0xb9) & 0xf0);
	  dp_regwrite (0xb8, 0x0d);
	  dp_regwrite (0xb9, dp_regread (0xb9) & 0xf0);
	}
      break;
    case 44: // 0002e99d
      if (x_dsp_mars)
	dp_regwrite (0xb8, 0x01);
      break;

    case 4: // 0002eaea
    case 5: // 0002eaea
    case 6: // 0002eaea
    case 19: // 0002eaea
    case 20: // 0002eaea
    case 21: // 0002eaea
    case 22: // 0002eaea
    case 33: // 0002eaea
    case 34: // 0002eaea
    case 37: // 0002eaea
    case 38: // 0002eaea
    default:
      break;
    }
}

/* }}} */
/* {{{ x_set_hardware_options */

void
x_set_hardware_options (void)
{
  if ((x_dsp_mars && S_0x75 == 2)
      || (!x_dsp_mars && (S_0x75 == 2 || S_0x75 == 0x40 || S_0x75 == 0x50
			  || S_0x75 == 0x5a || S_0x75 == 0xdb)))
    {
      H_mercury_cioca_MSB_rx_gain = 0;
      if (H_real_complex_impedance_setting & 2)
	{
	  H_mercury_ciocb_LSB_BOM_control |= 1;
	  H_mercury_cioca_LSB_hybrid = 6;
	}
      else
	H_mercury_cioca_LSB_hybrid = 12;
      if (H_DC_loop_limiting_relay_flag)
	x_output (27);
      if (H_DC_loop_VI_characteristics_relay_flag)
	x_output (29);
      if (H_ringer_impedance_relay_flag)
	x_output (31);
    }
}

/* }}} */

/* {{{ dp_init_local_phone_timer */

inline void
dp_init_local_phone_timer (void)
{
  dp_local_phone_state = 0;
  START_TIMER (dp_local_phone_timer); 
  dp_local_phone_status = 0;
}

/* }}} */
/* {{{ dp_init_local_phone_state */

inline void
dp_init_local_phone_state (void)
{
  START_TIMER (dp_local_phone_timer); 
}

/* }}} */
/* {{{ dp_init_local_phone_onhook */

inline void
dp_init_local_phone_onhook (void)
{
  dp_local_phone_state = 1;
  START_TIMER (dp_local_phone_timer); 
  dp_local_phone_status = 0;
}

/* }}} */

/* {{{ dp_board_enable */

void
dp_board_enable (void)
{
  if (CpqFlag == 1)
    {
      if (BaseAddress != ComAddress)
	{
	  if (BaseAddress != 0xec && BaseAddress != 0xbc)
	    BaseAddress &= 0xfff8;
	}
      else if (S_0x77 != 0xccb0 && S_0x77 != 0xcdb0)
	BaseAddress = 0xec;
      else
	BaseAddress = 0xbc;
    }

  x_chip_version   = 0;
  ToshibaFlag      = 1;
  x_dsp_mars       = 1;

  /* I am guessing that the call to Init_Mars () is conditional on x_dsp_mars
     being set, and that it's optimised away in ltmodem.sys. */
  if (x_dsp_mars == 1)
    Init_Mars ();

  dp_regwrite (0xcb, 0x08);

  if (x_dsp_mars == 1)
    return;

  if (dp_dsp_regread_poll (0x06, 0xc0, 2) != 2)
    return;

  if ((dp_regread (0x34) & 3) == 1)
    {
      if (dp_dsp_regread_poll (0x10, 0x00, 5) != 5)
	return;
      if (dp_dsp_regread_poll (0x06, 0x0b, 2) != 2)
	return;

      x_chip_version = dp_regread (0x34) >> 6;
      dp_regwrite (0x18, 0x00);

      if (V16550_IRQ_Number != Irq)
	{
	  if (x_chip_version != 0)
	    dp_regwrite (0xcc, 0x0c);
	  else
	    {
	      dp_regwrite (0xcc, 0x0d);
	      dp_regwrite (0xd6, 0x05);
	    }
	}
      else
	{
	  if (x_chip_version != 0)
	    dp_regwrite (0xcc, 0x01);
	  else
	    {
	      dp_regwrite (0xcc, 0x04);
	      dp_regwrite (0xd6, 0x0d);
	    }
	}
    }
  else
    {
      u8_t tmp1, tmp2;

      switch (Irq)
	{
	case 3:  tmp1 = 4;  tmp2 = 13; break;
	case 4:  tmp1 = 0;  tmp2 = 14; break;
	case 10: tmp1 = 8;  tmp2 = 11; break;
	case 11: tmp1 = 12; tmp2 = 7;  break;
	default: return;
	}

      dp_regwrite (0xcc, tmp1);
      dp_regwrite (0xd6, tmp2);
    }
}

/* }}} */
/* {{{ dp_board_disable */

void
dp_board_disable (void)
{
  dp_regwrite (0xd6, 0x0f);
  dp_regandor (0xcd, 0xff, 0x30);
  if (x_dsp_mars == 1)
    {
      Write_mdm_word (0x3c, 0x1ff);
      Write_mdm_word (0x3e, 0);
    }
}

/* }}} */
/* {{{ dp_enable_ring_int */

void
dp_enable_ring_int (void)
{
  dp_regwrite (0xd8, 0xff);
  dp_bamil_rd7 &= 0xf1;
  dp_regwrite (0xd7, dp_bamil_rd7);
}

/* }}} */
/* {{{ dp_sleep_mode */

void
dp_sleep_mode (int arg)
{
  dp_ring_int_count = 0;
  dp_sleep = 1;
  x_output (1);
  /* This is only ever called with arg == 0. */
  if (arg != 0)
    {
      dp_enable_ring_int ();
      dp_regwrite (0xbe, 0x00);
      dp_regwrite (0xbf, 0x20);
      dp_regwrite (0xbc, (S (0x40) == 3 ? 0xd7 : S (0x40) != 0 ? 0xcf : 0xdf));
      dp_regwrite (0xbd, 0xff);
      dp_regwrite (0xbd, 0x00);
      dp_bamil_rd7 &= 0xb1;
      dp_regwrite (0xd7, dp_bamil_rd7);
      if (x_dsp_mars == 1 && D3Flag != 0)
	{
	  if (x_chip_version != 0)
	    dp_regwrite (0xa4, 0x02);
	  D3Flag = 0;
	  Write_mdm_word (0x3e, 0x00);
	}
    }
  else
    {
      dp_bamil_rd7 = 0xff;
      dp_regwrite (0xd8, 0xff);
      dp_regwrite (0xd7, 0xff);
    }

  dp_modem_command (0x2e, 0, 0);
  dp_modem_command (0x2c, 0, 0);
  dp_modem_command (0x13, 0, 0);

  dp_regwrite (0xcb, 0x1f);
  if (x_dsp_mars != 0 || x_chip_version == 0)
    dp_regwrite (0xcb, 0xff);
}

/* }}} */
/* {{{ dp_init_modem */

int rom_checksum;

/* This is big.  Maybe I made a few mistakes :-) */
int
dp_init_modem (void)
{
  u16_t checksum;

  dp_bamil_rd7 = 0xff;
  dp_dsp_data_in_progress = 0;
  dp_sleep = 0;
  dp_board_enable ();
  dp_regwrite (0xd8, 0xff);
  dp_regwrite (0xd7, 0xff);
  dp_regwrite (0xcb, 0x08);
  dp_modem_command (0x13, 0, 0);
  dp_modem_command (0x12, 0, 0);
  dp_modem_command_long (0x29, 0xff, 0x01, 0x87, 0);
  dp_bamil_rbc = 0xff;
  dp_regwrite (0xbc, 0xff);
  dp_regwrite (0xbd, 0xff);
  dp_regwrite (0xbd, 0x00);
  dp_regwrite (0xbe, 0x00);
  dp_regwrite (0xb7, 0xff);
  dp_regwrite (0xd8, 0xff);
  dp_regwrite (0xb6, 0xee);
  dp_bamil_rd7 = 0xf3;
  dp_regwrite (0xd7, 0xf3);
  x_sleep (100);
  dp_modem_command (0x13, 0, 0);
  dp_modem_command (0x2d, 0, 0);
  dp_modem_command_long (0x2e, 0x01, 0x23, 0x07, 0x00);
  dp_modem_command (0x13, 0, 0);

  /* Is there anyone there? */
  dp_byte_f = 0;
  dp_read_dsp_ram (0x3a);
  dp_read_dsp_ram (0x3a);
  if (dp_byte_f != 1)
    return 0;

  /* Is it a known DSP? */
  checksum = dp_dsp_rom_checksum ();
  rom_checksum = checksum;
  x_dsp_mars = 0;
  if (!((checksum == 0xf15d && dp_version == 0x39)
	|| (checksum == 0xe49e && dp_version == 0x46)
	|| (checksum == 0xe851 && dp_version == 0x48)))
    {
      PRINTF( "Unknown DSP version\n" );
      return 0;
    }

  /* Is it a Mars DSP? */
  if (checksum == 0xe851)
    {
      x_dsp_mars = 1;
      x_chip_version = dp_dsp_regread (0x0b) >> 6;
      if (x_chip_version != 0)
	{
	  dp_regwrite (0xa0, 0x21);
	  dp_regwrite (0xa4, 0x00);
	}
    }

  /* We've got a modem.  Initialise away. */
  x_output (0);
  if (H_full_wave_ring_detect == 0)
    {
      dp_write_dsp_ram (0x40, H_min_ring_signal_period);
      dp_write_dsp_ram (0x41, H_max_ring_signal_period);
    }
  else
    {
      dp_write_dsp_ram (0x40, H_min_ring_signal_period >> 1);
      dp_write_dsp_ram (0x41, H_max_ring_signal_period >> 1);
    }
  dp_write_dsp_ram (0x43, 2);
  if (x_chip_version == 0 && x_dsp_mars == 0)
    dp_regwrite (0xde, dp_regread (0xde) | 0x10);
  if (x_dsp_mars == 1)
    dp_modem_command_long (0x03, 0xc0, 0x00, 0x08, 0x00);
  dp_tad_downloaded = 0;
  dp_fax_downloaded = 0;
  dp_init_local_phone_timer ();
  dp_first_call = 1;
  dp_set_mercury_gain ();
  if (D3Flag == 0)
    cell_init ();
  return 1;
}

/* }}} */
/* {{{ dp_deinit_modem */

void
dp_deinit_modem (void)
{
  dp_init_modem ();
  dp_sleep_mode (0);
  dp_board_disable ();
}

/* }}} */

/* Just idle speculation, not actually used.
   (Commands for dp_modem_command). */
#define MODEM_FAST_RETRAIN	0x02
#define MODEM_DTMF_DIAL		0x0a
#define MODEM_PULSE_DIAL	0x0b
#define MODEM_RETRAIN		0x0f
#define MODEM_GET_VERSION	0x10
#define MODEM_IDLE		0x13
#define MODEM_ROM_CHECKSUM	0x14

/* dp_periodic_task */
/* dp_task */
/* dp_fax_ced_tone_cmd */
/* dp_fax_cng_tone_cmd */
/* dp_fax_startup */
/* dp_fax_connect_success */
/* dp_fax_carrier_loss */

/* dp_local_phone_detect */
/* dp_fax_silence_detect */
/* init_cng_detect */
/* look_4_cng */
/* dp_get_delay */
/* dp_init_blacklist */
/* dp_filter_modifiers */
/* dp_search_list */
/* dp_check_delay */
/* dp_check_list */
/* dp_update_blacklist */
/* dp_blacklist_periodic */
/* dp_display_number */
/* dp_show_numbers */
/* dp_show_blacklist */

/* {{{ dp_DTMF_dial_cmd */

void
dp_DTMF_dial_cmd (char digit)
{
  dp_write_dsp_ram (0x17, S_DTMF_dial_speed);
  dp_write_dsp_ram (0x18, S_DTMF_dial_speed);
  PRINTF ("dial_speed = %d\n", S_DTMF_dial_speed);
  if (cell_active)
    {
      dp_write_dsp_ram (0x11, 5 + H_DTMF_high_tone_level_ndB);
      dp_write_dsp_ram (0x12, 5 + H_DTMF_high_tone_level_ndB
			+ H_DTMF_high_low_tone_level_difference_dB);
    }
  else
    {
      dp_write_dsp_ram (0x11, H_DTMF_high_tone_level_ndB);
      dp_write_dsp_ram (0x12, H_DTMF_high_tone_level_ndB
			+ H_DTMF_high_low_tone_level_difference_dB);
    }
  dp_modem_command (0x0a, digit, 0);
}

/* }}} */
/* {{{ dp_pulse_dial_cmd */

void
dp_pulse_dial_cmd (char digit)
{
  u16_t tmp;
  u8_t tmp1 = H_pulse_dial_make_time;
  u8_t tmp2 = H_pulse_dial_break_time;
  if (S (0x0e) & 0x40)
    {
      tmp1 >>= 1;
      tmp2 = 50 - tmp1;
    }
  dp_write_dsp_ram (0x19, tmp1);
  dp_write_dsp_ram (0x1a, tmp2);
  tmp = 755;
  if (country_code == COUNTRY_FRANCE
      || country_code == COUNTRY_ITALY
      || country_code == COUNTRY_AUSTRIA)
    tmp = 900;
  dp_write_dsp_ram (0x1b, tmp);
  dp_modem_command (0x0b, digit, 0);
}

/* }}} */

/* {{{ dp_set_lb_filter */

void
dp_set_lb_filter (void)
{
  static u16_t iircoef [128] =
  {
    0x2480, 0x8f16, 0x39a8, 0x0000, 0xfbf0, 0x0410, 0x91aa, 0x3cd5,
    0xed1d, 0x0bd4, 0x0bd4, 0x878e, 0x3dfe, 0xa5c3, 0x2e40, 0x2e40,
    0x1f60, 0x8a1b, 0x3d11, 0x0000, 0xfd50, 0x02b0, 0x8bf1, 0x3e72,
    0xeeda, 0x0a48, 0x0a48, 0x862a, 0x3ee6, 0xa53e, 0x2e43, 0x2e43,
    0x2480, 0x8b07, 0x3ce9, 0x0000, 0xfe10, 0x01e3, 0x8ce3, 0x3e6a,
    0xe94c, 0x0d71, 0x0d71, 0x8834, 0x3ebe, 0xb32a, 0x27a3, 0x27a3,
    0x3180, 0x8b5f, 0x3b2a, 0x0000, 0xfc37, 0x03c9, 0x8d40, 0x3d2a,
    0xf9a6, 0x04a0, 0x04a0, 0x8613, 0x3e2b, 0xb62b, 0x2536, 0x2536,
    0x0a00, 0x8a5b, 0x3ebf, 0x0000, 0xff53, 0x00ad, 0x8aba, 0x3f5e,
    0xe42e, 0x0fc4, 0x0fc4, 0x88de, 0x3f6c, 0xb3bd, 0x27db, 0x27db,
    0x3150, 0x91ed, 0x3704, 0x0000, 0xf84e, 0x07b2, 0x97ae, 0x3ab5,
    0xfa26, 0x0505, 0x0505, 0x8763, 0x3d3d, 0xb7b4, 0x2491, 0x2491,
    0x1f60, 0x8d12, 0x3c01, 0x0000, 0xfdc2, 0x023e, 0x8e96, 0x3e45,
    0xdeb8, 0x1353, 0x1353, 0x87e6, 0x3ebd, 0x82c3, 0x4116, 0x4116,
    0x8140, 0x9fee, 0x2f93, 0x0000, 0xee26, 0x11da, 0xbcae, 0x37e6,
    0xfe51, 0x03d5, 0x03d5, 0x87b3, 0x3d2d, 0xaad0, 0x2b58, 0x2b58
  };
  int ch, i;
  dp_write_dsp_ram (0x760, dp_threshold >= 35
		    ? dp_threshold + 4 : dp_threshold + 2);
  switch (H_dial_tone_or_call_progress_filter)
    {
    case 0: ch = 0x00; break;
    case 2: ch = 0x20; break;
    case 3: ch = 0x30; break;
    case 4: ch = 0x40; break;
    case 5: ch = 0x50; break;
    case 6: ch = 0x60; break;
    default: ch = 0x10; break;
    }
  if (S (0x40) != 0)
    ch = 0x70;
  for (i = 0; i < 16; i++)
    dp_write_dsp_ram (i + 0x764, iircoef [ch + i]);
}

/* }}} */
/* {{{ dp_detect_tones_cmd */

void
dp_detect_tones_cmd (void)
{
  dp_set_lb_filter ();
  dp_write_dsp_ram (0x26, H_data_or_fax_answer_detection_threshold_dB);
  dp_write_dsp_ram (0x1c, 0x2b9b);
  dp_write_dsp_ram (0x1d, 0x11cd);
  dp_write_dsp_ram (0x1e, 0x3d0a);
  dp_write_dsp_ram (0x1f, 0x3b57);
  dp_write_dsp_ram (0x20, 0x0851);
  dp_write_dsp_ram (0x21, 0xf190);
  dp_write_dsp_ram (0x22, 0xed78);
  dp_write_dsp_ram (0x23, 0xef70);
  dp_write_dsp_ram (0x24, 0xe8ce);
  dp_write_dsp_ram (0x25, 0xe782);

  if (S (0x38) != 10)
    {
      dp_write_dsp_ram (0xfc0, 0x172b);
      dp_write_dsp_ram (0xfc1, 0xf4bc);
      dp_modem_command (0x08, 12, 0);
    }
  else
    dp_modem_command (0x08, 10, 0);

  START_TIMER (dp_dial_tone_timer);
  START_TIMER (dp_quiet_answer_timer);
  START_TIMER (dp_busy_tone_timer);
  dp_busy_state = 0;
  dp_tone_state = 0;
  peak_energy = 0;
  peak_busy_energy = 0;
}

/* }}} */
/* {{{ dp_dial_tone_detected */

bool
dp_dial_tone_detected (void)
{
  u16_t energy2 = dp_read_dsp_ram (0x354); /* Haven't a clue! */
  u16_t energy = dp_read_dsp_ram (0x794);
  bool transition = false;
  switch (dp_tone_state)
    {
    case 0:
      if (dp_read_dsp_ram (0x3a) & 0x1000)
	{
	  dp_tone_on_time = 0;
	  dp_tone_off_time = 0;
	  dp_dialtone_cycles = 0;
	  START_TIMER (dp_tone_timer);
	  dp_tone_state = 1;
	}
      START_TIMER (dp_dial_tone_timer);
      break;

    case 1:
      if (!(dp_read_dsp_ram (0x3a) & 0x1000) || energy < (peak_energy >> 1))
	{
	  dp_tone_on_time = ELAPSED_TIME (dp_tone_timer);
	  dp_tone_state = 2;
	  transition = true;
	}
      if (country_code != COUNTRY_ITALY
	  && energy > (peak_energy >> 2)
	  && energy2 / ((u32_t) energy + 1) > 5)
	{
	  START_TIMER (dp_dial_tone_timer);
	  peak_energy = energy;
	}
      if (energy > peak_energy)
	peak_energy = energy;
      break;

    case 2:
      if (dp_read_dsp_ram (0x3a) & 0x1000)
	{
	  if (energy > (peak_energy >> 1))
	    {
	      peak_energy = 0;
	      dp_tone_off_time = ELAPSED_TIME (dp_tone_timer);
	      dp_tone_state = 1;
	      transition = true;
	    }
	  else if ((country_code == COUNTRY_NETHERLANDS
		    || country_code == COUNTRY_ITALY)
		   && ELAPSED_TIME (dp_tone_timer) > 500)
	    peak_energy = energy;
	}
      else
	{
	  if ((country_code != COUNTRY_ITALY
	       && ELAPSED_TIME (dp_tone_timer) > H_dial_tone_hole_time)
	      || ELAPSED_TIME (dp_tone_timer) > 1250)
	    START_TIMER (dp_dial_tone_timer);
	}
      break;
    }

  if (transition)
    {
      START_TIMER (dp_tone_timer);
      if (dp_tone_off_time > H_dial_tone_hole_time)
	{
	  if (country_code != COUNTRY_ITALY)
	    START_TIMER (dp_dial_tone_timer);
	  else
	    {
	      if ((dp_tone_off_time > 210 && dp_tone_off_time < 830)
		  || dp_tone_off_time < 180
		  || dp_tone_off_time > 1155
		  || (dp_tone_on_time > 230 && dp_tone_on_time < 530)
		  || dp_tone_on_time < 190
		  || dp_tone_on_time > 680)
		START_TIMER (dp_dial_tone_timer);

	      if ((dp_tone_off_time > 180 && dp_tone_off_time < 220)
		  || (dp_tone_on_time > 180 && dp_tone_off_time < 220))
		dp_dialtone_cycles++;
	      else
		START_TIMER (dp_dial_tone_timer);
	    }
	}
    }

  if (ELAPSED_TIME (dp_dial_tone_timer) >= 100 * H_dial_tone_validation_time
      && (dp_read_dsp_ram (0x3a) & 0x1000)
      && energy > 10 * H_dialtone_level
      && (country_code != COUNTRY_ITALY || dp_dialtone_cycles <= 2))
    return 1;

  return 0;
}

/* }}} */
/* {{{ dp_answer_tone_detected */

bool
dp_answer_tone_detected (void)
{
  static int i = -1;
  if (i != dp_answer_state)
    PRINTF ("dp_answer_state %d\n", (i = dp_answer_state));
  switch (dp_answer_state)
    {
    case 0:
      if (dp_read_dsp_ram (0x28) & 0x3f0)
	{
	  START_TIMER (dp_answer_on_timer);
	  START_TIMER (dp_answer_off_timer);
	  dp_answer_state = 1;
	}
      break;
	  
    case 1:
      if (dp_read_dsp_ram (0x28) & 0x3f0)
	{
	  return ((S (0x6a) == 2 && ELAPSED_TIME (dp_answer_on_timer) >= 100)
		  || (ELAPSED_TIME (dp_answer_on_timer)
		      > 10 * H_answer_tone_validation_time));
	}
      START_TIMER (dp_answer_off_timer);
      dp_answer_state = 2;
      break;

    case 2:
      if (dp_read_dsp_ram (0x28) & 0x3f0)
	{
	  if (ELAPSED_TIME (dp_answer_off_timer) > 50)
	    START_TIMER (dp_answer_on_timer);
	  dp_answer_state = 1;
	}
      break;
    }
  return false;
}

/* }}} */
/* {{{ dp_busy_tone_detected */

bool
dp_busy_tone_detected (void)
{
  u16_t energy2 = dp_read_dsp_ram (0x354); /* Haven't a clue! */
  u16_t energy = dp_read_dsp_ram (0x794);
  bool transition = false;
  switch (dp_busy_state)
    {
    case 0:
      if (ELAPSED_TIME (dp_busy_tone_timer) > 3)
	{
	  if (dp_read_dsp_ram (0x3a) & 0x1000)
	    {
	      dp_busy_on_cadence = 0;
	      dp_busy_off_cadence = 1;
	      dp_busy_state = 1;
	    }
	  else
	    {
	      dp_busy_on_cadence = 1;
	      dp_busy_off_cadence = 0;
	      dp_busy_state = 2;
	    }
	}
      break;

    case 1:
      if ((!(dp_read_dsp_ram (0x3a) & 0x1000)
	   || energy < (peak_busy_energy << 1))
	  && ELAPSED_TIME (dp_busy_tone_timer) > 30)
	{
	  dp_busy_on_cadence = ELAPSED_TIME (dp_busy_tone_timer);
	  dp_busy_state = 2;
	  transition = true;
	}
      else
	{
	  if (x_modem_mode != 2
	      && S_data_calling_tone == 1
	      && !dp_calling_tone_disable
	      && ELAPSED_TIME (dp_timer1) >= 1800)
	    {
	      dp_busy_on_cadence = ELAPSED_TIME (dp_busy_tone_timer);
	      transition = true;
	    }
	  if (energy2 > 3 * energy)
	    START_TIMER (dp_continuous_busy_timer);
	}
      break;

    case 2:
      if ((dp_read_dsp_ram (0x3a) & 0x1000)
	  && energy > (peak_busy_energy >> 1))
	{
	  dp_busy_off_cadence = ELAPSED_TIME (dp_busy_tone_timer);
	  dp_busy_state = 1;
	}
      else
	START_TIMER (dp_continuous_busy_timer);
      break;
    }

  if (ELAPSED_TIME (dp_busy_tone_timer) > 900)
    {
      dp_busy_cycles = 0;
      dp_busy_on_cadence = 0;
      dp_busy_off_cadence = 0;
    }

  if (transition)
    {
      PRINTF ("on = %u, off = %u\n", dp_busy_on_cadence, dp_busy_off_cadence);
      START_TIMER (dp_busy_tone_timer);
      if ((country_code != COUNTRY_NORTH_AMERICA
	   && country_code != COUNTRY_CANADA)
	  ? ((H_max_busy_cadence_on_time != 0
	      /* Parameterised busy tone detection. */
	      ? (dp_busy_on_cadence >= 10 * H_min_busy_cadence_on_time
		 && dp_busy_on_cadence <= 10 * H_max_busy_cadence_on_time
		 && dp_busy_off_cadence >= 10 * H_min_busy_cadence_off_time
		 && dp_busy_off_cadence <= 10 * H_max_busy_cadence_off_time)
	      /* Default algorithm. */
	      : (dp_busy_on_cadence >= 100
		 && dp_busy_on_cadence <= 850
		 && ((dp_busy_on_cadence <= 150
		      && (dp_busy_on_cadence + dp_busy_off_cadence > 250))
		     || (dp_busy_on_cadence >= 150
			 && (dp_busy_on_cadence
			     <= (75 + dp_busy_off_cadence
				 + (dp_busy_off_cadence / 2))))))))
	  /* American/Canadian busy tone detection. */
	  : ((dp_busy_on_cadence >= 195
	      && dp_busy_on_cadence <= 305
	      && dp_busy_off_cadence >= 195
	      && dp_busy_off_cadence <= 305)
	     || (dp_busy_on_cadence >= 420
		 && dp_busy_on_cadence <= 580
		 && dp_busy_off_cadence >= 420
		 && dp_busy_off_cadence <= 580)))
	{
	  dp_busy_cycles++;
	  dp_busy_on_cadence = 0;
	  dp_busy_off_cadence = 0;
	}
    }

  if (dp_busy_cycles >= H_busy_cycles_threshold)
    {
      dp_busy_cycles = 0;
      return true;
    }

  if (energy > peak_busy_energy)
    peak_busy_energy = energy;

  return false;
}

/* }}} */
/* {{{ dp_quiet_answer_detected */

bool
dp_quiet_answer_detected (void)
{
  if (dp_read_dsp_ram (0x27) != 0)
    {
      START_TIMER (dp_quiet_answer_timer);
      return false;
    }
  return (ELAPSED_TIME (dp_quiet_answer_timer) < 5000);
}

/* }}} */
/* {{{ dp_bong_tone_detected */

bool
dp_bong_tone_detected (void)
{
  if (dp_read_dsp_ram (0x28) & 3)
    {
      dp_bong_tone1_on_time = ELAPSED_TIME (dp_dial_timer);
      return false;
    }
  return ((dp_read_dsp_ram (0x28) & 0xc)
	  && dp_bong_tone1_on_time >= 50
	  && (ELAPSED_TIME (dp_dial_timer) - dp_bong_tone1_on_time) >= 100);
}

/* }}} */
/* {{{ dp_detect_over_current */

void
dp_detect_over_current (int arg)
{
  if (H_overcurrent_sense_validation_time == 0)
    return;
  if (x_input (65) == 0)
    START_TIMER (dp_ocp_timer);
  else if (ELAPSED_TIME (dp_ocp_timer)
	   >= 10 * H_overcurrent_sense_validation_time)
    {
      dp_state = 33;
      x_dial_residual = arg ? 9 : 2;
      x_output (3);
      cell_nec_rx ();
    }
}

/* }}} */

/* dp_ringback_init */
/* dp_ringback_detected */
/* dp_calling_tone_cmd */
/* dp_stored_number */
/* dp_test_dialstring */

/* dp_enable_ring_int */

/* dp_v32_ac_workaround */
/* dp_disable_33600 */
/* dp_v34_workaround */
/* dp_disable_workaround */
/* dp_transmit_level */
/* dp_async_user_mode */
/* dp_update_rate */
/* dp_update_modem_options */
/* dp_analog_loopback_cmd */
/* dp_rem_digital_loop_cmd */
/* dp_v54_terminate_cmd */

/* {{{ dp_modem_idle_cmd */

void
dp_modem_idle_cmd (void)
{
  dp_modem_command (0x13, 0, 0);
  dp_write_dsp_ram (0xf20, 0x42f8);
}

/* }}} */
/* {{{ dp_run_rom */

void
dp_run_rom (void)
{
  u8_t byte;

  dp_modem_idle_cmd ();

  byte = dp_regread (0xde);
  byte |= 1;
  dp_regwrite (0xde, byte);
  byte &= 0xdf;
  dp_regwrite (0xde, byte);
  x_sleep (2);
  byte &= 0xfe;
  dp_regwrite (0xde, byte);
  x_sleep (2);

  if (CpqFlag == 1)
    {
      u16_t save = BaseAddress;
      BaseAddress = 0x260;
      dp_regwrite (0xc5, save);
      dp_regwrite (0xc6, save >> 8);
      BaseAddress = save;
    }

  dp_init_modem ();
  dp_regandor (0xda, 0xf7, 0x00);
}

/* }}} */
/* {{{ dp_offhook_cmd */

bool
dp_offhook_cmd (void)
{
  if (cell_active)
    {
      at_init_diagnostics ();
      x_status = 0;
      return true;
    }

  START_TIMER (dp_timer);
  while (dp_ring_detected () && ELAPSED_TIME (dp_timer) < 3000)
    /*empty*/;

  if (S_speaker_enable != 0)
    {
      x_output (S_speaker_volume == 3 ? 11 : S_speaker_volume == 2 ? 10 : 9);
      dp_modem_command (0x13, 0, 0);
      x_output (8);
    }

  x_output (14);
  x_output (17);
  x_output (2);

  if (H_shunt_relay_on_time != 0xff)
    x_output (15);

  if (H_overcurrent_sense_validation_time != 0 && dp_dialing_status == 0)
    {
      unsigned local_timer;
      x_output (28);
      x_output (30);
      START_TIMER (dp_timer);
      local_timer = dp_timer;
      while (ELAPSED_TIME (local_timer) <= 10)
	{
	  if (x_input (65) == 0)
	    continue;
	  START_TIMER (local_timer);
	  if (ELAPSED_TIME (dp_timer)
	      >= H_overcurrent_sense_validation_time * 10)
	    {
	      x_output (3);
	      x_send_mail (7, 1, 0, 0);
	      cell_nec_rx ();
	      return false;
	    }
	}

      if (H_DC_loop_limiting_relay_flag != 0)
	x_output (27);
      if (H_DC_loop_VI_characteristics_relay_flag != 0)
	x_output (29);
    }

  if (x_modem_mode != 2 && x_modem_mode != 3)
    {
      x_output (19);
      at_init_diagnostics ();
      x_status = 0;
    }
  else
    {
      x_output (20);
      if (CpqFlag)
	x_output (13);
    }

  if (H_shunt_relay_on_time != 0xff)
    {
      int limit = H_shunt_relay_on_time * 10;
      START_TIMER (dp_timer);
      if (H_overcurrent_sense_validation_time != 0 && limit >= 10)
	limit -= 10;
      while (dp_dialing_status == 0 && ELAPSED_TIME (dp_timer) <= limit)
	/*empty*/;
      x_output (16);
    }

  dp_init_local_phone_timer ();
  return true;
}

/* }}} */
/* {{{ dp_get_line_rate */

void
dp_get_line_rate (void)
{
  u16_t word = dp_read_dsp_ram (0x36);
  x_line_rate_tx = ((word >> 8) & 0x1f) + 2;
  x_line_rate    = (word & 0x1f) + 2;
}

/* }}} */
/* {{{ dp_update_diagnostics */

u16_t x_status_6;
u16_t x_status_8;
u32_t x_status_c, x_modem_ec;
u16_t x_status_10;
u16_t x_status_12;
u16_t x_status_14;
u16_t x_status_16;
u16_t x_status_18;
u16_t x_status_1a;
u16_t x_status_32;
u16_t x_status_34;
u32_t x_status_38;

void
dp_update_diagnostics (void)
{
  if (V34Mode != 0)
    dp_get_line_rate ();
  x_status_6  = x_line_rate_tx;
  x_status_8  = x_line_rate;
  x_status_10 = dp_read_dsp_ram (0x4c);
  x_status_c  = x_modem_ec; /* long */
  x_status_12 = dp_read_dsp_ram (0x4a);
  x_status_14 = dp_read_dsp_ram (0x03);
  x_status_16 = dp_read_dsp_ram (0x54);
  x_status_18 = dp_read_dsp_ram (0x55);
  x_status_1a = dp_read_dsp_ram (0x56);
  x_status_32 = dp_read_dsp_ram (0x80a);
  x_status_34 = dp_read_dsp_ram (0x80b);
  x_status_38 = x_dial_residual; /* long */
}

/* }}} */
/* {{{ dp_onhook_cmd */

void
dp_onhook_cmd (void)
{
  cell_onhook ();

  dp_modem_command (0x13, 0, 0);
  x_output (7);
  x_output (14);
  x_output (20);
  x_output (16);

  if (S (0x72) == 0)
    {
      x_output (25);
      x_output (3);
      x_output (18);
    }
  else
    S (0x72) = 10;

  dp_write_dsp_ram (0xf20, 0x42f8);
  dp_regwrite (0xd8, 0xff);
  dp_regwrite (0xb7, 0xff);
  dp_tad_downloaded = 0;
  dp_fax_downloaded = 0;
  if (S (0x74) == 1)
    {
      S (0x74) = 0;
      S (0x52) = 0;
    }

  v8bis_app_reset ();
  dp_init_local_phone_state ();
  V34Mode = 0;
  dp_update_diagnostics ();

  if (x_status == 0)
    {
      x_status = 1;
      ll_save_diagnostics ();
    }

  if (S (0x72) == 0 && x_modem_mode != 3)
    dp_run_rom ();

  dp_first_call = 0;
}

/* }}} */

/* {{{ dp_retrain_cmd */

void
dp_retrain_cmd (void)
{
  dp_update_rate ();
  dp_modem_command (0x0f, 0, 0);
}

/* }}} */
/* {{{ dp_fast_retrain_cmd */

void
dp_fast_retrain_cmd (void)
{
  dp_update_rate ();
  dp_modem_command (0x02, 0, 0);
}

/* }}} */

/* {{{ dp_dial_modify_check */

bool
dp_dial_modify_check (char digit)
{
  switch (digit)
    {
    case ';': // 2b294
      if (dp_dialing_status == 3 || x_modem_mode == 3)
	break;
      cell_send ();
      x_modem_state = 4;
      x_send_mail (0x16, 2, 0, 0);
      return true;
    case ',': // 2b2c4
      dp_dial_pause = S_comma_dial_time * (S (0x40) == 2 ? 3000 : 1000);
      dp_dial_state = DIAL_PAUSE;
      break;
    case '$': // 2b2f5
      dp_bong_tone1_on_time = 0;
      dp_detect_tones_cmd ();
      dp_dial_state = DIAL_WAITING_FOR_BONG;
      break;
    default:
      return false;
    }
  cell_send ();
  return true;
}

/* }}} */
/* {{{ dp_dial */

int
dp_dial (void)
{
  char digit;
  static int i = -1;
  if (i != dp_dial_state)
    PRINTF ("dp_dial_state %d\n", (i = dp_dial_state));

  switch (dp_dial_state)
    {
    case DIAL_INIT: // 0, 2b341
      dp_busy_cycles = 0;
      dp_answer_state = 0;
      dp_max_pause = 0;
      START_TIMER (dp_continuous_busy_timer);
      if (dp_stored_number (dp_dial_string) == 0)
	{
	  x_dial_residual = 0;
	  return 3;
	}
      dp_dial_ptr = dp_dial_string;
      if (H_dial_modifier_validation == 1)
	{
	  if (dp_test_dialstring (dp_dial_string) == 0)
	    dp_dial_string [0] = 0;
	  dp_dial_ptr = dp_dial_string;
	  while ((*dp_dial_ptr == 'T' || *dp_dial_ptr == 'P'
		  || *dp_dial_ptr == ',')
		 && *(dp_dial_ptr+1) == ',')
	    {
	      *(dp_dial_ptr+1) = *dp_dial_ptr;
	      dp_dial_ptr++;
	    }
	  if (*dp_dial_ptr == ',')
	    dp_dial_ptr++;
	}
      if (*dp_dial_ptr == 'T')
	S (0x2f) = 1;
      if (*dp_dial_ptr == 'P' && H_not_pulse_dial_flag == 0)
	S (0x2f) = 0;

      if (dp_dialing_status == 0
	  && (x_modem_mode == 0 || x_modem_mode == 1))
	{
	  if (S (0x7a) == 0 && H_blacklist_or_delay_flag
	      && dp_check_list (dp_dial_string))
	    return 3;
	  if (country_code != COUNTRY_NORTH_AMERICA
	      && !dp_first_call
	      && (ELAPSED_TIME (dp_timer1)
		  < (country_code == COUNTRY_FRANCE) ? 6000 : 5000))
	    return 0;
	}

      START_TIMER (dp_dial_timer);

      if (cell_originate () == 1)
	dp_dial_state = DIAL_DIGIT;
      else if (dp_dialing_status == 0)
	{
	  if (dp_offhook_cmd () == 0)
	    {
	      x_dial_residual = 2;
	      return 3;
	    }
	  dp_dialing_status = 2;
	  START_TIMER (dp_ocp_timer);
	  dp_dial_state = DIAL_1;
	}
      else if (dp_dialing_status == 1)
	{
	  dp_offhook_cmd ();
	  dp_dialing_status = 2;
	  dp_dial_state = DIAL_DIGIT;
	}
      else
	{
	  if (S (0x6a) == 2)
	    dp_offhook_cmd ();
	  if (x_modem_mode != 2)
	    {
	      if (S_speaker_enable != 0)
		x_output (8);
	      x_output (14);
	      x_output (19);
	    }
	  dp_dial_state = DIAL_DIGIT;
	}
      START_TIMER (dp_dial_timer);
      if (S_speaker_enable == 3)
	x_output (7);
      break;
	    
    case DIAL_1: // 2b51f
      if (ELAPSED_TIME (dp_dial_timer) >= 55)
	{
	  if (*dp_dial_ptr != 0 && *dp_dial_ptr != ';')
	    {
	      dp_detect_tones_cmd ();
	      dp_dial_state = DIAL_WAITING_FOR_DIALTONE;
	      START_TIMER (dp_dial_timer);
	      PRINTF ("Entering DIAL_WAITING_FOR_DIALTONE\n");
	    }
	  else
	    {
	      if (*dp_dial_ptr == ';')
		dp_dialing_status = 0;
	      dp_dial_state = DIAL_DIGIT;
	    }
	}
      dp_detect_over_current (0);
      break;

    case DIAL_WAITING_FOR_DIALTONE: // 2b577
      if (S (0x2d) == 0 && H_blind_dial_flag != 0)
	{
	  if (ELAPSED_TIME (dp_dial_timer) > 1000 * S_wait_before_dialing)
	    {
	      if (H_blind_dial_flag == 2
		  && (x_modem_mode == 0 || x_modem_mode == 1))
		x_output (8);
	      START_TIMER (dp_timer);
	      dp_dial_state = DIAL_DIGIT;
	    }
	}
      else if (ELAPSED_TIME (dp_dial_timer) > 10 * H_dial_tone_detection_delay
	       && dp_dial_tone_detected () == 1)
	dp_dial_state = DIAL_DIGIT;
      else if (S (0x2e) != 0 && H_detect_busy_tone_during_dial_tone != 0
	       && dp_busy_tone_detected () == 1)
	{
	  x_dial_residual = 3;
	  if (H_blacklist_or_delay_flag != 0 && x_modem_mode != 3)
	    dp_update_blacklist (dp_dial_string, 1);
	  return 3;
	}
      else if (ELAPSED_TIME (dp_dial_timer)
	       >= (country_code == COUNTRY_AUSTRIA ? 6000 : 10000))
	{
	  x_dial_residual = 2;
	  if (H_blacklist_or_delay_flag != 0 && x_modem_mode != 3)
	    dp_update_blacklist (dp_dial_string, 1);
	  return 3;
	}
      dp_detect_over_current (0);
      break;

    case DIAL_WAITING_FOR_W_DIALTONE: // 2b6aa
      if (H_no_dialtone_timeout_with_W_modifier >= S_connect_completion_timeout
	  && dp_busy_tone_detected () == 1)
	{
	  x_dial_residual = 3;
	  return 3;
	}
      else if (dp_dial_tone_detected () == 1)
	{
	  if (S (0x2f) == 0)
	    x_output (15);
	  dp_dial_state = DIAL_DIGIT;
	}
      else if (ELAPSED_TIME (dp_dial_timer)
	       >= 1000 * H_no_dialtone_timeout_with_W_modifier)
	{
	  x_dial_residual = 2;
	  return 3;
	}
      break;

    case DIAL_HOOK_FLASH: // 2b713
      if (ELAPSED_TIME (dp_dial_timer) > 10 * H_hook_flash_time)
	{
	  x_output (2);
	  dp_dial_state = DIAL_HOOK_FLASH_RECOVERY;
	  x_output (15);
	  START_TIMER (dp_dial_timer);
	}
      break;

    case DIAL_HOOK_FLASH_RECOVERY: // 2b757
      if (ELAPSED_TIME (dp_dial_timer) > 20
	  && ELAPSED_TIME (dp_dial_timer) < 50)
	x_output (16);
      if (ELAPSED_TIME (dp_dial_timer) > 10 * H_hook_flash_time)
	dp_dial_state = DIAL_DIGIT;
      break;

    case DIAL_CHECKING_QUIET_ANSWER: // 2b7aa
      if (dp_busy_tone_detected () == 1)
	{
	  x_dial_residual = 3;
	  return 3;
	}
      if (dp_dial_tone_detected () != 1 && dp_answer_tone_detected () != 1)
	{
	  if (dp_quiet_answer_detected () == 1)
	    {
	      dp_dial_state = DIAL_DIGIT;
	      break;
	    }
	  if (ELAPSED_TIME (dp_long_timer) < 10 * S_connect_completion_timeout)
	    break;
	}
      x_dial_residual = 5;
      return 3;

    case DIAL_WAITING_FOR_BONG: // 2b800
      if (dp_busy_tone_detected () == 1)
	{
	  x_dial_residual = 3;
	  return 3;
	}
      else if (dp_bong_tone_detected () == 1)
	{
	  dp_dial_pause = 1000;
	  dp_dial_state = DIAL_PAUSE;
	}
      else if (ELAPSED_TIME (dp_dial_timer) >= 50000)
	{
	  x_dial_residual = 2;
	  return 3;
	}
      break;

    case DIAL_DIGIT: // 2b857
      if (dp_redial == 1)
	{
	  dp_redial = 0;
#if 0
	  FAE = 0; // BYTE
	  FAC = S_command_termination_char;
	  FAD = S_response_format_char;
	  x_strcat (&FAC, "Dialing...");
	  x_strcat (&FAC, dp_dial_ptr);
	  x_send_mail (0x21, 3, &FAC, x_strlen (&FAC));
#endif
	  PRINTF ("%cDialing...%s%c", S_response_format_char,
		  dp_dial_ptr, S_command_termination_char);
	}
      START_TIMER (dp_dial_timer);

      switch (*dp_dial_ptr)
	{
	case 0: // 2b8dc
	  if (cell_active == 1)
	    cell_send ();
	  if (x_dsp_mars && ((S (0x7e) & 7) == 1 || (S (0x7e) & 7) == 2)
	      && S_speaker_enable != 0)
	    x_output (S_speaker_volume == 3 ? 11
		      : S_speaker_volume == 2 ? 10
		      : 9);
	  return 2;

	case '!': // 2b930
	  if (H_hook_flash_time != 0
	      && (S (0x2f) != 0 || country_code == COUNTRY_NORTH_AMERICA))
	    {
	      x_output (3);
	      dp_dial_state = DIAL_HOOK_FLASH;
	    }
	  break;

	case '$': // 2b983
	  dp_bong_tone1_on_time = 0;
	  dp_detect_tones_cmd ();
	  dp_dial_state = DIAL_WAITING_FOR_BONG;
	  break;

	case ',': // 2b9b5
	  if (H_comma_pause_duration_limit == 255)
	    dp_dial_pause = 1000 * S_comma_dial_time;
	  else if (dp_max_pause < 1000 * H_comma_pause_duration_limit)
	    {
	      dp_max_pause += 1000 * S_comma_dial_time;
	      dp_dial_pause = 1000 * S_comma_dial_time;
	    }
	  else
	    dp_dial_pause = 0;
	  dp_dial_state = DIAL_PAUSE;
	  break;

	case ';': // 2ba27
	  if (S (0x2f) == 0)
	    x_output (16);
	  if (dp_dialing_status != 3 && x_modem_mode != 3)
	    {
	      x_modem_state = 4;
	      x_send_mail (0x16, 2, 0, 0);
	      return 1;
	    }
	  break;

	case '@': // 2ba69
	  dp_detect_tones_cmd ();
	  dp_dial_state = DIAL_CHECKING_QUIET_ANSWER;
	  START_TIMER (dp_long_timer);
	  break;

	case 'A': case 'B': case 'C': case 'D': // 2baa8
	  if (H_disallow_ABCD_dialing)
	    {
	      dp_dial_ptr++;
	      return 0;
	    }
	  /* Fall through */

	case '1': case '2': case '3':
	case '4': case '5': case '6':
	case '7': case '8': case '9':
	case '*': case '0': case '#': // 2bac7
	  if (S (0x40) != 0 && cell_state_online () == 0)
	    {
	      START_TIMER (dp_dial_timer);
	      while (*dp_dial_ptr != 0)
		{
		  if (dp_dial_modify_check (*dp_dial_ptr))
		    {
		      if (*dp_dial_ptr++ != ';'
			  || dp_dialing_status == 3
			  || x_modem_mode == 3)
			return 0;
		      return 1;
		    }
		  // I assumed cell_dial does not use dp_dial_ptr.
		  // Otherwise logic in next 3 lines is wrong.
		  if (*dp_dial_ptr != '-')
		    cell_dial (*dp_dial_ptr);
		  dp_dial_ptr++;
		}
	      return dp_dial ();
	    }

	  if (S (0x2f) == 0)
	    {
	      int pulses = *dp_dial_ptr;
	      if (pulses < '0' || pulses > '9')
		break;
	      pulses = pulses - '0';
	      if (H_pulse_dial_digit_pattern == 2)
		pulses++;
	      else if (H_pulse_dial_digit_pattern == 3)
		pulses = 10 - pulses;
	      else
		pulses = pulses ? pulses : 10;
	      START_TIMER (dp_dial_timer);
	      dp_pulse_dial_cmd (pulses);
	    } else {
	      int tone = *dp_dial_ptr;
	      if (x_dsp_mars && (S (0x7e) == 1 || S (0x7e) == 2))
		dp_write_dsp_ram (0x0d, 0x0090);
	      if (tone >= '0' && tone <= '9')
		tone = tone - '0';
	      else if (tone == '*')
		tone = 10;
	      else if (tone == '#')
		tone = 11;
	      else /* tone >= 'A' && tone < 'F' */
		tone = tone - 'A' + 12;
	      dp_DTMF_dial_cmd (tone);
	    }
	  dp_dial_state = DIAL_DIGIT_RECOVERY;
	  break;

	case 'P': // 2bc0e
	  if (!H_not_pulse_dial_flag
	      && (dp_dial_string [0] == 'P'
		  || !H_disallow_mixed_tone_and_pulse_dial))
	    {
	      S (0x2f) = 0;
	      if (dp_dial_ptr [1] != 0)
		{
		  x_output (15);
		  if (country_code == COUNTRY_NETHERLANDS
		      || country_code == COUNTRY_AUSTRIA)
		    {
		      dp_dial_pause = 250;
		      dp_dial_state = DIAL_PAUSE;
		    }
		}
	    }
	  break;

	case 'T': // 2bce9
	  if (dp_dial_string [0] == 'T'
	      || ! H_disallow_mixed_tone_and_pulse_dial)
	    S (0x2f) = 1;
	  break;

	case 'V': // 2bcc9
	  if (x_dsp_mars && ((S (0x7e) & 7) == 1 || (S (0x7e) & 7) == 2))
	    break;
	  dp_dialing_status = 3;
	  break;

	case 'W': // 2bca4
	  if (x_modem_mode != 3)
	    {
	      dp_detect_tones_cmd ();
	      if (S (0x2f) == 0)
		x_output (16);
	    }
	  dp_dial_state = DIAL_WAITING_FOR_DIALTONE;
	  break;

	case '^': // 2bc8e
	  if (H_calling_tone_flag == 1)
	    dp_calling_tone_disable = 1;
	  break;

	case 'S': // 2bd04
	  if (dp_dial_ptr [1] == '=')
	    load_number (dp_dial_ptr [2] - '0', dp_dial_ptr + 1);
	  dp_redial = 1;
	  break;
	}
      if (*dp_dial_ptr != 0)
	dp_dial_ptr++;
      break;

    case DIAL_PAUSE: // 2bd3c
      if (ELAPSED_TIME (dp_dial_timer) >= dp_dial_pause
	  || ELAPSED_TIME (dp_timer) > 0xfff5) // Unsigned.  Peculiar value.
	{
	  dp_dial_state = DIAL_DIGIT;
	  if (S (0x2f) == 0 && (*dp_dial_ptr == 0 || *dp_dial_ptr == 'T'))
	    x_output (16);
	}
      break;

    case DIAL_DIGIT_RECOVERY: // 2bd9d
      if (!(dp_read_dsp_ram (0x3a) & 0x10))
	{
	  dp_dial_state = DIAL_DIGIT;
	  if (S (0x2f) == 0 && (*dp_dial_ptr == 0 || *dp_dial_ptr == 'T'))
	    {
	      dp_dial_pause = (country_code == COUNTRY_FRANCE ? 900
			       : country_code == COUNTRY_AUSTRIA ? 300
			       : 750);
	      START_TIMER (dp_dial_timer);
	      dp_dial_state = DIAL_PAUSE;
	    }
	}
      break;
    }

  return 0;
}

/* }}} */
/* {{{ dp_init_variables */

void
dp_init_variables (void)
{
  x_modem_state = 1;
  x_dial_residual = 0;
  dp_state = 1;
  dp_calling_tone_disable = 0;
  dp_wDspRetrainState = 0;
  DpIdleFillChar = 0;
  dp_init_blacklist ();
  dp_init_local_phone_state ();
  dp_tad_downloaded = false;
  dp_fax_downloaded = false;
  dp_fdsp_record = false;
  v8bis_power_up_init ();
  dp_redial = false;
}

/* }}} */

/* dp_cleardown_cmd */
/* dp_originate_cmd */
/* dp_answer_cmd */
/* dp_line_connect */
/* dp_ring_detected */
/* dp_init_modem_connection */
/* dp_dial_connect_success */
/* dp_rate_change_cmd */
/* dp_auto_rate */
/* dp_carrier_loss */
/* dp_retrain */
/* dp_modem_rate */
/* dp_hdlc_mode */
/* dp_sync_mode */
/* dp_sync_mode_hdlc */
/* dp_async_8bit_mode */
/* dp_sync_mode_mark */
/* dp_hdlc_mode_no_crc */

/* dp_lapm_error_check */
/* dp_lapm_error_check_init */
/* dp_lapm_auto_rate */

/* {{{ dp_start_transmit */

void
dp_start_transmit (void)
{
  if (dp_dsp_data_in_progress)
    return;
  dp_dsp_data_in_progress = true;
  dp_regwrite (0xb0, 0x00);
}

/* }}} */

/* dual_port_tx */
/* dual_port_rx */
/* {{{ dp_dsp_isr */

int
dp_dsp_isr (void)
{
  u8_t isr = dp_int_regread (0xd7);

  if (isr == 0 || isr == 0xff)
    {
      dp_restore_regs_after_interrupt ();
      return 0;
    }
  if (isr==8) {
    //    PRINTF( "+" );
  } else
    PRINTF( "Unexpected interrupt: %x\n", isr );

  if (x_dsp_mars)
    Write_mdm_word (0x3c, 0x101);

  dp_int_regwrite (0xd7, 0xff);
  if (isr & 0x02)
    {
      dp_int_regwrite (0xd8, 0xff);
      if (dp_ring_int_count > 1)
	{
	  dp_bamil_rd7 |= 2;
	  dp_ring_int = 1;
	  dp_init_local_phone_timer ();
	}
      else
	{
	  if (dp_sleep && dp_ring_int_count == 1)
	    dp_regwrite (0xcb, 0x08);
	  x_wakeup ();
	}
    }

  if (isr & 0x08)
    {
      dp_int_regwrite (0xd8, 0x08);
      dp_byte_c = dp_int_regread (0x32);
      dp_byte_d = dp_int_regread (0x33);
      dp_byte_e = dp_int_regread (0x34);
      dp_byte_f = dp_int_regread (0x35);
    }

  if (isr & 0x04)
    {
      if (dp_int_regread (0xb6) & 0x10)
	{
	  if (S (0x6b) == 1 && V34Mode == 0)
	    io_dual_port_rx ();
	  else
	    dual_port_rx ();
	}
      if (dp_int_regread (0xb6) & 0x01)
	dual_port_tx ();
    }

  if (isr & 0x40)
    {
      dp_int_regwrite (0xbd, dp_int_regread (0xbc));
      if (isr & 0x18)
	{
	  cell_isr ();
	  x_wakeup ();
	}
    }

  dp_int_regwrite (0xd7, dp_bamil_rd7);
  dp_restore_regs_after_interrupt ();
  return 1;
}

/* }}} */

/* cell_nec_rx */

/* {{{ read_Blacktip_Resource */

bool
read_Blacktip_Resource (void)
{
  if ((CMOS_READ (0x50) & 3) != 1)
    return false;

  switch ((CMOS_READ (0x51) >> 2) & 7)
    {
    case 0:  V16550_IRQ_Number =  3; break;
    case 1:  V16550_IRQ_Number =  4; break;
    case 2:  V16550_IRQ_Number =  7; break;
    case 3:  V16550_IRQ_Number = 10; break;
    case 4:  V16550_IRQ_Number = 11; break;
    default: return false;
    }

  switch (CMOS_READ (0x52) & 3)
    {
    case 0: BaseAddress = 0x100; break;
    case 1: BaseAddress = 0x108; break;
    case 2: BaseAddress = 0x110; break;
    case 3: BaseAddress = 0x118; break;
    }

  Irq = V16550_IRQ_Number;
  return true;
}

/* }}} */

/* dp_read_constellation */

/* dp_download_fax */
/* dp_download_fax_v17 */
/* dp_download_V8bis */
/* eeprom_clock */
/* eeprom_start */
/* eeprom_stop */
/* write_opcode */
/* write_byte */
/* read_byte */
/* write_eeprom_chip */
/* read_eeprom_chip */
/* go_offhook */
/* test_dtmf */
/* fax_test_signals */
/* test_signals */
/* dp_test_cmd */

/* dp_a8_msg */
/* dp_a8a_msg */
/* v8_ci_ans_init */
/* v8_ci_sequence_detected */
/* v8_ci_orig */
/* dp_detect_v8_tones_cmd */
/* dp_v8_tone_detected */
/* dp_detect_cm */
/* dp_v8_originate_msg */
/* dp_v8_orig_no_ans_msg */
/* dp_hex_to_hexascii */
/* v8bis_power_up_init */
/* v8bis_init */
/* v8bis_reset */
/* v8bis_clear_dsp_ram */
/* v8bis_single_tone_detect_setup */
/* v8bis_rx_reset */
/* v8bis_app_reset */
/* v8bis_dte_cmd_tones_detection_setup */
/* measure_eng */
/* v8bis_handset_dualtones_detection_setup */
/* v8bis_handset_dualtones_detect */
/* v8bis_datacall_dualtones_detect */
/* v8bis_app_reset */
/* v8bis_in_cmd_state */
/* v8bis_clear_dsp_ram_from_vcid */
/* v8bis_new_cmd */
/* v8bis_v21_startup */
/* v8bis_a8r_msg */
/* save_rx_data */
/* v8bis_dual_tone_xmit */
/* v8bis_single_tone_xmit */
/* v8bis_msg_xmit */
/* v8bis_msg_xmit_setup */
/* v8bis_data_dualtones_detect */
/* v8bis_single_tone_detect */
/* v8bis_rx_msg_init */
/* get_v8bis_rx_msg */
/* v8bis_dte_cmd_dtones_detect */
/* v8bis_dte_cmd_and_tone_detect */
/* v8bis_dte_control_background */
/* dp_v8bis_originate */
/* dp_v8bis_answer */
/* dp_v8bis_voice_originate */
/* dp_v8bis_voice_answer */
/* dp_v8bis_dual_tone_xmit */
/* dp_v8bis_single_tone_xmit */
/* dp_v8bis_detect_singletone_cmd */
/* dp_v8bis_singletone_detect */
/* dp_v8bis_v21_startup */
/* v8bis_get_rx_msg */
/* dp_v8bis_preamble_tone_tx_setup */
/* dp_v8bis_msg_xmit */
/* dp_v8bis_orig_rx_setup */
/* get_v8bis_ack_msg */
/* get_56k_v8bis_cl_msg */
/* get_v70_v8bis_cl_msg */
/* dp_v8bis_resp_dtones_detect_setup */
/* dp_v8bis_resp_dtones_detect */
/* dp_v8bis_ans_rx_setup */
/* get_56k_v8bis_ms_msg */
/* get_v70_v8bis_ms_msg */
/* ms_msg_nak_required */
/* dp_v8bis_orig_background */
/* dp_v8bis_ans_background */
/* dp_v8bis_voice_orig_background */
/* dp_v8bis_voice_ans_background */
/* dp_download_56k */
/* dp_56k_get_digital_loss_and_rob_bit_status */
/* HostSlicerTbl */
/* dp_56k_download_digital_loss */
/* dp_56k_update_connect_status */
/* dp_56k_startup_cmd */
/* dp_56k_dial_connect_success */
/* dp_56k_retrain_jump */
/* dp_56k_to_v34_retrain */
/* dp_56k_v34_init */
/* dp_56k_retrain_cmd_luc */
/* dp_56k_retrain_cmd_rw */
/* dp_56k_fast_rate_change */
/* dp_56k_retrain_cmd */
/* dp_56k_init */

/* dp_56k_idle */
/* dp_56k_options_init */
/* dp_test_56k_rom */
/* dp_56k_speaker_cmd */
/* dp_print_dsp_sptr */
/* k56_retrain_detect */
/* auto_rate_max_limit_0db */
/* dp_56k_download_8level_table */
/* check_error_code */
/* dp_56k_fall_fb_check */
/* check_Mu_law_A_law */
/* dp_56k_background */
/* dp_init_decoder */
/* dp_decoder */
/* dp_demapper */
/* RxSymbolBitProc */
/* QM_bit_decode */
/* ResetFrame */
/* descrambyte */
/* ShellDemapper */
/* form_g2_g4_z8 */
/* pack_KQ_bits */
/* dp_init_vpcm_decoder */
/* dp_vpcm_decoder */
/* dp_demod_cnv */
/* pack_vpcm_bits */
/* vpcm_RxSymbolBitProc */
/* descrambyte_pcm */
/* descramword_pcm */
/* io_init_pdm */
/* io_rx_async */
/* io_rx_hdlc */
/* io_pdm_background */
/* io_dual_port_rx */
/* dp_download_vpcm */
/* dp_vpcm_get_digital_loss_and_rob_bit_status */
/* dp_vpcm_update_connect_status */
/* dp_vpcm_startup_cmd */
/* dp_vpcm_dial_connect_success */
/* dp_vpcm_retrain_jump */
/* dp_vpcm_to_v34_retrain */
/* dp_vpcm_v34_init */
/* dp_vpcm_retrain_cmd_luc */
/* dp_vpcm_retrain_cmd_rw */
/* dp_vpcm_fast_rate_change */
/* dp_vpcm_retrain_cmd */
/* dp_vpcm_init */
/* dp_vpcm_idle */
/* dp_vpcm_options_init */
/* dp_vpcm_print_dsp_sptr */
/* vpcm_retrain_detect */
/* dp_vpcm_fall_fb_check */
/* dp_vpcm_background */
/* init_ll_para */
/* dp_download_array */
/* dp_download_cpt */
/* dp_download_cp */
/* GetGain */
/* DetAMuMix6db */
/* UpLevel */
/* iabs */
/* HeaderInit */
/* GetMdist */
/* ToMsgBox */
/* VpcmDesFrmConsTbl */
/* GetLrnTblIndex */
/* PackBMS */
/* io_detect_inactivity */
/* io_init_task */
/* io_sleep */
/* io_send_break */
/* io_periodic_task */
/* io_task */
/* UART_Flush */
/* UART_SetRxQue */
/* UART_SetTxQue */
/* UART_SetQue */
/* io_get_dte_rx_char */
/* io_at_esc_seq_detected */
/* io_get_dte_tx_count */
/* io_put_dte_tx_char */
/* io_write_dte_fifo */
/* UART_CommReadCount */
/* io_get_dte_rx_bytes */
/* UART_CommWriteCount */
/* UART_CommWriteString */
/* UART_CopyDteTxData */
/* UART_dsr_on */
/* UART_dsr_off */
/* UART_cts_off */
/* UART_cts_on */
/* UART_cd_off */
/* UART_cd_on */
/* UART_ri_off */
/* UART_ri_on */
/* UART_dtr_off */
/* UART_dtr_on */
/* UART_dtr_blocked */
/* UART_rts_off */
/* UART_rts_on */
/* UART_start_break */
/* UART_stop_break */
/* UART_dce_break_on */
/* UART_data_bits */
/* UART_parity */
/* UART_stop_bits */
/* UART_xoff_on */
/* io_init_dce_tx_fifo */
/* io_init_dce_rx_fifo */
/* io_get_dce_rx_count */
/* io_read_dce_fifo */
/* io_start_dce_transmit */
/* io_write_dce_fifo */
/* io_get_dce_tx_count */
/* io_get_dce_queue_count */
/* io_dce_tx_fifo_empty */
/* io_tx_hdlc_frame */
/* io_sync_power_up_init */
/* io_sync_send_EM_flag */
/* v80_connect_rate_conversion */
/* io_sync_send_connect_msg */
/* io_sync_init */
/* io_sync_disconnect */
/* io_sync_on_line */
/* io_sync_retrain_done */
/* get_user_max_rate */
/* set_s37_reg */
/* io_send_dce_last_byte */
/* io_send_dce_one_byte */
/* io_tx_dte_to_dce_sync_data */
/* io_dce_to_dte_sync_data */
/* io_sync_background */
/* io_tx_break */
/* io_tx_dte_to_dce_data */
/* io_dce_to_dte_data */
/* io_tx_dce_to_dte_data */
/* io_dte_loopback */
/* io_tx_v54 */
/* io_dce_loopback */
/* io_compute_crc */
/* io_fax_hdlc_dte_to_buf */
/* io_fax_hdlc_buf_to_dce */
/* io_fax_sync_tx_ini */
/* io_fax_sync_dte_to_dce */
/* io_fax_sync_dce_to_dte */
/* io_fax_hdlc_dce_to_buf */
/* io_fax_hdlc_buf_to_dte */
/* io_fax_init */
/* io_fax_inactivity_check */
/* io_fax_monitor_dte */
/* io_fax_background */
/* io_fax_disconnect */
/* lapm_about */
/* lapm_disconnect */
/* lapm_init */
/* lapm_background */
/* setup_ETC_modem_rate */
/* lapm_tx_break */
/* lapm_next_tx_rframe */
/* lapm_tx_hdlc_frame */
/* lapm_transmit_odp */
/* lapm_transmit_adp */
/* lapm_adp_detected */
/* lapm_odp_detected */
/* lapm_mnp_lr_detected */
/* lapm_next_tx_wframe */
/* lapm_build_iframe */
/* lapm_decode_iframe */
/* lapm_get_frame_from_queue */
/* lapm_process_iframe */
/* lapm_init_variables */
/* lapm_reset_timer */
/* lapm_timer_expired */
/* lapm_clear_rx_frame */
/* lapm_put_frame_on_queue */
/* lapm_no_of_rx_frames_full */
/* lapm_rx_hdlc_frame */
/* lapm_clean_rcv_frame */
/* mnp_crc */
/* lapm_process_ack */
/* lapm_rx_frame_type */
/* lapm_send_control_frame */
/* lapm_build_xid_parameter */
/* lapm_send_xid */
/* lapm_process_xid_parameter */
/* lapm_process_xid */
/* lapm_send_break */
/* lapm_process_break */
/* v42bis_init_dict */
/* v42bis_init_encode */
/* v42bis_init_decode */
/* v42bis_init_variables */
/* v42bis_bitpack */
/* v42bis_bitpack_flush */
/* v42bis_unpack */
/* v42bis_unpack_flush */
/* v42bis_flush */
/* v42bis_encode */
/* v42bis_decode */
/* mnp_init_variables */
/* mnp_next_tx_wframe */
/* mnp_next_tx_rframe */
/* mnp_negotiate_params */
/* mnp_reset_flow_timer */
/* mnp_flow_timer_expired */
/* mnp_init */
/* mnp_disconnect */
/* mnp_background */
/* mnp_read_octet_from_dce_buf */
/* mnp_tx_break */
/* mnp_build_lr_frame */
/* mnp_build_control_frame */
/* mnp_rx_frame_type */
/* mnp_tx_control_frame */
/* mnp_build_lt_frame */
/* mnp_decode_lt_frame */
/* mnp_get_frame_from_queue */
/* mnp_process_lt_frame */
/* mnp_tx_octet_frame */
/* mnp_tx_hdlc_frame */
/* mnp_rx_data */
/* mnp5_ad_frq_tx_init */
/* mnp5_ad_frq_rx_init */
/* mnp5_ad_frq_init */
/* mnp5_div_freq_table */
/* mnp5_run_length_encoding */
/* mnp5_run_length_flush */
/* mnp5_ad_frq_xmt */
/* mnp5_encode */
/* mnp5_encode_flush */
/* mnp5_ad_frq_rcv */
/* mnp5_decode */
/* cell_disable_system_int */
/* cell_disable_dc_i_int */
/* cell_enable_dc_i_int */
/* cell_disable_dc_io1_int */
/* cell_enable_dc_io1_int */
/* cell_dc_cntrl2_low */
/* cell_dc_cntrl2_high */
/* cell_dc_i_read */
/* cell_dc_io2_read */
/* cell_dc_io1_read */
/* cell_set_busy_line_high */
/* cell_set_busy_line_low */
/* cell_init_io */
/* set_cell_state_online */
/* cell_motorola_rx */
/* cellular_bg */
/* cell_send_motorola_cmd_end */
/* cell_motorola_tx */
/* cell_send_motorola_cmd */
/* cell_send_nec_cmd */
/* send_cell_cmd */
/* cell_isr */
/* cell_init */
/* cell_send */
/* cell_dial */
/* cell_recall_land_setting */
/* cell_save_land_setting */
/* cell_set_cell_parameter */
/* cell_originate */
/* cell_answer */
/* cell_onhook */
/* enable_cell_filter */
/* cell_at_prnC */
/* cell_state_online */
/* cellDataRx */
/* vxdUpdateLeads */
/* vxdGetMSR */
/* V16550_Init */
/* V16550_Read_RBR */
/* V16550_Write_THR_buffer */
/* V16550_Write_THR */
/* V16550_Read_IER */
/* V16550_Write_IER */
/* V16550_Read_IIR */
/* V16550_Write_FCR */
/* V16550_Read_LCR */
/* V16550_baudrate_update */
/* V16550_Write_LCR */
/* V16550_Read_MCR */
/* V16550_Write_MCR */
/* V16550_Read_LSR */
/* V16550_Read_MSR */
/* V16550checkIIR */
/* at_init_variables */
/* at_atoi */
/* at_itoa */
/* at_dtoa */
/* at_dce_speed_code */
/* at_dte_speed_code */
/* at_form_connect_rate */
/* at_numericFormatResult */
/* at_output_msg */
/* at_connect_code */
/* at_task */
/* atparse_cmd */
/* at_cmd_filter */
/* get_next_char */
/* atparse_state */
/* display_id */
/* at_cmd_err_ret */
/* at_isdegit */
/* at_dummy */
/* at_f */
/* at_y */
/* at_and_k */
/* at_backslash_j */
/* at_backslash_b */
/* at_backslash_k */
/* at_backslash_q */
/* at_a */
/* at_b */
/* at_d */
/* at_e */
/* at_h */
/* at_i */
/* at_l */
/* at_m */
/* at_n */
/* at_o */
/* at_p */
/* at_q */
/* at_t */
/* at_v */
/* at_w */
/* at_x */
/* init_ETC_defaults */
/* init_landline_ETC */
/* at_z */
/* at_and_c */
/* at_and_d */
/* init_defaults */
/* at_and_f */
/* at_and_g */
/* at_and_m */
/* at_and_p */
/* at_and_q */
/* at_and_s */
/* at_and_w */
/* at_and_z */
/* at_and */
/* at_backslash_a */
/* at_backslash_n */
/* at_backslash_v */
/* at_backslash_t */
/* at_backslash_x */
/* at_backslash */
/* at_percent_b */
/* at_percent_c */
/* at_percent_e */
/* at_percent_x */
/* at_dash */
/* at_percent */
/* at_data_compression */
/* at_and_t */
/* s_PRINTF */
/* printViewLine */
/* printViewLineStr */
/* printViewLineED */
/* printViewLineComp */
/* printViewLineUnNum */
/* printViewLineNum */
/* at_putsCrLf */
/* at_init_diagnostics */
/* dp_lin_to_db */
/* at_diagnostics */
/* at_view */
/* at_and_v */
/* decdigit_to_hexchar */
/* at_atohex */
/* at_and_and_c */
/* at_and_and_r */
/* at_and_and_n */
/* at_and_and */
/* at_percent_t */
/* at_prn */
/* at_pound */
/* ud_display */
/* ud_response */
/* at_v90 */
/* at_Srange */
/* at_s */
/* GetReadOnlySReg */
/* at_get_faxrate */
/* at_fax */
/* at_v8_cmd */
/* get_value */
/* at_sync */
/* at_sync_access_mode */
/* at_sync_mode */
/* at_sync_flow_control */
/* get_v8bis_parameter */
/* get_v8bis_msg */
/* at_v8bis_a8t_cmd */
/* at_v8_v8bis_cmd */
/* at_plus_GMM */
/* InterlockedExchange */
/* DbgBreakPoint */
/* _seh_longjmp_unwind */
/* RtlUnwind */

/**** section PAGESER */
/* SerialCommError */
/* SerialFlush */
/* SerialStartFlush */
/* SerialStartImmediate */
/* SerialCompleteImmediate */
/* SerialTimeoutImmediate */
/* SerialGetNextImmediate */
/* SerialCancelImmediate */
/* SerialGiveImmediateToIsr */
/* SerialGrabImmediateFromIsr */
/* SerialPropagateDeleteSharers */
/* SerialReset */
/* SerialGetDivisorFromBaud */
/* SerialUnload */
/* SerialCleanupDevice */
/* SerialCleanupExternalNaming */
/* SerialMemCompare */
/* SerialLogError */
/* SerialUnReportResourcesDevice */
/* SerialGetStats */
/* SerialClearStats */
/* SerialSetChars */
/* SerialSetBaud */
/* SerialSetLineControl */
/* SerialGetModemUpdate */
/* SerialGetCommStatus */
/* SerialSetEscapeChar */
/* SerialIoControl */
/* SerialGetProperties */
/* SerialPutChar */
/* SerialProcessLSR */
/* SerialSetDTR */
/* SerialClrDTR */
/* SerialSetRTS */
/* SerialClrRTS */
/* SerialSetupNewHandFlow */
/* SerialSetHandFlow */
/* SerialTurnOnBreak */
/* SerialTurnOffBreak */
/* SerialPretendXoff */
/* SerialPretendXon */
/* SerialHandleReducedIntBuffer */
/* SerialProdXonXoff */
/* SerialHandleModemUpdate */
/* SerialPerhapsLowerRTS */
/* SerialStartTimerLowerRTS */
/* SerialInvokePerhapsLowerRTS */
/* SerialDecrementRTSCounter */
/* SerialNullSynch */
/* SerialCreateOpen */
/* SerialClose */
/* SerialCheckOpen */
/* SerialMarkOpen */
/* SerialMarkClose */
/* SerialCleanup */
/* SerialGetCharTime */
/* SerialStartPurge */
/* SerialPurgeInterruptBuff */
/* SerialQueryInformationFile */
/* SerialSetInformationFile */
/* SerialRead */
/* SerialStartRead */
/* SerialCompleteRead */
/* SerialCancelCurrentRead */
/* SerialGrabReadFromIsr */
/* SerialReadTimeout */
/* SerialUpdateReadByIsr */
/* SerialIntervalReadTimeout */
/* SerialGetCharsFromIntBuffer */
/* SerialUpdateInterruptBuffer */
/* SerialUpdateAndSwitchToUser */
/* SerialResizeBuffer */
/* SerialMoveToNewIntBuffer */
/* SerialUpdateAndSwitchToNew */
/* SerialKillAllReadsOrWrites */
/* SerialGetNextIrp */
/* SerialTryToCompleteCurrent */
/* SerialRundownIrpRefs */
/* SerialStartOrQueue */
/* SerialCancelQueued */
/* SerialCompleteIfError */
/* SerialStartMask */
/* SerialGrabWaitFromIsr */
/* SerialGiveWaitToIsr */
/* SerialFinishOldWait */
/* SerialCancelWait */
/* SerialCompleteWait */
/* SerialWrite */
/* SerialStartWrite */
/* SerialGetNextWrite */
/* SerialCompleteWrite */
/* SerialProcessEmptyTransmit */
/* SerialGiveWriteToIsr */
/* SerialCancelCurrentWrite */
/* SerialWriteTimeout */
/* SerialGrabWriteFromIsr */
/* SerialGrabXoffFromIsr */
/* SerialCompleteXoff */
/* SerialTimeoutXoff */
/* SerialCancelCurrentXoff */
/* SerialGiveXoffToIsr */

/**** section INIT: ****/
/* DriverEntry */
/* SerialInitializeRootInterrupt */
/* SerialInitializeRootMultiPort */
/* SerialInitializeOneController */
/* SerialInitializeController */
/* SerialDoesPortExist */
/* SerialItemCallBack */
/* SerialGetConfigInfo */
/* SerialPutInConfigList */
/* SerialGetMappedAddress */
/* SerialSetupExternalNaming */
/* SerialReportResourcesDevice */

/* Voice functions */

void dp_apollo_command(int a8, int aC, int a10, int a14, int a18, int a1c, int a20, int a24)
{
  dp_regwrite(0xD8, 0x18);
  dp_regwrite(0x30, a24);
  dp_regwrite(0x31, a20);
  dp_regwrite(0x32, a1c);
  dp_regwrite(0x33, a18);
  dp_regwrite(0x34, a14);
  dp_regwrite(0x35, a10);
  dp_regwrite(0x36, aC);
  dp_regwrite(0x37, a8);
  wait_for_core_read();
}

void
dp_download_tad(void)
{
  dp_download_dsp(wDspTad, 0x949);
  dp_modem_command(0x13, 0, 0);
}

int dp_vt_cntrl, dp_vsp, dp_monitor_hec, dp_dialtone_enable, 
    x_dma_xfer = 0;

/*
  at_voice() {
  on AT+VTR
    x_send_mail(0x4f, 3, 0, 0)
    if (S (0x44) == {1,5,7})
      x_send_mail(0x3f, 1, 0, 0)	But these are _nops_ ?!
    else
      x_send_mail(0x3e, 1, 0, 0)

  on AT+VLS:
    set register _S+0x44

  on AT+VTX:
    on +VRX goto .L84
    x_send_mail(0x4a, 3, 0, 0)
    if (S (0x44) == 1)
      x_send_mail(0x3b, 1, 0, 0)
    else if (S (0x44) == 2)
      x_send_mail(0x47, 1, 0, 0)
    else if (S (0x44) == {5, 7})
      x_send_mail(0x38, 1, 0, 0)
    else x_send_mail(0x3c, 1, 0, 0)

  on AT+VRX: .L84: 
    x_send_mail(0x4b, 3, 0, 0)
    if (S (0x44) == 1)
      x_send_mail(0x3a, 1, 0, 0)
    else if (S (0x44) == 2)
      x_send_mail(0x48, 1, 0, 0)
    else if (S (0x44) == 5)
      x_send_mail(0x3d, 1, 0, 0)	-- Suitable for receive
    else if (S (0x44) == 7)
      x_send_mail(0x57, 1, 0, 0)
    else x_send_mail(0x37, 1, 0, 0)

  }
  AT+VLS=1 is suitable for AT+VTX

  S register 0x44 is Analog Source/Destination, set by AT+VLS.

  
  
*/


void
dp_start_tad(int arg)
{
  int vsm2 = 0, var_minus4 = 0, var_30 = 0;
  int vsm;	/* Sampling mode, as set by AT+VSM */
  int al;

  if (dp_sleep == 1)
    dp_init_modem();
  dp_modem_command(0x13, 0, 0);
  dp_vt_cntrl = arg;	/* Strange: uninitialized! */
  x_sleep(0x32);
  if (dp_tad_downloaded == 0) {
    x_output(0x07);	/* Basically hangup */
    dp_modem_command(0x13, 0, 0);
    dp_download_tad();
    dp_vsp = 0;
    x_output(0x17);
  }
  dp_monitor_hec = 0;
  dp_dialtone_enable = 1;

  if (0 == x_dma_xfer && dp_vt_cntrl) {	/* Hmm, if I do if (0), I basically crash the modem */
    dp_write_dsp_ram(0x3d, 0x200);
    dp_regwrite(0xb6, 0xff);
    dp_regread(0xb0);
    dp_regread(0xb0);
    al = dp_vt_cntrl;
  }
  /* 38aa0: */
  /* ecx = sampling mode, from 68229. 0x84 == ADPCM, confirmed now.
     0x81 is 16-bit linear, which should be closest to native.
     ecx -= 0x80 */

  switch (S (0x49)) {
  case 0x80: arg |= 0x0e; break;
  case 0x82: arg |= 0x09; break;
  case 0x81:
  case 0x83: break;
  default: arg |= 0x04; break;
  }

  al = S (0x4a);
  if (al!=1) {
  L7:
    PRINTF( "Sorry, I dunno what setne does\n" );
    if (al != 0x02)
      al = 1;
    else 
      al = 0;
    al--;
    al &= 0x08;
    var_minus4 = al;
  } else
    var_minus4 = 0x04;	/* This is 8kHz */

 L8:
  dp_apollo_command(0x20, arg, 0, 
		    7, H_TAD_tx_level_dB -3, 
		    0, 0, var_minus4 );
  // dp_set_tad_speaker_gain();
  // dp_set_tad_mic_gain();
  // dp_detect_dtmf();
  dp_write_dsp_ram(0x3c, 0x28);
  al = dp_vt_cntrl;
  if ((al == 0xc0) ||
      (al == 0xe0) ||
      (al == 0x40) ||
      (al == 0x80)) {
   L9:
    dp_write_dsp_ram(0x85, 0x105);
    dp_write_dsp_ram(0x85, 0x1305);
  }
  else 
    dp_write_dsp_ram(0x85, 0x1205);
  /* bc2: */
  al = dp_vt_cntrl;
  if (!(al & 0x80))
    goto L13;

  if ((al == 0xa0) || (al == 0xf0))
    /* bda */
    dp_write_dsp_ram(0x87e, 0xa000);
  else
    dp_write_dsp_ram(0x87e, 0x8000);
  dp_write_dsp_ram(0x87c, 0x3000);
  /* bf0 */
  goto L14;

 L13:
  dp_write_dsp_ram(0x87e, 0);
 L14:
  dp_write_dsp_ram(0xe0, 0x80);
  dp_write_dsp_ram(0xe4, 0x80);
  dp_write_dsp_ram(0xe5, 0xfff0);
  dp_dsp_data_in_progress = 0;
  dp_tad_downloaded = 1;
}

int
Read_mdm_byte (u16_t offset)
{
  return inb_p (BaseAddress + offset);
}

void
expect_irqs(void)
{
  int i;
  for (i=0; i<500; i++) {
    dp_dsp_isr();
    if (Read_mdm_byte(0x3c))
      PRINTF( "!" );
    usleep(1000);
  }
}

int io_voice_count = 0;

char io_dce_tx_sptr[0x100] = { 0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50,0, 50, 100, 50, };

int io_voice_count_rx = 0;

char io_dce_rx_sptr[0x100];

int debug = 0;

int
dp_dualport_vc_Rc(void)
{
  int cnt, esi, edi, ebx, al;
  char *ebp;
  int readreg = 0; /* 0x10(%esp,1) */
  if (io_voice_count_rx != 0)
    return 0;
  cnt = dp_read_dsp_ram(0xe7);
  if (0x100 > cnt) {
    if (debug)
      PRINTF( "Rc count too low: %x\n", cnt );
    return 0;
  }
  ebp = io_dce_rx_sptr;
#if 0
  if (0x100 >= io_voice_count_rx) {
    io_voice_count = 0;
    return 0;
  }
#endif

  do {
    if (dp_regread(0xb0)) {
      readreg = 0x90;

      do {
	al = dp_regread(readreg);
	//	PRINTF( "%x", al );
	*ebp++ = al;
	readreg++;
      } while (readreg <= 0x9f);

      io_voice_count_rx += 0x10;
    }

    dp_regread(0xb0);
    ebx = x_current_time();
    while(1) {
      if (dp_regread(0xb7) & 0x10)
	break;
      if (x_elapsed_time(ebx) > 0x32) {
	PRINTF( "Timeout receiving\n" );
	io_voice_count = 0;
	return 0;
      }
    }
  } while (0x100 > io_voice_count_rx);
  return 1;

}

int
dp_dualport_vc_Tx(void)
{
  int writereg, esi, ebx, al;
  char *edi;

  if (io_voice_count != 0x100) {
    //    PRINTF( "io_voice_count != 0x100 (%x)\n", io_voice_count );
    return 1;
  }
  if ((0x100 > (esi = dp_read_dsp_ram(0xe1)))) {
    if (debug)
      PRINTF( "read_dsp_ram(0xe1) returned less than 0x100 (%x)\n", esi );
    return 0;
  }
  edi = io_dce_tx_sptr;
  if (0 != io_voice_count) {
    do {
      writereg = 0x80;	/* -4(%ebp) */
      do {
	al = *edi++;
	dp_regwrite(writereg++, al);
      } while (0x8f >= writereg);
      dp_regwrite(0xb0, 0x10);
      io_voice_count -= 0x10;
      ebx = x_current_time();
      while (1) {
	if (dp_regread(0xb7) & 1)
	  break;
	if (x_elapsed_time(ebx) >= 0x32) {
	  PRINTF( "Timeout while sending :-(\n" );
	  io_voice_count = 0;
	  return 0;
	}
      }
    L4:
    } while (io_voice_count != 0);
  }
 L5:
  return 1;
}

int
transmit(int fd_to_exit)
{
  int i;

  PRINTF( "Starting tad: " );
  dp_start_tad(0x80);	/* d0, b0, 90 looks nice but does not work! */ /* 0x80 seems to work */
  x_modem_state = 0xc;
  dp_state = 0x28;
  PRINTF( "ok\n" );
  fcntl(0, F_SETFL, O_NONBLOCK);

  fprintf( stderr, "+Fullduplex transmit/receive started\n" );
  while(1) {
    fd_set rfds;
    struct timeval tv;
    int retval;
    io_voice_count_rx = 0;


    /* Watch stdin (fd 0) to see when it has input. */
    FD_ZERO(&rfds);
    FD_SET(0, &rfds);
    if (fd_to_exit > 0)
      FD_SET(fd_to_exit, &rfds);

    tv.tv_sec = 0;
    tv.tv_usec = 20;

    retval = select(4, &rfds, NULL, NULL, &tv);
    /* Don't rely on the value of tv now! */

    if (FD_ISSET(0, &rfds)) {
      int res;
      res = read(0, io_dce_tx_sptr, 0x100 - io_voice_count);
      io_voice_count += res;
      while (!dp_dualport_vc_Tx()) {
	usleep(1);
      }
    }
    if (fd_to_exit > 0 && FD_ISSET(fd_to_exit, &rfds)) {
      PRINTF( "Exiting fullduplex mode\n" );
      return 0;
    }

    if (dp_dualport_vc_Rc())
      if (write(1, io_dce_rx_sptr, 0x100)!=0x100) {
	PRINTF("Could not write on stdout\n" );
      }
    dp_dsp_isr();
  }
}

void
port_io_init(int a1, int a2)
{
  char buf [200];

  ComAddress = 0x2f8;
  BaseAddress = a2;
  Irq = 3;
  V16550_IRQ_Number = 3;

  iopl (3);
  PRINTF( "Assuming communication at port %x, irq %d\n", BaseAddress, Irq );
}

void
big_init(void)
{
  x_init_modem_options ();
  PRINTF( "Initializing modem... " );
  if (!dp_init_modem ()) {
    PRINTF( "fail\n" );
    exit(1);
  }
  PRINTF( "ok\n" );
}

void
nothing(void)
{

  signal (SIGUSR1, dp_dsp_isr);
#if 0
  system ("/sbin/rmmod i2sig");
  sprintf (buf, "/sbin/insmod ./i2sig.o irq=%d pid=%d", Irq, getpid ());
  system (buf);
#endif


  PRINTF( "Going offhook: " );
  dp_offhook_cmd();
  PRINTF( "ok\n" );
#if 0
  dp_DTMF_dial_cmd(0);
  sleep(2);
#endif

  //receive();
  transmit(-1);

  PRINTF( "Going back onhook: " ); 
  dp_onhook_cmd();
  PRINTF( "ok\n" );
  exit(0);

  dp_dial_state = 0;
  x_modem_state = 5;
  x_modem_originate = 1;
  dp_threshold = H_dial_tone_detection_threshold_dB;
  strcpy (dp_dial_string, "T;31415926,,,,,,");
  while (dp_dial () == 0)
    {
      PRINTF ("busy = %d\n", (int) dp_busy_tone_detected ());
      usleep (1000);
    }
  if (!dp_sleep)
    dp_onhook_cmd ();
  PRINTF( "Deinitializing modem... " );
  dp_deinit_modem ();
  PRINTF( "ok\n" );
  PRINTF ("x_dial_residual = %d\n", x_dial_residual);
  

  // system ("/sbin/rmmod i2sig");
}

#if 0
  Local variables:
  comment-start: "/* "
  comment-end: " */"
#endif
