/*
 *	$Id: ltmodem.c,v 1.5 1999/08/22 03:35:38 root Exp root $
 *
 *	Lucent PCI modem diagnostics tool.
 *
 *	Copyright (c) 1999 Richard J.M. Close
 *  Copyright (c) 1999 Pavel Machek <pavel@ucw.cz>
 *
 *	Can be freely distributed and used under the terms of the GNU GPL.
 */

#include <fcntl.h>
#include "ltmodem.h"
#include "control.h"

#define TRY_INTERRUPTS 0

unsigned int portbase;
unsigned int modem_irq;

int verbose = 0;

// Default is tone dialling.
static int pulseDial = 0;

// Main program.

extern int x_chip_version, dp_version, rom_checksum;

void
command_mode(void)
{
  char buf[10240];
  FILE *cmd = fdopen(3, "r");

  fprintf(stderr, "PROTOCOL 0.0\n" );
  fprintf(stderr, "SWVERSION ltmodem " LTMODEM_VERSION "\n" );
  //  show_modem();
  setup_irq();
  io_init();
  big_init();
  fprintf(stderr, "HWVERSION mars V%x.%x (rom checksum %x)\n", x_chip_version, dp_version, rom_checksum );
  fprintf(stderr, "+ Command mode ready\n" );

  while (fgets(buf, 10235, cmd) != NULL) {
    //    fprintf(stderr, "# Got command %s\n", buf );

    if (!strncmp(buf, "onhook", 6)) {
      dp_onhook_cmd();
      fprintf(stderr, "+ OK\n" );
    } else if (!strncmp(buf, "offhook", 7)) {
      dp_offhook_cmd();
      fprintf(stderr, "+ OK\n" );
    } else if (!strncmp(buf, "duplex", 6)) {
      transmit(3);
    } else if (!strncmp(buf, "quit", 4)) {
      fprintf(stderr, "+ exiting\n" );
      exit(0);
    } else
      fprintf(stderr, "- unknown command %s\n", buf );
  }

  PRINTF( "# Exiting command mode\n" );
  exit(1);
}

int main(int argc, char **argv)
{
  char c;
  sync();	// it's for your own protection!
  setvbuf(stdout, NULL, _IONBF, 0);

  /* In case we are not running command mode, we want fd3 to be /dev/null */

  //  puts("ltmodem version " LTMODEM_VERSION);

  modem = 0;
  io_cnt = 0;

  pacc = pci_alloc();
  pacc->error = die;
  pci_filter_init(pacc, &filter);

  pci_init(pacc);

  scan_devices();
  sort_them();

  /* If modem is found then it sets modem_dev to device structure
     for modem, and modem =1 */
  if ( !find_modem() ) {
    printf ("Sorry, no Lucent PCI modem that I know of was found.\n");
    printf ("If you have one fitted then please mail me with the details\n");
    printf ("at: richard@close.u-net.com please include an output from\n");
    printf ("cat /proc/pci and as much info. on the card as possible.\n");
    exit(1);
  }

  while ((c = getopt(argc, argv, "movthpd:uc" )) != -1) {
    switch(c) {
    case 'c':
      command_mode();
      break;
    case 't':
      show_modem();
      setup_irq();
      io_init();
      big_init();
      break;
    case 'u':	/* Full duplex transmit/receive */
      //      show_modem();
      setup_irq();
      io_init();
      big_init();
      dp_offhook_cmd();
      transmit(-1);
      die( "Should be unreachable\n" );
      break;
    case 'd':
      show_modem();
      setup_irq();
      io_init();
      big_init();

      //	  setSpeaker(true, 3);
      printf( "About to dial %s:\n", optarg ); sleep(3);
      dp_offhook_cmd();

	  // Dial how you want.
      dial(optarg);
	  sleep(1);

#if 0	  
	  // Wait for answer tone.
	  if (waitForAnswerTone())
		printf("Answer tone detected!\n");
	  else
		printf("No answer tone detected, bummer!\n");
#endif
	  
	  // Delay before hanging up, make this long enough to hear
	  // any response from line when messing around with dialling.
	  sleep(5);

      printf( "About to hangup\n" ); sleep(5);
      dp_onhook_cmd();
      break;
    case 'o':
      show_modem();
      setup_irq();
      io_init();
      big_init();
      printf("Ready\n" );
      dp_offhook_cmd();
      dp_pulse_dial_cmd(3);	/* How do you prove that number PI does not exist? */
      dp_pulse_dial_cmd(1);	/* You take telephone and dial 3 1 4 1 5 9 2 6 5 4 */
      dp_pulse_dial_cmd(4);	/* and then you hear ta da da called number does */
      dp_pulse_dial_cmd(1);	/* not exist. (It works in Prague at least :-) */
      dp_pulse_dial_cmd(5);
      dp_pulse_dial_cmd(9);
      dp_pulse_dial_cmd(2);
      dp_pulse_dial_cmd(6);
      dp_pulse_dial_cmd(5);
      dp_pulse_dial_cmd(4);
      dp_DTMF_dial_cmd(1);
#if 0
      sleep(1);
      dp_synth_ring_init();
      sleep(1);
      dp_synth_ring_on();
      sleep(1);
      dp_synth_ring_off();
#endif
      sleep(1);

#if 1
      printf( "Staring tad\n" );
      dp_start_tad(0xf0);
      expect_irqs();
#endif

#if 0
      printf( "X_output(13)\n" );
      x_output(0x13);
      expect_irqs();
      x_output(0x1a);
      printf( "Should be ready?\n" );
#endif
      expect_irqs();
      printf( "Should dial 2\n" );
      dp_DTMF_dial_cmd(2);
      expect_irqs();
#if 0
      printf( "Really should dial 2\n" );
      dp_offhook_cmd();
      dial("2");
      expect_irqs();
#endif
#if 0
      expect_irqs();
      printf( "init hdsp\n" );
      dp_init_hdsp();
      expect_irqs();
      printf( "Starting speakerphone\n" );
      dp_start_hdsp();
      expect_irqs();
      printf( "Switching fdsp\n" );
      dp_switch_fdsp();
      expect_irqs();
      {
	int i = 0;
	for (i = 0; i<5; i++) {
	  dp_mute_fdsp(1);
	  sleep(1);
	  dp_mute_fdsp(0);
	  sleep(1);
	}
      }
      printf( "Stop fdsp:\n" );
      dp_stop_fdsp();
      sleep(3);
      printf( "About to hangup.\n" );
      dp_onhook_cmd();
      /* go_offhook(); -- seems to do something slightly different */
      /* dp_download_fax(); */
#endif
      break;
    case 'p':
      pulseDial = true;
      break;
    case 'v':
      verbose++;
      break;
    case 'h':
    default:
      printf( "Usage: ltmodem [-...]\n" );
      printf( "-o prove PI does not exist and turn modem into microwave owen.\n");
      /* -o is really meant for debugging only, and it does strange
         things. Not a toy for people who read help. It does different
         things in every release. */
	  printf("-v increase level of verbosity.\n");
      printf( "-t basic test\n");
      printf( "-c enter command mode, read commands from fd#3\n");
      printf( "-u put phone offhook, then start transmitting data from stdin\n"
	      "   and receive data from modem to stdout. You _must_ provide data on stdin\n" );
      printf( "-p enable Pulse dialling, default is Tone.\n");
      printf( "-d dial the specified number.\n");
      printf( "-h print this message!\n");
      goto exit;
    }
  }
  exit:
  pci_cleanup(pacc);

  return 0;
}

void
setup_irq(void)
{
  // Setup IRQ variable used by other stuff.
  modem_irq = modem_dev->dev->irq;
}

/* Find any known any Lucent modems. */

static int
find_modem(void)
{
  struct device *d;
  int i;

  for(d=first_dev; d; d=d->next) {

	struct pci_dev *p = d->dev;

	/* Only cards with vendor ID = Lucent!. */
	if (p->vendor_id == 0x11c1) {
	  modem_dev = d;
	  modem = 1;
	  io_cnt = 0;

	  /* Record I/O addresses. */
	  for(i=0; i<htype0_cnt; i++){
		pciaddr_t pos = p->base_addr[i];
		u32 flg = get_conf_long(d, PCI_BASE_ADDRESS_0 + 4*i);
		if (flg == 0xffffffff) flg = 0;
		if (!pos && !flg) continue;
		if (flg & PCI_BASE_ADDRESS_SPACE_IO)
		  {
			io_address[i] = pos & PCI_BASE_ADDRESS_IO_MASK;
            io_length[i] = (io_address[i] < 0x3f8) ? 8 : 256;
			io_cnt = i;
		  }
	  }
	}
  }
  return modem;
}

// Show any found modem.
static void show_modem(void)
{
  int c;
  struct pci_dev *p = modem_dev->dev;
  byte classbuf[128], devbuf[128];

  unsigned int irq = p->irq;
  byte latency = get_conf_byte( modem_dev, PCI_LATENCY_TIMER);
  word status = get_conf_word( modem_dev, PCI_STATUS);
  word cmd = get_conf_word( modem_dev, PCI_COMMAND);
  byte max_lat, min_gnt;
  word subsys_v, subsys_d;
  char ssnamebuf[256];

  printf("%02x:%02x.%x %s: %s",
	    p->bus,
	    p->dev,
	    p->func,
	    pci_lookup_name(pacc, classbuf, sizeof(classbuf),
			 PCI_LOOKUP_CLASS,
			 get_conf_word( modem_dev, PCI_CLASS_DEVICE), 0),
	    pci_lookup_name(pacc, devbuf, sizeof(devbuf),
			 PCI_LOOKUP_VENDOR | PCI_LOOKUP_DEVICE,
			 p->vendor_id, p->device_id));
 
  if (c = get_conf_byte( modem_dev, PCI_REVISION_ID))
	 printf(" (rev %02x)", c);
  if (c = get_conf_byte( modem_dev, PCI_CLASS_PROG))
     printf(" (prog-if %02x)", c);
  putchar('\n');

  // Only cards with device ID 0x44* are mars chipset based.
  if ((p->device_id & 0xfff0) == 0x440)
	 printf("Mars chipset detected.\n");
  else
	 printf("Warning: your modem does not have a mars chipset.\n");

  max_lat = get_conf_byte( modem_dev, PCI_MAX_LAT);
  min_gnt = get_conf_byte( modem_dev, PCI_MIN_GNT);
  subsys_v = get_conf_word( modem_dev, PCI_SUBSYSTEM_VENDOR_ID);
  subsys_d = get_conf_word( modem_dev, PCI_SUBSYSTEM_ID);
  if (subsys_v && subsys_v != 0xffff)
     printf("\tSubsystem: %s\n",
           pci_lookup_name(pacc, ssnamebuf, sizeof(ssnamebuf),
		   PCI_LOOKUP_SUBSYSTEM | PCI_LOOKUP_VENDOR | PCI_LOOKUP_DEVICE,
		   subsys_v, subsys_d));

  printf("\tFlags: ");
  if (cmd & PCI_COMMAND_MASTER)
     printf("bus master, ");
  if (cmd & PCI_COMMAND_VGA_PALETTE)
	 printf("VGA palette snoop, ");
  if (cmd & PCI_COMMAND_WAIT)
   	 printf("stepping, ");
  if (cmd & PCI_COMMAND_FAST_BACK)
	 printf("fast Back2Back, ");
  if (status & PCI_STATUS_66MHZ)
	 printf("66Mhz, ");
  if (status & PCI_STATUS_UDF)
	 printf("user-definable features, ");
     printf("%s devsel",
	        ((status & PCI_STATUS_DEVSEL_MASK) == PCI_STATUS_DEVSEL_SLOW) ? "slow" :
	        ((status & PCI_STATUS_DEVSEL_MASK) == PCI_STATUS_DEVSEL_MEDIUM) ? "medium" :
	        ((status & PCI_STATUS_DEVSEL_MASK) == PCI_STATUS_DEVSEL_FAST) ? "fast" : "??");
  if (cmd & PCI_COMMAND_MASTER)
	 printf(", latency %d", latency);
  if (irq)
	 printf(", IRQ " IRQ_FORMAT, irq);
  putchar('\n');
  show_htype0( modem_dev);

}

/********************************************************************************
*
* PCI utilities.
*
*********************************************************************************/

static struct device *
scan_device(struct pci_dev *p)
{
  int how_much = 0x40; /* First 64 bytes. */
  struct device *d;

  if (!pci_filter_match(&filter, p))
    return NULL;
  d = xmalloc(sizeof(struct device));
  bzero(d, sizeof(*d));
  d->dev = p;
  if (!pci_read_block(p, 0, d->config, how_much))
    die("Unable to read %d bytes of configuration space.", how_much);
  if (how_much < 128 && (d->config[PCI_HEADER_TYPE] & 0x7f) == PCI_HEADER_TYPE_CARDBUS)
    {
      /* For cardbus bridges, we need to fetch 64 bytes more to get the full standard header... */
      if (!pci_read_block(p, 0, d->config+64, 64))
	die("Unable to read cardbus bridge extension data.");
      how_much = 128;
    }
  d->config_cnt = how_much;
  pci_setup_cache(p, d->config, d->config_cnt);
  pci_fill_info(p, PCI_FILL_IDENT | PCI_FILL_IRQ | PCI_FILL_BASES | PCI_FILL_ROM_BASE);
  return d;
}

static void
scan_devices(void)
{
  struct device *d;
  struct pci_dev *p;

  pci_scan_bus(pacc);
  for(p=pacc->devices; p; p=p->next)
    if (d = scan_device(p))
      {
	d->next = first_dev;
	first_dev = d;
      }
}

static int
check_root(void)
{
  static int is_root = -1;

  if (is_root < 0)
    is_root = !geteuid();
  return is_root;
}

static int
config_fetch(struct device *d, unsigned int pos, unsigned int len)
{
  if (pos + len < d->config_cnt)
    return 1;
  if (pacc->method != PCI_ACCESS_DUMP && !check_root())
    return 0;
  return pci_read_block(d->dev, pos, d->config + pos, len);
}

/* Config space accesses */

static inline byte
get_conf_byte(struct device *d, unsigned int pos)
{
  return d->config[pos];
}

static word
get_conf_word(struct device *d, unsigned int pos)
{
  return d->config[pos] | (d->config[pos+1] << 8);
}

static u32
get_conf_long(struct device *d, unsigned int pos)
{
  return d->config[pos] |
    (d->config[pos+1] << 8) |
    (d->config[pos+2] << 16) |
    (d->config[pos+3] << 24);
}

/* Sorting */

static int
compare_them(const void *A, const void *B)
{
  const struct pci_dev *a = (*(const struct device **)A)->dev;
  const struct pci_dev *b = (*(const struct device **)B)->dev;

  if (a->bus < b->bus)
    return -1;
  if (a->bus > b->bus)
    return 1;
  if (a->dev < b->dev)
    return -1;
  if (a->dev > b->dev)
    return 1;
  if (a->func < b->func)
    return -1;
  if (a->func > b->func)
    return 1;
  return 0;
}

static void
sort_them(void)
{
  struct device **index, **h, **last_dev;
  int cnt;
  struct device *d;

  cnt = 0;
  for(d=first_dev; d; d=d->next)
    cnt++;
  h = index = alloca(sizeof(struct device *) * cnt);
  for(d=first_dev; d; d=d->next)
    *h++ = d;
  qsort(index, cnt, sizeof(struct device *), compare_them);
  last_dev = &first_dev;
  h = index;
  while (cnt--)
    {
      *last_dev = *h;
      last_dev = &(*h)->next;
      h++;
    }
  *last_dev = NULL;
}

/* Normal output */

static void
show_bases(struct device *d, int cnt)
{
  struct pci_dev *p = d->dev;
  word cmd = get_conf_word(d, PCI_COMMAND);
  int i;

  for(i=0; i<cnt; i++)
    {
      pciaddr_t pos = p->base_addr[i];
      u32 flg = get_conf_long(d, PCI_BASE_ADDRESS_0 + 4*i);
      if (flg == 0xffffffff)
	flg = 0;
      if (!pos && !flg) continue;
      printf("\tRegion %d: ", i);
      if (pos && !flg)			/* Reported by the OS, but not by the device */
	{
	  printf("[virtual] ");
	  flg = pos;
	}
      if (flg & PCI_BASE_ADDRESS_SPACE_IO)
	{
	  pciaddr_t a = pos & PCI_BASE_ADDRESS_IO_MASK;
	  printf("I/O ports at ");
	  if (a) {
	    portbase = (unsigned int) a;
	    printf(IO_FORMAT, a); 
	  }
	  else if (flg & PCI_BASE_ADDRESS_IO_MASK)
	    printf("<ignored>");
	  else
	    printf("<unassigned>");
	  if (!(cmd & PCI_COMMAND_IO))
	    printf(" [disabled]");
	}
      else
	{
	  int t = flg & PCI_BASE_ADDRESS_MEM_TYPE_MASK;
	  pciaddr_t a = pos & PCI_ADDR_MEM_MASK;
	  int done = 0;
	  u32 z = 0;

	  printf("Memory at ");
	  if (t == PCI_BASE_ADDRESS_MEM_TYPE_64)
	    {
	      if (i >= cnt - 1)
		{
		  printf("<invalid-64bit-slot>\n");
		  done = 1;
		}
	      else
		{
		  i++;
		  z = get_conf_long(d, PCI_BASE_ADDRESS_0 + 4*i);
		  if (buscentric_view)
		    {
		      if (a || z)
			printf("%08x" ADDR_FORMAT, z, a);
		      else
			printf("<unassigned>");
		      done = 1;
		    }
		}
	    }
	  if (!done)
	    {
	      if (a)
		printf(ADDR_FORMAT, a);
	      else
		printf(((flg & PCI_BASE_ADDRESS_MEM_MASK) || z) ? "<ignored>" : "<unassigned>");
	    }
	  printf(" (%s, %sprefetchable)",
		 (t == PCI_BASE_ADDRESS_MEM_TYPE_32) ? "32-bit" :
		 (t == PCI_BASE_ADDRESS_MEM_TYPE_64) ? "64-bit" :
		 (t == PCI_BASE_ADDRESS_MEM_TYPE_1M) ? "low-1M" : "type 3",
		 (flg & PCI_BASE_ADDRESS_MEM_PREFETCH) ? "" : "non-");
	  if (!(cmd & PCI_COMMAND_MEMORY))
	    printf(" [disabled]");
	}
      putchar('\n');
    }
}

static void
show_htype0(struct device *d)
{
  unsigned long rom = d->dev->rom_base_addr;

  show_bases(d, htype0_cnt);
  if (rom & 1)
    printf("\tExpansion ROM at %08lx%s\n", rom & PCI_ROM_ADDRESS_MASK,
	   (rom & PCI_ROM_ADDRESS_ENABLE) ? "" : " [disabled]");
  if (get_conf_word(d, PCI_STATUS) & PCI_STATUS_CAP_LIST)
    {
      int where = get_conf_byte(d, PCI_CAPABILITY_LIST);
      while (where)
	{
	  int id, next, cap;
	  printf("\tCapabilities: ");
	  if (!config_fetch(d, where, 4))
	    {
	      puts("<available only to root>");
	      break;
	    }
	  id = get_conf_byte(d, where + PCI_CAP_LIST_ID);
	  next = get_conf_byte(d, where + PCI_CAP_LIST_NEXT);
	  cap = get_conf_word(d, where + PCI_CAP_FLAGS);
	  printf("[%02x] ", where);
	  if (id == 0xff)
	    {
	      printf("<chain broken>\n");
	      break;
	    }
	  switch (id)
	    {
	    case PCI_CAP_ID_PM:
	       printf("Power Management version %d\n", cap & PCI_PM_CAP_VER_MASK);
	      break;
	    case PCI_CAP_ID_AGP:
	      break;
	    default:
	      printf("#%02x [%04x]", id, cap);
	    }
	  where = next;
	}
    }
}

static struct bus *
find_bus(struct bridge *b, unsigned int n)
{
  struct bus *bus;

  for(bus=b->first_bus; bus; bus=bus->sibling)
    if (bus->number == n)
      break;
  return bus;
}

static struct bus *
new_bus(struct bridge *b, unsigned int n)
{
  struct bus *bus = xmalloc(sizeof(struct bus));

  bus = xmalloc(sizeof(struct bus));
  bus->number = n;
  bus->sibling = b->first_bus;
  bus->first_dev = NULL;
  bus->last_dev = &bus->first_dev;
  b->first_bus = bus;
  return bus;
}

static void
insert_dev(struct device *d, struct bridge *b)
{
  struct pci_dev *p = d->dev;
  struct bus *bus;

  if (! (bus = find_bus(b, p->bus)))
    {
      struct bridge *c;
      for(c=b->child; c; c=c->next)
	if (c->secondary <= p->bus && p->bus <= c->subordinate)
	  return insert_dev(d, c);
      bus = new_bus(b, p->bus);
    }
  /* Simple insertion at the end _does_ guarantee the correct order as the
   * original device list was sorted by (bus, devfn) lexicographically
   * and all devices on the new list have the same bus number.
   */
  *bus->last_dev = d;
  bus->last_dev = &d->next;
  d->next = NULL;
}

/* Bus mapping mode */

static void
do_map_bridges(int bus, int min, int max)
{
  struct bus_info *bi = bus_info + bus;
  struct bus_bridge *b;

  bi->guestbook = 1;
  for(b=bi->bridges; b; b=b->next)
    {
      if (bus_info[b->first].guestbook)
	b->bug = 1;
      else if (b->first < min || b->last > max)
	b->bug = 2;
      else
	{
	  bus_info[b->first].via = b;
	  do_map_bridges(b->first, b->first, b->last);
	}
    }
}


/*  Show hexadecimal dump of first 64 bytes of the PCI configuration space
    (the standard header). Useful for debugging of drivers and lspci itself. */
static void
show_hex_dump(struct device *d)
{
  unsigned int i;

  printf("Command reg: "); 
  printf(" %02x", get_conf_byte(d, 5));
  printf(" %02x", get_conf_byte(d, 4));
  putchar('\t');

  printf("Status reg: "); 
  printf(" %02x", get_conf_byte(d, 7));
  printf(" %02x", get_conf_byte(d, 6));
  putchar('\n');
  
  /* Note! the byte order is lsb msb */  
  printf("Base address regs: "); 
  for(i=0x10; i<0x24; i++){
	printf(" %02x", get_conf_byte(d, i));
  }
  putchar('\n');

}

// Lucent modem specific stuff.  

void go_onhook(void)
{
  dp_onhook_cmd();
}

void io_init(void)
{

   // Set up the port IO.
   port_io_init(io_address[1], io_address[2]);

   // Set up EEPROM ?
   //SetUpEEPROM();

   // Set up the modem variables etc.
   //MimicInit();

}

// Dial the number held in the argument string.
void dial(char *s)
{
  sleep(2);	/* We do no dial tone detection, so we'll just wait */
  while (*s) {

	//printf( "Dialing: `%c'.\n", *s );

    switch (*s) {
    case '0': case '1': case '2': case '3': case '4': case '5': case '6': case '7': case '8': case '9':
	  // Delay before each one.
	  usleep(400000);
      if (pulseDial) {
		usleep(50000 * (*s));
		dp_pulse_dial_cmd(*s-'0');
	  }
	  else 
		dp_DTMF_dial_cmd(*s-'0');
	  break;

    default:
      printf( "Unknown character in dial sequence `%c' ignored.\n", *s );
      break;
    }
    s++;
  }
}

#if 0
// Mess around with speaker.
void setSpeaker (bool Enabled, unsigned char Volume)
{

  // Set speaker globals.
  speakerEnable = (unsigned char)Enabled;
  speakerVolume = Volume; 

  dp_56k_speaker_cmd ();

}

// Wait for answer tone, timing out after 5 seconds.
bool waitForAnswerTone (void)
{

  int timeout = 5000;

  dp_answer_state = 0;

  dp_timer = x_current_time();

  // Set up tone detection.
  dp_detect_tones_cmd();

  // Wait until answer tone detected or 5 seconds has expired.
  while (!dp_answer_tone_detected()) {
	if (x_elapsed_time(dp_timer) > timeout)
	  return false;
  }
  return true;
}
#endif


