/*
 *	$Id: ltmodem.h,v 1.6 1999/08/22 03:36:13 root Exp root $
 *
 *	Lucent PCI modem diagnostics tool.
 *
 *	Copyright (c) 1999 Richard J.M. Close
 *
 *	Can be freely distributed and used under the terms of the GNU GPL.
 */

#include <stdio.h>
#include <string.h>
#include <stdlib.h>
#include <stdarg.h>
#include <unistd.h>
#include <ctype.h>

#include "portIO.h"
#include "irq.h"
#include "lib/header.h"
#include "lib/pci.h"
#include "stdbool.h"
#include "pciutils.h"

/* Format strings used for IRQ numbers and memory addresses */

#ifdef ARCH_SPARC64
#define IRQ_FORMAT "%08x"
#else
#define IRQ_FORMAT "%d"
#endif

#ifdef HAVE_64BIT_ADDRESS
#define ADDR_FORMAT "%016Lx"
#else
#define ADDR_FORMAT "%08lx"
#endif

#ifdef ARCH_SPARC64
#define IO_FORMAT "%016Lx"
#elif defined(HAVE_64BIT_ADDRESS)
#define IO_FORMAT "%04Lx"
#else
#define IO_FORMAT "%04lx"
#endif

#define LTMODEM_VERSION "0.0.7"

#define PRINTF(a...) fprintf(stderr, "# " a)

struct device {
  struct device *next;
  struct pci_dev *dev;
  unsigned int config_cnt;
  byte config[256];
};

/* Communication with libpci */

static struct pci_access *pacc;

/* Our view of the PCI bus */

static struct device *first_dev;
static struct device *modem_dev;

/* Tree output */

struct bridge {
  struct bridge *chain;			/* Single-linked list of bridges */
  struct bridge *next, *child;		/* Tree of bridges */
  struct bus *first_bus;		/* List of busses connected to this bridge */
  unsigned int primary, secondary, subordinate;	/* Bus numbers */
  struct device *br_dev;
};

struct bus {
  unsigned int number;
  struct bus *sibling;
  struct device *first_dev, **last_dev;
};

/* Options */
static int modem;			/* Boolean: modem has been found = 1 */
/*static int verbose;		Show detailed information */
static int buscentric_view;		/* Show bus addresses/IRQ's instead of CPU-visible ones */
static struct pci_filter filter;	/* Device filter */

/* general scanning stuff. */
static struct device *
scan_device(struct pci_dev *p);
static void
scan_devices(void);

static int
check_root(void);

static int
config_fetch(struct device *d, unsigned int pos, unsigned int len);


/* Config space accesses */
static inline byte
get_conf_byte(struct device *d, unsigned int pos);

static word
get_conf_word(struct device *d, unsigned int pos);

static u32
get_conf_long(struct device *d, unsigned int pos);

/* Count used to access bases of htype0 devices. */
static int htype0_cnt = 6;
/* Array to hold IO addresses and their width (in bytes). */
unsigned int io_address[6];
unsigned int io_length[6];
/* The number of I/O ports found, = length of io_address. */
static unsigned int io_cnt;

/* Sorting */
static int
compare_them(const void *A, const void *B);

static void
sort_them(void);

/* Normal output */
#define FLAG(x,y) ((x & y) ? '+' : '-')

static void
show_bases(struct device *d, int cnt);

static void
show_htype0(struct device *d);

static struct bus *
find_bus(struct bridge *b, unsigned int n);

static struct bus *
new_bus(struct bridge *b, unsigned int n);

static void
insert_dev(struct device *d, struct bridge *b);

/* Bus mapping mode */

struct bus_bridge {
  struct bus_bridge *next;
  byte this, dev, func, first, last, bug;
};

struct bus_info {
  byte exists;
  byte guestbook;
  struct bus_bridge *bridges, *via;
};

static struct bus_info *bus_info;

static void
do_map_bridges(int bus, int min, int max);

/*  Show hexadecimal dump of first 64 bytes of the PCI configuration space
    (the standard header). */
static void
show_hex_dump(struct device *d);

/* Lucent modem specific stuff. */
static int
find_modem(void);

static void
show_modem(void);

/* Modem monitoring functions. */
static void
monitor_modem(void);

static int
set_monitoring_interval(void);

static void
continous_monitoring(int monitor_type, int monitoring_interval);

static void
redisplay_modem_regs(void);

/* Modem writing functions. */
static void
set_modem(void);
/* Interactively set the control register. */
static void
set_modem_cont_reg(void);
/* Interactively write to the IO ports. */
static void
set_modem_io(void);

/* Main command menu. */
void main_menu(void);

void setup_irq(void);

void io_init(void);

// Dial the number held in the argument string.
void dial(char *s);

#if 0
// Mess around with the speaker.
void setSpeaker (bool Enabled, unsigned char Volume);

bool waitForAnswerTone (void);
#endif
